import 'dart:async';
import 'dart:io';
import 'dart:ui';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:progress_dialog/progress_dialog.dart';
import 'package:rapidfire/Config.dart';
import 'package:rapidfire/Db.dart';
import 'package:rapidfire/GlobalRF.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/SystemRF.dart';
import 'package:rapidfire/uiux/DialogRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'dart:convert';
import 'BGHelperDB.dart';
import 'BGSyncSMain.dart';
import 'Dynamic.dart';
import 'GlobalScopeRF.dart';
import 'models/Login.dart';
import 'Annotation.dart' as annotation;
import 'dart:io';

abstract class ISyncRF {
  void complete();
}

class Api {
  static Function onSyncCompleted;
  static Function onFetchCompleted;
  static Function onFetchFailed;

  var db = Db();
  ProgressDialog pdlg;

  var _loginRequest = {
    "ModelName": "",
    "Password": "",
    "Username": "",
  };

  login(BuildContext context, String userName, String password) async {
    _loginRequest["DeviceUniqueId"] = "945dba59869a781b";
    _loginRequest["Username"] = userName;
    _loginRequest["Password"] = password;
    return await _singleFetch(context, Login(),
        jsonPacket: _loginRequest, dialogMsg: "Logging In . . .");
  }

  var _syncRequest = {
    "UserId": 0,
    "TableName": "",
    "Command": "add",
    "TotalRecord": 0,
    "PageNo": 0,
    "PageSize": 0,
    "ApiPacket": {
      "Packet": null,
      "PacketList": null,
    }
  };

  syncModels(String jobName, BuildContext context, Function dispatcher, [List<Type> models]) async {
    bool isNetworkAvilable = await SystemRF.network.hasInternet();
    if (isNetworkAvilable) {
      if (dispatcher != null && !Platform.isIOS) {
        var inputData = Map<String, dynamic>();
        if (models != null) {
          for (var item in models) {
            inputData[item.toString()] = item.toString();
          }
        }
        _syncWithService(
            jobName, context, dispatcher, models == null ? null : inputData); // pass table
      } else {
        _syncWithoutService(models, context); // pass table
      }
    } else {
      rf.msg(context, 'No Internet Connection');
    }
  }

  _syncWithService(
      String jobName, BuildContext context, Function dispatcher, Map<String, dynamic> inputData,
      {String dialogMsg = "Uploading..."}) async {
    pdlg = ProgressDialog(
      context,
      type: ProgressDialogType.Normal,
      isDismissible: true,
    )..style(message: dialogMsg);
    await resetServiceSync(jobName);
    await pdlg.show();
    startService(jobName, dispatcher, inputData);
    onSyncComplete(jobName);
  }

  runSyncService(
      {Function reflectionAgent, IConfigRF config, Map<String, dynamic> inputData}) async {
    reflectionAgent.call();
    Configuration _configuration = new Configuration();
    config.configure(_configuration);
    Models modelList = new Models();
    _configuration.model.appModels.register(modelList);

    if (inputData != null) {
      for (var model in inputData.entries) {
        var modelRF = Models.models[model.key];
        var jsonData = await Dynamic(modelRF.model).syncableDataJsonMap();
        for (var jsonRow in jsonData) {
          await _sync(null, modelRF.model, jsonRow, showLoader: false, config: _configuration);
        }
      }
    } else {
      /// Sync ALL Data here
      for (var entry in modelList.get().entries) {
        var modelRF = entry.value as ModelRF;
        if (modelRF.syncable) {
          var jsonData = await Dynamic(modelRF.model).syncableDataJsonMap();
          for (var jsonRow in jsonData) {
            await _sync(null, modelRF.model, jsonRow, showLoader: false, config: _configuration);
          }
        }
      }
    }
  }

  onSyncComplete(String serviceName) async {
    Timer.periodic(const Duration(seconds: 1), (timer) async {
      var value = await watchServiceCompleted(serviceName);
      print("timer");
      if (value.first['CompleteStatus'] == 1) {
        if (onSyncCompleted != null) onSyncCompleted.call();
        await pdlg.hide();
        timer.cancel();
      }
    });
  }

  _syncWithoutService(List<Type> models, BuildContext context) async {
    print('Starting Sync Without Service.');
    //BuildContext context = Libs.context;

    pdlg = ProgressDialog(context, type: ProgressDialogType.Download, isDismissible: true);
    if (Libs.appModels.get().length > 0) await pdlg.show();

    // if selected model
    if (models != null) {
      for (var model in models) {
        var modelRF = Models.models[model.toString()];
        var jsonData = await Dynamic(modelRF.model).syncableDataJsonMap();
        for (var jsonRow in jsonData) {
          await _sync(context, modelRF.model, jsonRow, showLoader: false);
        }
      }
    } else {
      // else all models
      for (var entry in Libs.appModels.get().entries) {
        var modelRF = entry.value as ModelRF;
        var jsonData = await Dynamic(modelRF.model).syncableDataJsonMap(); //multy row ??????
        for (var jsonRow in jsonData) {
          await _sync(context, modelRF.model, jsonRow, showLoader: false);
        }
      }
    }

    //Libs.appModels.get().forEach((key, value) {});
    if (Libs.appModels.get().length > 0) await pdlg.hide();
    print('Completed Sync Without Service.');
  }

  double percentage = 0;

  _sync(
    BuildContext context,
    Object model,
    dynamic jsonData, {
    Function onSuccess,
    bool showLoader = true,
    String command = 'add',
    Configuration config,
  }) async {
    var itemCount = 0;

    bool isInternet = await SystemRF.network.hasInternet();
    if (!isInternet) {
      print('Please turn on your internet connection');
      return;
    }

    if (showLoader)
      pdlg = ProgressDialog(context, type: ProgressDialogType.Download, isDismissible: true);

    itemCount++;
    var tableName = model.runtimeType.toString();
    var packet = jsonData; //Dynamic(model).syncableJsonMap();
    var pk = db.getPrimaryKeyName(model);
    var sid = pk == null ? "" : packet[pk];

    //if (command.contains('add')) if (pk != null) packet[pk] = 0;
    _syncRequest['TableName'] = tableName;
    _syncRequest['Command'] = command;
    _syncRequest['ApiPacket'] = {"Packet": packet, "PacketList": null};

    var content = json.encode(_syncRequest);

    if (showLoader) await pdlg.show();
    if (showLoader)
      pdlg.update(message: 'Please wait, uploading data... (${tableName})', progress: percentage);

    var baseUrl = Libs?.config ?? config;
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Sync/SendPacket';

    print('Request : ');
    printWrapped(content);
    var response = await http.post(Uri.parse(apiBaseURL), body: content, headers: {
      HttpHeaders.contentTypeHeader: "application/json",
    });
    return await _onSyncResponse(response, sid, onSuccess, showLoader);

    if (showLoader) await pdlg.hide();
  }

  void printWrapped(String text) {
    final pattern = RegExp('.{1,800}'); // 800 is the size of each chunk
    pattern.allMatches(text).forEach((match) => print(match.group(0)));
  }

  _onSyncResponse(http.Response response, dynamic sid, Function onSuccess,
      [bool showLoader = true]) async {
    if (response.statusCode == 200) {
      var json = jsonDecode(response.body);
      print('Response : $json');

      var modelRF = Libs.appModels.get()[json['ModelName']] as ModelRF;

      var pk = Dynamic(modelRF.model).annotationData(annotation.Key);

      var uSql =
          'update ${json['ModelName']} set ${modelRF.syncStatusField} = 1 where ${pk.first['field']} = \'${json['RecordId']}\'';
      await db.execute(uSql);

      if (showLoader) await pdlg.hide(); // for add
      // SystemRF.hideKeyboard(context); //??
      if (onSuccess != null) onSuccess(json['Message'] + sid.toString(), 'success', json);
//        double per = ((itemCount / (modelList as List).length) * 100).roundToDouble();
//        percentage = per == 100.0 ? 0 : per;
      return json;
      print('Sync was successful!');
    } else {
      if (response.body.isNotEmpty) {
        print('Error : ${response.statusCode}');
        var json = jsonDecode(response.body);
        if (onSuccess != null) onSuccess(json['Message'] + sid.toString(), 'failed');
      } else {
        if (onSuccess != null) onSuccess('500: Internal Server Error', 'success');
      }
      return json;
    }
    if (showLoader) await pdlg.hide();
  }

  /*FETCH*/

  var fetchRequest = {
    "UserId": 0,
    "TableName": "Student",
    "Command": "GetData",
    "TotalRecord": 0,
    "PageNo": 1,
    "PageSize": 50,
    "UserGuid": "0",
    "ApiPacket": {},
  };
  var pageSize = 50;
  var currentPage = 1;
  var pages = '';
  var totalRecord = 0;
  List<int> completedPages = List();
  var isOldDelete = '';
  var showNotify = '';

  int currentModel = 1;

  fetchInitials(BuildContext context, String serviceName, Function dispatcher, List models,
      [Function onFetchComplete]) async {
    var modelsName = '';
    for (var item in models) {
      modelsName += '$item,';
    }

    await Api().fetchModel(
      context: context,
      serviceName: serviceName,
      dispatcher: dispatcher,
      modelsName: modelsName,
      showDialog: false,
      closeDialog: false,
    );
    //await rf.api.fetchModel(context, serviceName, dispatcher, model[currentModel], false, false);
    //_onFetchCompletedObserver(context, serviceName, dispatcher, model);
  }

  fetchAll(BuildContext context, String serviceName, Function dispatcher, List models,
      [Function onFetchComplete]) async {
    // add also without service
    var modelsName = '';
    if (models == null) {
      models = List();
      for (var item in Libs.appModels
          .get()
          .entries
          .where((element) => (element.value as ModelRF).syncable == true)) {
        modelsName += '${item.key},';
      }
    }
    await fetchModel(
      context: context,
      serviceName: serviceName,
      dispatcher: dispatcher,
      modelsName: modelsName,
      showDialog: true,
      closeDialog: true,
    );
  }

  fetchModel({
    BuildContext context,
    String serviceName,
    Function dispatcher,
    String modelsName,
    bool showDialog = true,
    bool closeDialog = true,
    bool showNotification = true,
    bool allowRetry = false,
    Function onFetchComplete,
  }) async {
    bool isInternet = await SystemRF.network.hasInternet();
    if (!isInternet) {
      rf.msg(context, 'Please turn on your internet connection');
      return;
    }

    currentPage = 1;
    pages = '';
    totalRecord = 0;
    completedPages = List();
    isOldDelete = '';
    showNotify = showNotification ? '1' : '';
    pageSize = Libs.config.data.pageSize;
    fetchRequest['PageSize'] = pageSize;
    if (modelsName == null) {
      Models modelList = new Models();
      for (var entry in modelList.get().entries) {
        var modelRF = entry.value as ModelRF;
        if (modelRF.syncable) {
          if (modelsName == null)
            modelsName = modelRF.model.runtimeType.toString();
          else
            modelsName = modelsName.toString() + "," + modelRF.model.runtimeType.toString();
        }
      }
    }

    _onAccept() async {
      Map<String, dynamic> inputData = Map();
      inputData['Model'] = modelsName;

      pdlg = ProgressDialog(context, type: ProgressDialogType.Download, isDismissible: true);
      //_fetchWithoutService(context, modelsName, onFetchComplete);
      if (dispatcher != null && !Platform.isIOS) {
        await _fetchWithService(
            serviceName, context, dispatcher, inputData, modelsName, showDialog);
      } else {
        _fetchWithoutService(serviceName, context, modelsName, onFetchComplete, showDialog);
      }
    }

    rf.pref.setBool('AdminNotify', showNotification);

    if (showDialog) {
      DialogRF(
        context,
        height: 100,
        title: 'Download Data',
        subTitle: 'Download old data from server.',
        avatarIcon: Icons.cloud_download,
        type: DialogType.popup,
        body: Padding(
          padding: EdgeInsets.only(top: 20),
          child: CheckRF(
            height: 60,
            data: [
              RadioCheckData(
                label: 'Do you want to delete local data before downloading server data?',
                value: 0,
              )
            ],
            onChange: (val) => isOldDelete = val,
          ),
        ),
        onAccept: () async {
          await _onAccept();
        },
      );
    } else {
      isOldDelete = "1";
      await _onAccept();
    }
  }

  _fetchWithoutService(
      String serviceName, BuildContext context, dynamic modelName, Function FetchComplete,
      [bool showDialog = true, bool closeDialog = true]) async {
    if (isOldDelete.isNotEmpty) {
      print('Deleting old data');
      for (var item in modelName.split(',')) {
        if (item.isNotEmpty) {
          await db.execute('delete from ${item}');
        }
      }
      //await db.execute('delete from ${model}');
    }

    await resetServiceFetch(serviceName);
    if (showDialog) await pdlg.show();

    var splitModels = modelName.split(',');

    notify('Download Status:', 'Starting data download!');

    onFetchComplete(serviceName, closeDialog: closeDialog, modelName: modelName);
    for (int i = 0; i < splitModels.length; i++) {
      if (splitModels[i].isNotEmpty) {
        var modelRF = Libs.appModels.get()[splitModels[i]] as ModelRF;
        await _fetch(null, modelRF.model, serviceName: serviceName, showLoader: false, pageNo: 1);
      }
    }
    if (showDialog) await pdlg.hide();

    notify('Download Status:', 'Data download has been successfully completed!');

    //await _fetch(context, model, onFetchComplete: onFetchComplete); // non service
  }

  _fetchWithService(String serviceName, BuildContext context, Function dispatcher,
      Map<String, dynamic> inputData, String modelName,
      [bool showDialog = true, bool closeDialog = true, bool showNotification = true]) async {
    pdlg = ProgressDialog(context, type: ProgressDialogType.Normal, isDismissible: true)
      ..style(message: "Downloading...");

    await resetServiceFetch(serviceName);
    if (showDialog) await pdlg.show();

    inputData['DeleteOldData'] = isOldDelete;
    inputData['showNotification'] = showNotify;
    await startService(serviceName, dispatcher, inputData);
    onFetchComplete(serviceName, closeDialog: closeDialog, modelName: modelName);
  }

  runFetchService(
      {String serviceName,
      Function reflectionAgent,
      IConfigRF config,
      Map<String, dynamic> inputData}) async {
    reflectionAgent.call();
    Configuration _configuration = new Configuration();
    config.configure(_configuration);
    Models modelList = new Models();
    _configuration.model.appModels.register(modelList);

    if (inputData['DeleteOldData'].isNotEmpty) {
      //same for non service
      print('Deleting old data');
      for (var item in inputData['Model'].split(',')) {
        if (item.isNotEmpty) {
          await db.execute('delete from ${item}');
        }
      }
      // String model = inputData['Model'];
      // var splitModels = model.split(',');
      // for (int i = 0; i < splitModels.length; i++) {
      //   if (splitModels[i].isNotEmpty) {
      //     await db.execute('delete from ${splitModels[i]}');
      //   }
      // }
    }

    if (inputData != null) {
      String model = inputData['Model'];
      var splitModels = model.split(',');

      if (inputData['showNotification'].isNotEmpty)
        notify('Download Status:', 'Starting data download!');

      for (int i = 0; i < splitModels.length; i++) {
        if (splitModels[i].isNotEmpty) {
          var modelRF = Libs.appModels.get()[splitModels[i]] as ModelRF;

          ///???????
          await _fetch(null, modelRF.model,
              serviceName: serviceName,
              onFetchComplete: onFetchCompleted,
              showLoader: false,
              config: _configuration,
              pageNo: 1);
        }
      }
      if (inputData['showNotification'].isNotEmpty)
        notify('Download Status:', 'Data download has been successfully completed!');
    }
  }

  onFetchComplete(String serviceName, {bool closeDialog = true, String modelName}) async {
    int i = 0;
    Timer.periodic(const Duration(seconds: 1), (timer) async {
      i++;
      var value = await watchServiceCompleted(serviceName);
      var fetchValue = await watchFetchCompleted(serviceName);

      if (value.first['CompleteStatus'] == 1) {
        timer.cancel();
        if (onFetchCompleted != null) {
          onFetchCompleted('all fetch completed', 0);
        }
        if (closeDialog) {
          await pdlg.hide();
        }
      } else {
        //print("Timer counter:" + i.toString());
        if (fetchValue != null && fetchValue.length != 0) {
          if (fetchValue.first["Success"] == 1) {
            if (onFetchCompleted != null)
              onFetchCompleted(fetchValue.first["TableName"], fetchValue.first["TotalRecord"]);
          } else if (fetchValue.first["Success"] == 4) {
            if (onFetchFailed != null) onFetchFailed(fetchValue.first["TableName"]);
          }
        }
        if (i > 500) {
          timer.cancel();
          if (closeDialog) {
            await pdlg.hide();
          }
        }
      }
    });
  }

  Future<http.Response> httpPost({
    String url,
    String requestBody,
    String headers = "application/json",
  }) async {
    print('Request body:' + requestBody);
    var response = await http.post(Uri.parse(url), body: requestBody, headers: {
      HttpHeaders.contentTypeHeader: headers,
    });
    print('Response body:' + response.body);
    return response;
  }

  Future<List<T>> get<T>({String url, dynamic jsonNode}) async {
    return _post<T>();

    /// it should be generic get ???????????
  }

  Future<ResponseRF> register(context, userInfo) async {
    ProgressDialog pdlg =
        ProgressDialog(context, type: ProgressDialogType.Normal, isDismissible: true)
          ..style(message: "Registering....\nPlease Wait");
    await pdlg.show();
    var baseUrl = Libs?.config?.http?.apiBaseURL;
    var apiBaseURL = '$baseUrl/api/CustomApi/UserRegistration';
    var syncRequest = {
      "ApiPacket": {},
      "Command": "add",
      "PageNo": 0,
      "PageSize": 0,
      "TableName": "UserInfo",
      "TotalRecord": 0,
      "UserId": 0
    };
    userInfo.UserId = 0;
    var masterJson = Dynamic(userInfo).modelToDataMap();
    syncRequest['ApiPacket'] = {"Packet": masterJson, "PacketList": null};
    var response = await Api().httpPost(url: apiBaseURL, requestBody: json.encode(syncRequest));
    print(response.body);

    var _response = jsonDecode(response.body);
    print("Signup Json Data $_response");
    await pdlg.hide();
    return ResponseRF.parse(_response);
  }

  Future<ResponseRF> save<T>({Object model}) async {
    var baseUrl = Libs?.config?.http?.apiBaseURL;
    var apiBaseURL = '$baseUrl/api/Sync/SendPacket';

    dynamic fr = this.fetchRequest;
    fr["TableName"] = model.runtimeType.toString();
    fr["Command"] = 'Add';
    //fr["UserGuid"] = userGuid;  //UserId
    fr['ApiPacket']['Packet'] = Dynamic(model).modelToDataMap();
    var response = await rf.api.httpPost(url: apiBaseURL, requestBody: json.encode(fr));

    var _response = jsonDecode(response.body); //[Message] ??

    return ResponseRF.parse(_response);
  }

  Future<ResponseRF> delete({Object model, bool includeChild = false}) async {
    var baseUrl = Libs?.config?.http?.apiBaseURL;
    var apiBaseURL = '$baseUrl/api/Sync/SendPacket';

    dynamic fr = this.fetchRequest;
    fr["TableName"] = model.runtimeType.toString();
    fr["Command"] = 'Delete';
    //fr["UserGuid"] = userGuid;  //UserId
    fr['ApiPacket']['Packet'] = Dynamic(model).modelToDataMap();
    var response = await rf.api.httpPost(url: apiBaseURL, requestBody: json.encode(fr));

    var _response = jsonDecode(response.body); //[Message] ??

    return ResponseRF.parse(_response);
  }

  Future<List<T>> read<T>({
    String select = "",
    String where = "",
    String order = "",
    int pageNo = 1,
    int pageSize,
    Function onComplete,
    //packetType = Sigle/List
  }) async {
    return _post<T>(
      command: 'GetData',
      select: select,
      where: where,
      order: order,
      pageNo: pageNo,
      pageSize: pageSize ?? Libs.config.data.pageSize,
      onComplete: onComplete,
    );
  }

/*
  "UserId": 0,
        "TableName": "Student",
        "Command": "GetData",
      "TotalRecord": 0,
      "PageNo": 1,
      "PageSize": 50,
      "UserGuid": "0",
  "ApiPacket": {},

* */
  Future<List<T>> _post<T>({
    String command,
    bool getTotal = false,
    int pageNo,
    int pageSize,
    String userGuid,
    String select = "",
    String where = "",
    String order = "",
    bool includeChild = false,
    dynamic apiPacket,
    List<T> packetList,
    Function onComplete,
  }) async {
    if (where.isNotEmpty) {
      if (select.isEmpty) {
        select = 'select * from ${T.toString()}';
      }
    }

    dynamic fr = this.fetchRequest;
    fr["TableName"] = T.toString();
    fr["Command"] = command;
    fr["TotalRecord"] = getTotal ? 100 : 0;
    fr["PageNo"] = pageNo;
    fr["PageSize"] = pageSize;
    fr["UserGuid"] = userGuid;
    fr['ApiPacket'] = apiPacket ??
        {
          "Packet": {
            "ModelName": T.toString(),
            "Select": select,
            "Where": where,
            "OrderBy": order,
            "IncludeChild": includeChild
          },
          "PacketList": packetList
        };

    var baseUrl = Libs?.config?.http?.apiBaseURL;
    var apiBaseURL = '$baseUrl/api/Fetch/GetPacket';

    List<T> _model = [];

    print('rf.http.post.request : ' + json.encode(fr));
    http.post(Uri.parse(apiBaseURL), body: json.encode(fr), headers: {
      HttpHeaders.contentTypeHeader: "application/json",
    }).then((value) async {
      print('rf.http.post.response : ' + value.body);
      var models = jsonDecode(value.body)['ApiPacket']['PacketList'];
      for (var item in models) {
        var newModel = Dynamic(T).newClass();
        Dynamic(newModel).dataMapToModel(item);
        _model.add(newModel);
      }
      await Future.delayed(Duration(milliseconds: 2000)); //FOR TEST
      if (onComplete != null) onComplete(_model);
      return _model;
    });
  }

  Future<http.Response> httpGet({
    String url,
    String headers = "application/json",
  }) async {
    var response = await http.get(Uri.parse(url), headers: {
      HttpHeaders.contentTypeHeader: headers,
    });
    return response;
  }

  //region Single Sync/fetch
  _singleFetch(
    BuildContext context,
    Object model, {
    int pageNo = 1,
    Function onFetchComplete,
    bool showLoader = true,
    Configuration config,
    String dialogMsg = "Loading",
    dynamic jsonPacket = null,
  }) async {
    print(DateTime.now());

    percentage = 0; //??

    if (showLoader)
      pdlg = ProgressDialog(context, type: ProgressDialogType.Normal, isDismissible: true)
        ..style(message: dialogMsg);

    var baseUrl = Libs?.config ?? config;

    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Login';

    print('TotalRecord: $totalRecord');
    print('Request: ' + jsonPacket.toString());

    if (showLoader) await pdlg.show();

    var response = await postSingleFetchRequest(apiBaseURL, model, 1, jsonPacket);

    return await processSingleFetchResponse(response, model, 1, onFetchComplete);
  }

  postSingleFetchRequest(String apiBaseURL, dynamic model, int pageNo, [dynamic jsonPacket]) async {
    var modelName = model.runtimeType.toString();
    packet() =>
        {"ModelName": "$modelName", "Select": "", "Where": "", "OrderBy": "", "IncludeChild": true};

    var userGuiID = await Libs.prefGet('UserGeoId') ?? '0';
    fetchRequest['ApiPacket'] = {"Packet": jsonPacket ?? packet(), "PacketList": null};
    fetchRequest['TableName'] = modelName;
    fetchRequest['PageNo'] = pageNo;
    fetchRequest['UserGuid'] = userGuiID;

    print('Post Request Call : $pageNo');
    fetchRequest['TotalRecord'] = pageNo == 1 ? 1 : 0;
    print('Request Body $pageNo : $fetchRequest');

    var apiResponse =
        await http.post(Uri.parse(apiBaseURL), body: json.encode(jsonPacket), headers: {
      HttpHeaders.contentTypeHeader: "application/json",
    });

    await dbHelper.insertDataHistory(modelName, json.encode(jsonPacket), apiResponse.body);
    return apiResponse;
  }

  processSingleFetchResponse(
      http.Response response, dynamic model, int pageNo, Function onFetchComplete) async {
    if (response.statusCode == 200) {
      var json = jsonDecode(response.body);
      print('Api Response : $response');
      print('Api Response $pageNo - JSON : ${json}');

      if (pageNo == 1) totalRecord = json['TotalRecord'];

      await pdlg.hide();
      //print('Is successfull request : '+json['Success'].t);
      return jsonDecode(response.body);
    } else {
      await pdlg.hide();
      try {
        if (response.body.isNotEmpty) {
          var json = jsonDecode(response.body);
          if (onFetchComplete != null) {
            print(json['Message'] + pageNo.toString());
            onFetchComplete(
                json['Message'] + pageNo.toString(), 'failed', json['ApiPacket']['PacketList']);
          }
        } else {
          print('500: Internal Server Error');
          if (onFetchCompleted != null) {
            onFetchCompleted('500', 0);
          }
        }
      } on Exception catch (_) {
        return null;
      }

      return null;
    }
  }

  //endregion

  //paging + service
  _fetch(
    BuildContext context,
    Object model, {
    String serviceName,
    int pageNo = 1,
    Function onFetchComplete,
    bool showLoader = true,
    Configuration config,
    String dialogMsg = "Downloading",
    dynamic jsonPacket = null,
  }) async {
    print(DateTime.now());

    totalRecord = 0;
    percentage = 0; //??
    completedPages.clear();

    print('Scanning Fetch History');
    await dbHelper.clearFetchHistory(model.runtimeType.toString()); //No Need

    FetchHistory fetchHistory = await dbHelper.readFetchHistory(model.runtimeType.toString());

    print('FetchHistory : $fetchHistory');
    await dbHelper.insertFetchHistory(
        model.runtimeType.toString(), pages, totalRecord, 0); //No Need

    if (showLoader)
      pdlg = ProgressDialog(context, type: ProgressDialogType.Normal, isDismissible: true)
        ..style(message: dialogMsg);

    var baseUrl = Libs?.config ?? config;
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Fetch/GetPacket';

    if (fetchHistory != null) totalRecord = fetchHistory.TotalRecord;
    print('TotalRecord: $totalRecord');

    if (showLoader) await pdlg.show();

    if (totalRecord == 0) {
      if (!completedPages.contains(pageNo)) {
        var response = await postFetchRequest(apiBaseURL, model, 1, jsonPacket);
        await processFetchResponse(serviceName, response, model, 0, 1, onFetchComplete);
      }
    }

    int totalPage = (totalRecord / pageSize).ceil();
    sleep(Duration(seconds: 1));
    for (int i = 2; i <= totalPage; i++) {
      if (!completedPages.contains(i)) {
        currentPage = i;
        pdlg != null ?? pdlg.update(message: model.runtimeType.toString() + " " + dialogMsg);
        var response = await postFetchRequest(apiBaseURL, model, i, jsonPacket);
        await processFetchResponse(serviceName, response, model, totalPage, i, onFetchComplete);
        sleep(Duration(seconds: 2));
      }
    }
    print(DateTime.now());
    print('Completed Fetch With Service');
  }

  postFetchRequest(String apiBaseURL, dynamic model, int pageNo, [dynamic jsonPacket]) async {
    var userID = await Libs.prefGet('UserId') ?? 0;
    var userGuID = await Libs.prefGet('UserGeoId') ?? 0;
    print('query:' + Models.models[model.runtimeType.toString()].whereFetch.toString());
    var where = Models.models[model.runtimeType.toString()].whereFetch
        .toString()
        .replaceAll("{SystemUser}", userID.toString());

    //where = where.isEmpty ? ' UserId = $userID' : ' and UserId = $userID';
    var select = Models.models[model.runtimeType.toString()].selectFetch
        .toString()
        .replaceAll("{SystemUser}", userID.toString());

    where = await processPrefKey(where);
    select = await processPrefKey(select);
    var order = Models.models[model.runtimeType.toString()].orderFetch;

    var modelName = model.runtimeType.toString();
    packet() => {
          "ModelName": "$modelName",
          "Select": "$select",
          "Where": "$where",
          "OrderBy": "",
          "IncludeChild": true
        };
    //packet() => {"ModelName": "$modelName", "Select": "", "Where": "", "OrderBy": "", "IncludeChild": true};

    fetchRequest['ApiPacket'] = {"Packet": jsonPacket ?? packet(), "PacketList": null};
    fetchRequest['TableName'] = modelName;
    fetchRequest['PageNo'] = pageNo;
    fetchRequest['UserGuid'] = userGuID;

    print('Post Request Call : $pageNo');
    fetchRequest['TotalRecord'] = pageNo == 1 ? 1 : 0;
    print('Request Body $pageNo : ' + json.encode(fetchRequest));

    var apiResponse =
        await http.post(Uri.parse(apiBaseURL), body: json.encode(fetchRequest), headers: {
      HttpHeaders.contentTypeHeader: "application/json",
    });
    await dbHelper.insertDataHistory(modelName, json.encode(fetchRequest), apiResponse.body);
    return apiResponse;
  }

  processPrefKey(String data) async {
    RegExp regExp = new RegExp('{([a-zA-Z]*)}');
    var match = regExp.allMatches(data);
    print('working');
    var mData = data;
    for (var m in match) {
      var sdata = await Libs.prefGet(m.group(1).toString());
      mData = await mData.replaceFirst(regExp, sdata.toString() ?? '0');
    }
    return mData;
  }

  processFetchResponse(String serviceName, http.Response response, dynamic model, int totalPage,
      int pageNo, Function onFetchComplete) async {
    if (response.statusCode == 200) {
      var json = jsonDecode(response.body);
      print('Api Response : $pageNo');
      print('Api Response $pageNo - JSON : ${json}');

      if (json['Success']) {
        if (pageNo == 1) totalRecord = json['TotalRecord'];
        if (json['ApiPacket']['PacketList'] != null) {
          var data = json['ApiPacket']['PacketList'] as List;
          print('Total Record : ${data.length}');
          print('Dumping data....');

          var newModel = Dynamic(model).newClass();

          //saving to database
          if (data.length > 0) {
            await Dynamic(newModel).apiJsonMapToDb(newModel, data);
          }

          completedPages.add(pageNo);
          print('Pages Completed: ${completedPages.toList().toString()}');
          pages += pageNo.toString() + ',';

          bool allPageCompleted = false;

          if (totalPage == pageNo || pageSize >= totalRecord) {
            allPageCompleted = true;
          }

          await dbHelper.updateFetchHistoryPages(
              model.runtimeType.toString(), pages, totalRecord, allPageCompleted ? 1 : 0);

          if (allPageCompleted) {
            await dbHelper.updateFetchTable(serviceName, model.runtimeType.toString());
            print("Updated Remark to :" + model.runtimeType.toString());
          }
        }
      } else {
        await dbHelper.updateFetchHistoryPages(model.runtimeType.toString(), pages, totalRecord, 4);
        await dbHelper.updateFetchTable(serviceName, model.runtimeType.toString());
        print(
            "```````````````````````````````````````````````````````````````````````````````````");
        print(
            "``````````````````````````````````Failed to Fetch``````````````````````````````````");
        print("``````````````````````````````````" +
            model.runtimeType.toString() +
            '``````````````````````````````````');
        print(
            "```````````````````````````````````````````````````````````````````````````````````");
        print(
            "```````````````````````````````````````````````````````````````````````````````````");
        print("Error Reason : " + json['Message'].toString());
      }
    } else {
      await dbHelper.updateFetchHistoryPages(model.runtimeType.toString(), pages, totalRecord, 4);
      await dbHelper.updateFetchTable(serviceName, model.runtimeType.toString());
      print("```````````````````````````````````````````````````````````````````````````````````");
      print("``````````````````````````````````Failed to Fetch``````````````````````````````````");
      print("``````````````````````````````````" +
          model.runtimeType.toString() +
          '``````````````````````````````````');
      print("```````````````````````````````````````````````````````````````````````````````````");
      print("```````````````````````````````````````````````````````````````````````````````````");

      if (response.body.isNotEmpty) {
        var json = jsonDecode(response.body);
        if (onFetchComplete != null) {
          print(json['Message'] + pageNo.toString());
          //onFetchComplete(json['Message'] + pageNo.toString(), 'failed', json['ApiPacket']['PacketList']);
        }
      } else {
        print('500: Internal Server Error');
        if (onFetchCompleted != null) {
          onFetchCompleted('500', 0);
        }
        if (onFetchFailed != null) onFetchFailed(model.runtimeType.toString());
      }
      return null;
    }
  }

  remoteGet(Object model, int pageNo, Function onComplete) async {
    //await fetchModel(Libs.context, model, pageNo: pageNo, onComplete: onComplete);
    await fetchModel(
        context: Libs.context,
        modelsName: model.runtimeType.toString(),
        onFetchComplete: onComplete);
  }

  remoteDelete(Object model, Function onSuccess) async {
    var result;
    var modelList = [];
    modelList.add(model);
    for (var model in modelList) {
      //var json = Dynamic(model).syncableDataJsonMap();//??????
      var json = Dynamic(model).modelToDataMap(); //??????
      await _sync(Libs.context, model, json, command: 'delete', onSuccess: (m, s, o) {
        result = m;
        if (onSuccess != null) onSuccess(m, s, o);
      });
    }

    return result;
  }

  remoteSave(Object model, String command, Function onSuccess) async {
    var result;
    var modelList = List();
    modelList.add(model);
    for (var model in modelList) {
      //var json = Dynamic(model).syncableDataJsonMap();
      var json = Dynamic(model).modelToDataMap(); //??????
      await _sync(Libs.context, model, json, command: command, onSuccess: (m, s, o) {
        result = m;
        if (onSuccess != null) onSuccess(m, s, o);
      });
    }
    return result;
  }

  downloadBeforeSignup(
    BuildContext context, {
    bool forceDownload = false,
    String instructionText,
    String retryText,
    double instructionFontSize,
    String titleText,
    String subTitleText,
    String serviceName,
    Color indicatorColor,
    Color instructionTextColor,
    Color buttonTextColor,
    double buttonFontSize,
    String imagePath,
    Color imageBackgroundColor,
    double imageSize = 44,
    Color cancelButtonColor,
    Color acceptButtonColor,
    Color retryButtonColor,
    double dialogBorderRadius,
    String animationPath,
    Function dispatcher,
    Function onCancel,
    Function onComplete,
  }) {
    String tablesName = '';
    Models models = Models();
    bool isFailed = false;
    bool isLoading = false;
    bool isInternetAvailable = true;
    List<ModelRF> modelRfs = [];
    String failedModels = '';
    for (var item in models.get().entries) {
      var modelRF = item.value as ModelRF;
      if (modelRF.fetchBeforeSignUp && (!modelRF.alreadyFetched || forceDownload)) {
        print('item :' + item.key.toString());
        modelRF.alreadyFetched = false;
        tablesName = tablesName + ',' + modelRF.model.runtimeType.toString();
        modelRfs.add(modelRF);
      }
    }

    Dialog alert = Dialog(
      elevation: 10,
      backgroundColor: Colors.transparent,
      child: AnimatedContainer(
          duration: Duration(seconds: 2),
          decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(dialogBorderRadius ?? 30)),
          ),
          width: MediaQuery.of(context).size.width,
          child: StatefulBuilder(
            builder: (BuildContext context, setState) {
              onFetchCompleted = (model, count) {
                print('Model: ' + model.toString());
                if (model == 'all fetch completed') {
                  print("Printing on Complete: " + model + " " + count.toString());
                  isLoading = false;
                  if (!isFailed) {
                    Navigator.pop(context);
                    rf.msg(context, 'Data loaded successfully', type: MessageType.success);
                    if (onComplete != null) onComplete.call();
                  } else {
                    setState(() {});
                  }
                } else {
                  modelRfs
                      .where((element) => element.model.runtimeType.toString() == model)
                      .first
                      .alreadyFetched = true;
                  setState(() {});
                }
              };
              onFetchFailed = (model) {
                isFailed = true;
                //isLoading = false;
                failedModels = failedModels + ',' + model.toString();
                setState(() {});
              };
              return Container(
                padding: const EdgeInsets.all(12),
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Padding(
                      padding: const EdgeInsets.only(left: 12.0, top: 12.0),
                      child: Row(
                        children: <Widget>[
                          CircleAvatar(
                            radius: imageSize / 2,
                            backgroundColor: imageBackgroundColor ?? Colors.grey,
                            child: imagePath == null
                                ? Icon(Icons.api)
                                : Center(
                                    child: Image.asset(imagePath,
                                        height: imageSize ?? 40, width: imageSize ?? 40)),
                          ),
                          SizedBox(
                            width: 10,
                          ),
                          titleText == null
                              ? Container()
                              : Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: <Widget>[
                                    Text(titleText,
                                        style:
                                            TextStyle(fontSize: 20.0, fontWeight: FontWeight.bold)),
                                    subTitleText == null
                                        ? Container()
                                        : Text(subTitleText,
                                            style: TextStyle(
                                                fontSize: 14.0,
                                                fontWeight: FontWeight.w400,
                                                color: Colors.black54)),
                                  ],
                                )
                        ],
                      ),
                    ),
                    SizedBox(
                      height: 30,
                    ),
                    isLoading
                        ? Center(
                            child: Column(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisSize: MainAxisSize.min,
                            children: [
                              Image.asset(
                                animationPath ?? 'assets/images/loaderm.gif',
                                width: 50,
                                height: 50,
                              ),
                              SizedBox(
                                height: 20,
                              ),
                              TextRF(
                                'Downloading . . .',
                                size: instructionFontSize ?? 20,
                                color: instructionTextColor ?? Colors.blue,
                              ),
                            ],
                          ))
                        : Container(
                            margin: const EdgeInsets.symmetric(horizontal: 20),
                            child: Center(
                              child: TextRF(
                                isFailed ? retryText ?? instructionText : instructionText,
                                size: instructionFontSize ?? 20,
                                color: instructionTextColor ?? Colors.blue,
                              ),
                            ),
                          ),
                    SizedBox(
                      height: 30,
                    ),
                    Container(
                      width: ScreenRF.width(context),
                      margin: const EdgeInsets.only(bottom: 30),
                      height: 4,
                      child: Center(
                        child: ListView.builder(
                            itemCount: modelRfs.length,
                            shrinkWrap: true,
                            scrollDirection: Axis.horizontal,
                            itemBuilder: (context, index) {
                              return Padding(
                                padding: const EdgeInsets.symmetric(horizontal: 5),
                                child: Container(
                                  width: 12,
                                  height: 4,
                                  decoration: BoxDecoration(
                                    color: modelRfs[index].alreadyFetched
                                        ? indicatorColor ?? Colors.green
                                        : Colors.grey,
                                    borderRadius: BorderRadius.all(Radius.circular(3)),
                                  ),
                                ),
                              );
                            }),
                      ),
                    ),
                    isLoading
                        ? Container()
                        : Row(
                            children: [
                              Expanded(child: SizedBox()),
                              ButtonRF(
                                height: 45,
                                width: ScreenRF.width(context) / 4,
                                color: cancelButtonColor ?? Colors.red,
                                buttonType: ButtonType.flat,
                                borderRadious: 10,
                                text: Text(
                                  'Cancel',
                                  style: TextStyle(
                                      fontSize: buttonFontSize ?? 18,
                                      color: buttonTextColor ?? Colors.black),
                                ),
                                onTap: () {
                                  Navigator.pop(context);
                                  if (onCancel != null) onCancel.call();
                                },
                              ),
                              SizedBox(
                                width: 20,
                              ),
                              ButtonRF(
                                height: 45,
                                width: ScreenRF.width(context) / 4,
                                color: isFailed
                                    ? retryButtonColor ?? Colors.blue
                                    : acceptButtonColor ?? Colors.blue,
                                buttonType: ButtonType.flat,
                                borderRadious: 10,
                                text: Text(
                                  isFailed ? 'Retry' : 'Start',
                                  style: TextStyle(
                                      fontSize: buttonFontSize ?? 18,
                                      color: buttonTextColor ?? Colors.black),
                                ),
                                onTap: () async {
                                  isInternetAvailable = await SystemRF.network.hasInternet();
                                  if (isInternetAvailable) {
                                    isLoading = true;
                                    if (!isFailed) {
                                      setState(() {});
                                      print(tablesName);
                                    }
                                    await fetchModel(
                                      context: context,
                                      serviceName: serviceName,
                                      dispatcher: dispatcher,
                                      showDialog: false,
                                      closeDialog: false,
                                      showNotification: false,
                                      modelsName: isFailed ? failedModels : tablesName,
                                    );
                                  } else {
                                    rf.msg(context, 'Internet Connection unavailable');
                                  }
                                },
                              ),
                              SizedBox(
                                width: 10,
                              ),
                            ],
                          ),
                    SizedBox(
                      height: 10,
                    ),
                  ],
                ),
              );
            },
          )),
    );
    if (tablesName.isNotEmpty)
      showDialog(
          context: context,
          barrierDismissible: false,
          builder: (BuildContext context) {
            return WillPopScope(
                onWillPop: () async => false,
                child:
                    BackdropFilter(filter: ImageFilter.blur(sigmaX: 5, sigmaY: 5), child: alert));
          });
  }
}

class ResponseRF {
  String Command;
  String Tag;
  bool Success;
  String Message;
  String ServerRecordId;
  String RecordId;
  int TotalRecord;
  int PageNo;
  int PageSize;
  ApiPackets ApiPacket;
  int Status;

  ResponseRF({
    this.Command,
    this.Tag,
    this.Success,
    this.Message,
    this.ServerRecordId,
    this.RecordId,
    this.TotalRecord,
    this.PageNo,
    this.PageSize,
    this.ApiPacket,
    this.Status,
  });

  static ResponseRF parse(dynamic response) {
    return ResponseRF(
      Command: response['Command'],
      Tag: response['Tag'],
      Success: response['Success'],
      Message: response['Message'] == 'ADD has been Successful'
          ? "Data saved successfully"
          : response['Message'].toString(),
      ServerRecordId: response['ServerRecordId'].toString(),
      RecordId: response['RecordId'].toString(),
      TotalRecord: response['TotalRecord'],
      PageNo: response['PageNo'],
      PageSize: response['PageSize'],
      ApiPacket: response['ApiPacket'],
      Status: response['Status'],
    );
  }
}

class ApiPackets {
  dynamic packet;
  dynamic packetList;
}

/*backlog
0. remove g from config
1.      not deliting old DONE (WS Pening)
2.      remove old fetch fn DONE
2.1 non service old delete
3. test bulk sync + fetch (S + WS)
4. remote CRUD
5. if response error ??
6. delete bad sync files & class
*/

//RDB ???????
//BAD CODE
//  onFetchResponse(Response response, dynamic model, bool showLoader, Function onSuccess) async {
//    //responseCount++;
//
//    var fetchItemCount = 0;
//    print('Api Response Status Code : ${response.statusCode}');
//    if (response.statusCode == 200) {
//      var json = jsonDecode(response.body);
//      print('Api Response ${json['PageNo']} - JSON : $json');
//
//      var data = json['ApiPacket']['PacketList'] as List;
//
//      var newModel;
//      print('Total Record : ${data.length}');
//      for (var item in data) {
//        fetchItemCount++; // for upload status percentage
//        print('Record Count : $fetchItemCount');
//        newModel = Dynamic((Libs.appModels.get()[model.runtimeType.toString()] as ModelRF).model).newClass();
//        item['StudentId'] = null; //???
//        Dynamic(newModel).dataMapToModel(item);
//        db.save(newModel);
//
//        double per = ((fetchItemCount / data.length) * 100).roundToDouble();
//        percentage = per == 100.0 ? 0 : per;
//      }
//
//      completedPages.add(json['PageNo']);
//      pages += json['PageNo'].toString() + ',';
//      dbHelper.updateFetchHistoryPages(newModel.runtimeType.toString(), pages, totalRecord, 0);
//
//      if (onSuccess != null)
//        onSuccess(json['Message'] + fetchItemCount.toString(), 'success', json['ApiPacket']['PacketList']);
//
//      print('Pages Completed (Async): ${completedPages.toList().toString()}');
//
//      if ((totalRecord / pageSize).ceil() == completedPages.length) {
//        dbHelper.updateFetchHistorySuccess(newModel.runtimeType.toString(), 1);
//        if (showLoader) await pdlg.hide();
//        print('Fetch Completed');
//      }
//
//      return json['ApiPacket']['PacketList']; // to db
//    } else {
//      if (response.body.isNotEmpty) {
//        var json = jsonDecode(response.body);
//        if (onSuccess != null)
//          onSuccess(json['Message'] + fetchItemCount.toString(), 'failed', json['ApiPacket']['PacketList']);
//      } else {
//        if (onSuccess != null) onSuccess('500: Internal Server Error', 'success', null);
//      }
//      return null;
//    }
//  }

//  dynamic fetch123(
//    BuildContext context,
//    Object model, {
//    bool valid,
//    String loadingMessage,
//    int pageNo = 0,
//    Function onComplete,
//    bool showProgress = true,
//    bool storeWithPaging = false,
//  }) async {
//    if (storeWithPaging) {}
//
//    var params = {
//      "ApiPacket": {
//        "Packet": {
//          "IncludeChild": false,
//          "ModelName": "Student",
//          "OrderBy": "",
//          "Select": "select * from student inner join UserSchool  ON  student.SchoolId=UserSchool.SchoolId",
//          "Where": " UserSchool.UserId = 23 and student.SchoolId=16"
//          // and student.SchoolId=16
//        }
//      },
//      "Command": "GetData",
//      "PageNo": 1,
//      "PageSize": 0,
//      "TableName": "Student",
//      "TotalRecord": 1,
//      "UserId": 23
//    };
//
//    dynamic fetchRND(
//      Object model, {
//      bool valid,
//      String processAlertText,
//      int pageNo = 1,
//      dynamic onComplete,
//    }) async {
//      params['PageNo'] = pageNo;
//
//      var response =
//          await post("http://frmrnd.scibd.info/api/Fetch/GetPacket", body: json.encode(params), headers: {
//        HttpHeaders.contentTypeHeader: "application/json",
//      });
//
//      if (response.statusCode == 200) {
//        var jsonMap = jsonDecode(response.body);
//        //List packetList = jsonMap['ApiPacket']['PacketList'] as List;
////      List<dynamic> records = List();
////      for (int i = 0; i < packetList.length; i++) {
////        var rec =
////            Mapper().jsonToModel(Dynamic(model).newClass(), packetList[i]);
////        records.add(rec);
////      }
//        //onComplete(jsonMap['ApiPacket']['PacketList']);
//        return jsonMap['ApiPacket']['PacketList'];
//      } else {
//        throw Exception('Failed to load album');
//      }
//    }
//  }

///BAD
///

//  ApiResponse abc(dynamic data) {
//    ApiResponse response = ApiResponse();
//
//    response.PageNo = data['PageNo'];
//    var packet = data['ApiPacket']['Packet'];
//    List packetList = data['ApiPacket']['PacketList'] as List;
//
//    response.Success = data['Success']; //??
//    response.Message = data['Message']; //??
//    response.ServerRecordId = data['ServerRecordId']; //
//    response.RecordId = data['RecordId']; //
//    response.TotalRecord = data['TotalRecord']; //
//    response.PageNo = data['PageNo']; //
//    response.PageSize = data['PageSize']; //
//    response.ApiPacket = data['ApiPacket'];
//    response.Status = data['Status']; //
//  }

//class SyncRF {
//  final model;
//  final valid;
//  final handeler;
//  final onComplete;
//  final processAlert;
//
//  SyncRF(this.model, this.handeler, {this.onComplete, this.valid, this.processAlert});
//}
//
//class FetchRF {
//  final model;
//  final valid;
//  final handeler;
//  final onComplete;
//  final processAlert;
//
//  FetchRF(this.model, this.handeler, {this.onComplete, this.valid, this.processAlert});
//}
//
////@Model
//class ApiRequest {
//  int UserId;
//  String TableName;
//  String Command;
//  int TotalRecord;
//  int PageNo;
//  int PageSize;
//  ApiPacket1 ApiPacket = new ApiPacket1();
//}
//
////@Model
//class ApiResponse {
//  bool Success;
//  String Message;
//  int ServerRecordId;
//  Object RecordId;
//  int TotalRecord;
//  int PageNo;
//  int PageSize;
//  ApiPacket1 ApiPacket = new ApiPacket1();
//  int Status;
//}
//
////@Model
//class ApiPacket1<T> {
//  T Packet;
//  List<T> PacketList;
//
//  T getPacket() {
//    return Packet;
//  }
//
//  void setPacket(T packet) {
//    Packet = packet;
//  }
//
//  List<T> getPacketList() {
//    return PacketList;
//  }
//
//  void setPacketList(List<T> packetList) {
//    PacketList = packetList;
//  }
