import 'package:flutter/material.dart';
import 'package:rapidfire/store/modern/onboarding/Language.dart';

import 'Dynamic.dart';
import 'store/StoreRF.dart';
import 'uiux/StyleRF.dart';
import 'uiux/WidgetStylesRF.dart';
import 'uiux/WidgetsRF.dart';

abstract class IConfigRF {
  void configure(Configuration _configuration);
}

abstract class IStoryboardRF {
  StoryboardRF customStoryboard(BuildContext context);
}

abstract class IModelRF {
  void register(Models models);
}

abstract class IBaseModelUpdaterRF {
  void update(Dynamic model);
}

class Configuration {
  AppConfigurationRF app = AppConfigurationRF();
  DbConfigurationRF db = DbConfigurationRF();
  HttpConfigurationRF http = HttpConfigurationRF();
  ModelsConfigurationRF model = ModelsConfigurationRF();
  PageConfigurationRF page = PageConfigurationRF();
  Data data = Data();
  BackgroundJob service = BackgroundJob();
}

class AppConfigurationRF {
  List<String> language = ['en'];
  List<Function> init;
  List<LanguageItems> languages;
  String title;
  String subTitle;
  String appLogo;
  String databaseName;
  bool forceAppUpgrade = false;
  TemplateRF template;
  ThemeRF theme;
  IThemeDataRF themeData;

  IPageOutlineRF outline;
  StyleStoreDefaults widgetStyle;
  dynamic onboard; //IAppOnboardRF
}

class DbConfigurationRF {
  String name = 'app.db';
  String backUpMail = '';
  int version = 1;
  Function onUpgrade;
}

class HttpConfigurationRF {
  String apiBaseURL = '';
}

class ModelsConfigurationRF {
  IModelRF appModels;
  List<Function> validators;
  IBaseModelUpdaterRF baseModelUpdater;
}

class PageConfigurationRF {
  TransitionTypeRF transition;
  ScrollPhysicsRF scrollPhysics;
  IPageRF appBar;
}

class Data {
  int pageSize;
  int downloadLimit;
}

class BackgroundJob {
  List<Function> jobs;
  IConfigRF bgConfig;
}

enum ScrollPhysicsRF { glow, bounce }

enum TransitionTypeRF {
  fade,
  rightToLeft,
  leftToRight,
  upToDown,
  downToUp,
  scale,
  rotate,
  size,
  rightToLeftWithFade,
  leftToRightWithFade,
}

class Models {
  static Map<String, ModelRF> models = new Map();

  const Models();

  Map<String, Object> get() {
    return models;
  }

  void add(ModelRF model) {
    models[model.model.runtimeType.toString()] = model;
  }
}

class ModelRF {
  Object model;
  String displayName;
  Icon displayIcon;
  String displayImage;
  bool syncable;
  String syncStatusField;
  List<Object> fetchDependencies;
  bool fetchIncludeChild;
  bool initialFetch;
  String selectFetch;
  String whereFetch;
  String orderFetch;
  bool fetchBeforeSignUp;
  bool alreadyFetched;
  Function fetchDependencyCallback;

  ModelRF({
    this.model,
    this.displayName,
    this.displayIcon,
    this.displayImage = "assets/images/profile.png",
    this.syncable = true,
    this.syncStatusField = 'DataStatus',
    this.fetchDependencies,
    this.fetchIncludeChild = true,
    this.initialFetch = false,
    this.selectFetch = "",
    this.whereFetch = "",
    this.orderFetch = "",
    this.fetchBeforeSignUp = false,
    this.alreadyFetched = false,
    this.fetchDependencyCallback,
  });
}
