import 'package:flutter/material.dart';
import 'package:rapidfire/UtilRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:uuid/uuid.dart';

import 'Dynamic.dart';

extension HexColor on Color {
  /// String is in the format "aabbcc" or "ffaabbcc" with an optional leading "#".
  Color fromHex(String hexString) {
    final buffer = StringBuffer();
    if (hexString.length == 6 || hexString.length == 7) buffer.write('ff');
    buffer.write(hexString.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }

  /// Prefixes a hash sign if [leadingHashSign] is set to `true` (default is `true`).
  String toHex({bool leadingHashSign = true}) => '${leadingHashSign ? '#' : ''}'
      '${alpha.toRadixString(16).padLeft(2, '0')}'
      '${red.toRadixString(16).padLeft(2, '0')}'
      '${green.toRadixString(16).padLeft(2, '0')}'
      '${blue.toRadixString(16).padLeft(2, '0')}';
}

extension StringColor on String {
  Color get toColor {
    final buffer = StringBuffer();
    final color = this;
    if (color.length == 6 || color.length == 7) buffer.write('ff');
    buffer.write(color.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }
}

extension TabBarConverter on TabBarRF {
  TabBar get toTabBar {
    var tab = null;
    if (this is TabBarRF) {
      var _tab = this.tab;
      tab = TabBar(
        controller: _tab.controller,
        tabs: _tab.tabs,
      );
    }
    return tab;
  }
}

extension DynamicExt on Object {
  bool get changed {
    return Dynamic(this).changed1();
  }
}

class BaseModelParm {
  static int UserId;
  static int GeoType;
  static dynamic Gps;
  static dynamic Pref;
}

extension updateBaseModelExt on Object {
  Object updateBaseModel() async {
    final model = this;

    BaseModelParm.Pref = BaseModelParm.Pref ?? await SharedPreferences.getInstance();
    BaseModelParm.UserId = BaseModelParm.UserId ?? await BaseModelParm.Pref.getInt('UserId') ?? 0;
    BaseModelParm.GeoType = BaseModelParm.GeoType ?? await BaseModelParm.Pref.getString('GeoType') ?? 0;
    BaseModelParm.Gps = BaseModelParm.Gps ?? await UtilRF.geoPosition();

    var userId = BaseModelParm.UserId;
    var geoType = BaseModelParm.GeoType;
    var gps = BaseModelParm.Gps; //await UtilRF.geoPosition();
    var dynModel = Dynamic(model);

    var dataCollectionDate = await dynModel.get('DataCollectionDate');
    var dataCollectionTime = await dynModel.get('DataCollectionTime');
    var dataStatus = await dynModel.get('DataStatus');
    var modifiedBy = await dynModel.get('ModifiedBy');

    dynModel.set('GeoType', geoType);
    dynModel.set('CompanyCode', '6'); //geo controller
    dynModel.set('TeaEstateCode', '01');
    dynModel.set('EstateDivisionCode', '01');
    dynModel.set('ChildrenClubCode', '01');
    dynModel.set('DataCollectionDate', dataCollectionDate ?? UtilRF.getDate());
    dynModel.set('DataCollectionTime', dataCollectionTime ?? UtilRF.getTime());
    dynModel.set('UserId', userId); //'pref'
    dynModel.set('Latitude', gps == null ? 0 : gps.latitude.toString()); //SystemRF.gps
    dynModel.set('Longitude', gps == null ? 0 : gps.longitude.toString());
    dynModel.set('DataStatus', dataStatus ?? 0);
    dynModel.set('ModifiedBy', modifiedBy ?? 0);
    dynModel.set('ModificationDate', UtilRF.getDate());

    // in future make it dynamic
    //Dynamic(model).modelMembers.forEach((key, value) {});

    return model;
  }
}

//debugger(when: name.contains('Rokybul1Ex'));
