import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:rapidfire/LocalizationProviderRF.dart';
import 'package:rapidfire/GlobalScopeRF.dart';

class LocalizationRF {
  static final LocalizationRF _singleton = new LocalizationRF._internal();

  LocalizationRF._internal();

  static LocalizationRF get instance => _singleton;

  Map<dynamic, dynamic> _localisedValues;

  Future<LocalizationRF> load(Locale locale) async {
    String jsonContent = await rootBundle.loadString("assets/language/lang_${locale.languageCode}.json");
    _localisedValues = json.decode(jsonContent);
    try {
      //dangerous on hot reloading ??
      if (LanguageRF.onLoadComplete != null) {
        LanguageRF.onLoadComplete.call();
      }
    } catch (e) {}

    return this;
  }

  String text(String model, String key) {
    if (model == null || model.isEmpty) model = 'System';
    if (_localisedValues[model] == null) return "$key";
    else return _localisedValues[model][key] ?? "$key";
  }
}

class LanguageDelegateSingle extends LocalizationsDelegate<LocalizationRF> {
  const LanguageDelegateSingle();

  @override
  bool isSupported(Locale locale) => Libs.config.app.language.contains(locale.languageCode); //'en', 'es'

  @override
  Future<LocalizationRF> load(Locale locale) async {
    return await LocalizationRF.instance.load(locale);
  }

  @override
  bool shouldReload(LanguageDelegateSingle old) => true;
}

class LanguageDelegate extends LocalizationsDelegate<LocalizationRF> {
  final Locale overriddenLocale;

  const LanguageDelegate(this.overriddenLocale);

  @override
  bool isSupported(Locale locale) => overriddenLocale != null;

  @override
  Future<LocalizationRF> load(Locale locale) async {
    return await LocalizationRF.instance.load(overriddenLocale);
  }

  //Future<AppLocalizations> load(Locale locale) => AppLocalizations.instance.load(overriddenLocale);

  @override
  bool shouldReload(LanguageDelegate old) => true;
}
