library rapidfire;

import 'package:catcher/catcher.dart';

//import 'package:flushbar/flushbar.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:flutter_statusbarcolor/flutter_statusbarcolor.dart';
import 'package:page_transition/page_transition.dart';
import 'package:rapidfire/AccessControl.dart';
import 'package:rapidfire/Api.dart';
import 'package:rapidfire/ContentPost.dart';
import 'package:rapidfire/models/AppUser.dart';
import 'package:rapidfire/uiux/StyleRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:rapidfire/vendor/flushbar.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'BGSyncSMain.dart';
import 'Config.dart';
import 'Const.dart';
import 'Db.dart';
import 'Dynamic.dart';
import 'GlobalRF.dart';
import 'GlobalScopeRF.dart';
import 'GlobalScopeRF.dart' as g;
import 'LocalizationRF.dart';
import 'Rdb.dart';
import 'RestartRF.dart';
import 'SystemRF.dart';
import 'chatting/Chat.dart';
import 'models/ChatModels.dart';

export 'package:carousel_slider/carousel_slider.dart';
export 'package:chips_choice/chips_choice.dart';
export 'package:ext_storage/ext_storage.dart';
export 'package:firebase_core/firebase_core.dart';
export 'package:firebase_database/firebase_database.dart';
export 'package:firebase_messaging/firebase_messaging.dart';
export 'package:flutter_local_notifications/flutter_local_notifications.dart';
export 'package:flutter_speed_dial/flutter_speed_dial.dart';
export 'package:intl/intl.dart';
export 'package:jiffy/jiffy.dart';
export 'package:permission_handler/permission_handler.dart';
export 'package:progress_dialog/progress_dialog.dart';
export 'package:rapidfire/Api.dart';
export 'package:rapidfire/BGSyncSMain.dart';
export 'package:rapidfire/Const.dart';
export 'package:rapidfire/Db.dart';
export 'package:rapidfire/Extensions.dart';
export 'package:rapidfire/GlobalRF.dart';
export 'package:rapidfire/store/StoreRF.dart';
export 'package:rapidfire/uiux/DrawerRF.dart';
export 'package:rapidfire/uiux/RipplesRF.dart';
export 'package:rapidfire/uiux/StyleRF.dart';
export 'package:rapidfire/uiux/WidgetStylesRF.dart';
export 'package:rapidfire/uiux/WidgetsRF.dart';
export 'package:upgrader/upgrader.dart';
export 'package:workmanager/workmanager.dart';

export 'Config.dart';
export 'Dynamic.dart';
export 'UtilRF.dart';
export 'ContentPost.dart';

enum MessageType { info, error, warning, success }

abstract class IJobWorkerRF {
  execute(String taskName);
}

Future<void> _firebaseMessagingBackgroundHandler(RemoteMessage message) async {
  await Firebase.initializeApp();
  print('Handling a background message ${message.messageId}');
}

class RapidFire {
  Widget _startPage;
  bool _handelGlobalError;
  bool addNotification;
  AccessControl accessControl;
  IConfigRF _config;
  Configuration _configuration = new Configuration();
  SharedPreferences pref;
  App app;
  ModelObject model; //????
  Map<String, dynamic> models;
  Res lang;
  Db db;
  Rdb rdb;
  Api api;
  Mapper mapper;
  AppStyles style;
  AppOnboardRF onboard;
  PageAction page;
  AppUser appUser;
  ChatRf chat;
  Function onMessageReceive;
  Function onMemberRemoved;
  Function onChatReceived;

  NetworkRF network = NetworkRF();
  HttpRF http = HttpRF();

  BuildContext context;
  IJobWorkerRF worker;

  ContentPost post = ContentPost();

  static Catcher catcher;
  CatcherOptions debugOptions, releaseOptions;

  RapidFire(IConfigRF configuration, Function reflectionAgent,
      {Widget startupPage, bool globalErrorEnabled = true, bool enableNotification = true, onChatReceived}) {
    if (reflectionAgent != null) reflectionAgent.call();

    WidgetsFlutterBinding.ensureInitialized();

    _config = configuration;
    _startPage = startupPage;
    _handelGlobalError = globalErrorEnabled;
    addNotification = enableNotification;

    if (enableNotification) initFirebase();

    if (_handelGlobalError) {
      run(() {
        if (globalErrorEnabled) {
          var explicitMap = {"Error": ConsoleHandler()};
          debugOptions = CatcherOptions(
            DialogReportMode(),
            [
              ConsoleHandler(
                enableApplicationParameters: false,
                enableCustomParameters: false,
                enableStackTrace: true,
                enableDeviceParameters: false,
              )
            ],
            explicitExceptionHandlersMap: explicitMap,
          );
          releaseOptions = CatcherOptions(DialogReportMode(), [
            EmailManualHandler(["a@b.com"]),
          ]);
          catcher = Catcher(
            rootWidget: RestartRF(RapidFireAppRF(_startPage ?? onboard.next())),
            debugConfig: debugOptions,
            releaseConfig: releaseOptions,
          );
        }
      });
    }
  }

  AndroidNotificationChannel channel = const AndroidNotificationChannel(
    'high_importance_channel', // id
    'High Importance Notifications', // title
    'This channel is used for important notifications.', // description
    importance: Importance.high,
  );

  FlutterLocalNotificationsPlugin flutterLocalNotificationsPlugin;

  initFirebase() async {
    await Firebase.initializeApp();
  }

  Future<dynamic> run([dynamic callback]) async {
    rf = this;
    pref = await SharedPreferences.getInstance();
    if (addNotification) {
      await Firebase.initializeApp();
      FirebaseMessaging.onBackgroundMessage(_firebaseMessagingBackgroundHandler);
      flutterLocalNotificationsPlugin = FlutterLocalNotificationsPlugin();
      await flutterLocalNotificationsPlugin
          .resolvePlatformSpecificImplementation<AndroidFlutterLocalNotificationsPlugin>()
          ?.createNotificationChannel(channel);
    }
    _config.configure(_configuration);
    _configuration.model.appModels.register(g.modelList); //==>>>>
    Libs.config = _configuration;

    accessControl = AccessControl();
    db = Db();
    rdb = Rdb();
    api = Api();
    app = App();
    lang = Res();
    mapper = Mapper();
    style = AppStyles();
    page = PageAction();
    chat = ChatRf();
    models = Libs.appModels.get();

    if (AppOnboardRF.currentStep == 0) {
      onboard = AppOnboardRF(
        splash: AppOnboardRF.addItem(_configuration, OnboardItemsRF.splash),
        language: AppOnboardRF.addItem(_configuration, OnboardItemsRF.language),
        intro: AppOnboardRF.addItem(_configuration, OnboardItemsRF.intro),
        login: AppOnboardRF.addItem(_configuration, OnboardItemsRF.login),
        otp: AppOnboardRF.addItem(_configuration, OnboardItemsRF.otp),
        initialSetup: AppOnboardRF.addItem(_configuration, OnboardItemsRF.setup),
        home: AppOnboardRF.addItem(_configuration, OnboardItemsRF.home),
      );
      AppOnboardRF.loadOnboardItems(onboard);
    }

    appUser = AppUser();
    appUser.Name = 'Anawer Hossain';
    Libs.appUser = appUser;

    Libs.msg = msg;

    _configuration.app.init?.forEach((element) => element.call());

    if (callback.toString().contains('Closure: () => Null')) {
      if (_handelGlobalError) {
        callback();
      }
    } else {
      if (!_handelGlobalError) {
        callback(RestartRF(RapidFireAppRF(_startPage ?? onboard.next())));
      }
    }

    await pref.setString(ConstRF.dbName, _configuration.db.name);
    await pref.setInt(ConstRF.dbVersion, _configuration.db.version);
    //await rf.db.execute('update Beneficiary set UserId = 240');
    PageRF.keyboardOpened = false;
  }

  static updateGlobalErrorLanguage() {
    var explicitMap = {"Error": ConsoleHandler()};
    if (catcher != null) {
      catcher.updateConfig(
        debugConfig: CatcherOptions(
          DialogReportMode(),
          [
            ConsoleHandler(
              enableApplicationParameters: false,
              enableCustomParameters: false,
              enableStackTrace: true,
              enableDeviceParameters: false,
            )
          ],
          localizationOptions: [globalErrorMessage()],
          explicitExceptionHandlersMap: explicitMap,
        ),
      );
    }
  }

  static globalErrorMessage() {
    return LocalizationOptions(
      "en",
      dialogReportModeTitle: rf.lang.text('System', 'dialogReportModeTitle'),
      dialogReportModeDescription: rf.lang.text('System', 'dialogReportModeDescription'),
      dialogReportModeAccept: rf.lang.text('System', 'dialogReportModeAccept'),
      dialogReportModeCancel: rf.lang.text('System', 'dialogReportModeCancel'),
    );
  }

  addChatListener() {
    var userid = pref.getString('UserGeoId');
    print('addChatListener UserID:' + userid);

    if (userid != null && userid.isNotEmpty) {
      FirebaseDatabase.instance.reference().child('users/${userid}').onValue.listen((event) async {
        print('golobal listener');
        if (onChatReceived != null) onChatReceived(event.snapshot.value);
        var result = 0;
        if (event.snapshot.value['IsRemoved']) {
          result =
              await db.execute('delete from ChatGroup where ChatGroupId = \'${event.snapshot.value['ChatGroupId']}\'');
          /* result = await db.execute('delete from TrainersForum where ForumId = \'${event.snapshot.value['ChatGroupId']}\'');
          print(result);*/
          if (onMemberRemoved != null) onMemberRemoved(event.snapshot.value['ChatGroupId']);
        }
        var chatGroupData = await db.get(
            ChatGroup(), 'select * from ChatGroup where ChatGroupId = \'${event.snapshot.value['ChatGroupId']}\'');
        if (chatGroupData != null && chatGroupData.length > 0) {
          if (event.snapshot.value['LastMessageType'] == 'notification') {
            result = await db.execute(
                'Update ChatGroup set Name=\'${event.snapshot.value['ChatGroupName']}\',Message=\'${event.snapshot.value['LastMessage']}\' where ChatGroupId = \'${event.snapshot.value['ChatGroupId']}\'');
          } else {
            result = await db.execute(
                'Update ChatGroup set SendAt=\'${event.snapshot.value['LastMessageTime']}\',Name=\'${event.snapshot.value['ChatGroupName']}\',Message=\'${event.snapshot.value['LastMessage']}\',SendAtTime=\'${event.snapshot.value['LastUpdate']}\' where ChatGroupId = \'${event.snapshot.value['ChatGroupId']}\'');
          }
        } else if (event.snapshot.value['UpdateContact'] != null) {
          var chatGroupMember = await db.get(ChatGroupMember(),
              'Select * from ChatGroupMember WHERE userid =\'${event.snapshot.value['UpdateContact']}\' group BY chatgroupid having count(chatgroupid) = 1 limit 1');
          if (chatGroupMember != null && chatGroupMember.length > 0) {
            result = await db.execute(
                'Update ChatGroup set ChatGroupId = \'${event.snapshot.value['ChatGroupId']}\',SendAt=\'${event.snapshot.value['LastMessageTime']}\',Name=\'${event.snapshot.value['ChatGroupName']}\',Message=\'${event.snapshot.value['LastMessage']}\',SendAtTime=\'${event.snapshot.value['LastUpdate']}\' where ChatGroupId = \'${chatGroupMember[0].ChatGroupId}\'');
            result = db.execute(
                'Update ChatGroupMember set ChatGroupId = \'${event.snapshot.value['ChatGroupId']}\' where ChatGroupId = \'${chatGroupMember[0].ChatGroupId}\'');
          }
        } else {
          if (!event.snapshot.value['IsRemoved'] && event.snapshot.value['GroupType'] != 'Fixed') {
            ChatGroup chatGroup = ChatGroup();
            chatGroup.ChatGroupId = event.snapshot.value['ChatGroupId'];
            chatGroup.CreatedAt = '';
            chatGroup.Message = event.snapshot.value['LastMessage']; //ChatGroupName
            chatGroup.Name = event.snapshot.value['ChatGroupName'];
            chatGroup.SendAt = event.snapshot.value['LastMessageTime'];
            chatGroup.SendAtTime = event.snapshot.value['LastUpdate'];
            rf.db.save(chatGroup);
          }
        }
        if (onMessageReceive != null) onMessageReceive(event.snapshot.value);
      });
    }
  }

  addFcmListener() {
    FlutterLocalNotificationsPlugin fltNotification;
    var androiInit = AndroidInitializationSettings('ic_launcher');

    var iosInit = IOSInitializationSettings();

    var initSetting = InitializationSettings(android: androiInit, iOS: iosInit);

    fltNotification = FlutterLocalNotificationsPlugin();

    fltNotification.initialize(initSetting);
    FirebaseMessaging.onMessage.listen((RemoteMessage message) {
      RemoteNotification notification = message.notification;
      AndroidNotification android = message.notification?.android;
      if (notification != null && android != null) {
        notify(notification.title, notification.body);
      }
    });
  }

  getFcmTocken() async {
    String token = await FirebaseMessaging.instance.getToken();
    return token;
  }

  Flushbar fb;

  Flushbar<bool> msg(
    BuildContext context,
    String text, {
    MessageType type = MessageType.info,
    IconData icon,
    Color color = Colors.black87,
    Color backgroundColor = Colors.redAccent,
    String actionText = '',
    Function action,
    Duration duration = const Duration(seconds: 2),
  }) {
    fb = Flushbar<bool>(
      //title:  null,
      messageText: Padding(
        padding: const EdgeInsets.only(left: 20),
        child: TextRF(
          text,
          size: 20.0.h,
          style: TextStyle(
            fontWeight: FontWeight.w500,
            color: color,
            fontFamily: "ShadowsIntoLightTwo",
          ),
        ),
      ),
      duration: duration,
      margin: EdgeInsets.all(10),
      padding: EdgeInsets.all(8),
      borderRadius: 50,
      backgroundColor: type == MessageType.info
          ? Colors.pink[200]
          : type == MessageType.warning
              ? Colors.deepOrange
              : type == MessageType.error
                  ? Colors.red
                  : type == MessageType.success
                      ? Colors.green
                      : Colors.lightBlueAccent,
      icon: Icon(
        icon ?? Icons.info,
        color: Colors.white,
        size: 52,
      ),
      mainButton: actionText.isNotEmpty
          ? Padding(
              padding: const EdgeInsets.only(right: 6.0),
              child: FlatButton(
                shape: new RoundedRectangleBorder(borderRadius: new BorderRadius.circular(50.0)),
                color: Colors.white,
                onPressed: () {
                  fb.dismiss(true);
                },
                child: TextRF(
                  actionText,
                  size: 20.0.h,
                  style: TextStyle(
                    color: Colors.black,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
            )
          : null,
    )..show(context).then((value) {
        if (action != null) if (value != null) if (value) action();
      });
    return fb;
  }
}

var pageCaller;
var pageCalled;

class PageAction {
  Future call(
    BuildContext context,
    dynamic pageToCall, {
    bool refreshCaller = false,
    TransitionTypeRF pageTransition,
  }) async {
    var callerStatusBarColor = await FlutterStatusbarcolor.getStatusBarColor();
    var callerNavigationBarColor = await FlutterStatusbarcolor.getNavigationBarColor();

    PageTransitionType getPageTransitionType() {
      PageTransitionType type;
      if (pageTransition == null) {
        if (Libs.config.page.transition == null) {
          type = PageTransitionType.bottomToTop;
        } else {
          type = PageTransitionType.values[Libs.config.page.transition.index];
        }
      } else {
        type = PageTransitionType.values[pageTransition.index];
      }
      return type;
    }

    //PageRF.parentContext = pageToCall.toString();
    var callerClassName = context;
    //PageRF.currentPageClassName = pageToCall.toString();

    pageCaller = context.widget;
    pageCalled = pageToCall;

    Navigator.push(
      context,
      PageTransition(curve: Curves.linear, type: getPageTransitionType(), child: pageToCall),
    ).then((value) async {
      SystemRF.os.setOsBarColors([callerStatusBarColor, callerNavigationBarColor]);
      PageRF.refresh(context);

      // if (refreshCaller) {
      //   if (PageRF.onCallerRefreshMap != null) {
      //     var className = callerClassName.widget.toString();
      //     var func = PageRF.onCallerRefreshMap[className];
      //     if (func != null) {
      //       await func();
      //     }
      //   }
      // }
      // PageRF.currentPageClassName = callerClassName;
    });
  }

  back(BuildContext context, {model}) {
    PageRF.keyboardOpened = false;
    PageRF.willPop(context, model: model);
  }

  replace(BuildContext context, dynamic page) {
    return Navigator.pushReplacement(
      context,
      new MaterialPageRoute(builder: (context) => page),
    );
  }

  Route _createRoute(dynamic page) {
    return PageRouteBuilder(
      pageBuilder: (context, animation, secondaryAnimation) => page,
      transitionDuration: Duration(milliseconds: 250),
      maintainState: false,
      transitionsBuilder: (context, animation, secondaryAnimation, child) {
        var begin = Offset(0.0, 1.0);
        var end = Offset.zero;
        var curve = Curves.ease;

        var tween = Tween(begin: begin, end: end).chain(CurveTween(curve: curve));

        return SlideTransition(
          position: animation.drive(tween),
          child: child,
        );
      },
    );
  }
}

class ModelObject {
  dynamic _model;

  ModelObject(dynamic model) {
    _model = model;
  }

  bool changed() {
    return Dynamic(_model).changed1();
  }
}

class Res {
  String text(String model, String name) {
    return LocalizationRF.instance.text(model, name);
  }
}

class App {
  void restart(BuildContext context) {
    RestartRF.restartApp(context);
  }
}

class Mapper {
  Object jsonToModel(Object obj, Map<String, dynamic> json) {
    Dynamic dynamic = Dynamic(obj);
    dynamic.dataMapToModel(json);
    return obj;
  }
}

class ScreenRF {
  static height(BuildContext context) {
    return MediaQuery.of(context).size.height;
  }

  static width(BuildContext context) {
    return MediaQuery.of(context).size.width;
  }

  static getWidth() {
    return MediaQuery.of(rf.context).size.width;
  }

  static listViewHeight(DataSourceRF dataSource, {double cardHeight = 120.0, double maxHeight = 700.0}) {
    var height = (dataSource.modelList.length * cardHeight) == 0.0
        ? cardHeight
        : (dataSource.modelList.length * cardHeight) + 10;
    if (height > maxHeight) height = maxHeight;
    return height;
  }
}
