import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:dio/dio.dart';
import 'package:ext_storage/ext_storage.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:progress_dialog/progress_dialog.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'dart:io';
import 'package:connectivity/connectivity.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:geolocator/geolocator.dart';
import 'package:rapidfire/Extensions.dart';
import 'package:workmanager/workmanager.dart';
import 'package:flutter_statusbarcolor/flutter_statusbarcolor.dart';
import 'JobRF.dart';
import 'GlobalScopeRF.dart';
import 'package:flutter/services.dart';
import 'package:http/http.dart' as http;

class SystemRF {
  static NetworkRF network = NetworkRF();
  static GPSRF gps = GPSRF();
  static RegisterJobRF job = RegisterJobRF();
  static NotificationRF notification = NotificationRF();
  static Os os = Os();

  static hideKeyboard(BuildContext context) {
    FocusScope.of(context).requestFocus(FocusNode());
  }

  static Future<Map<String, dynamic>> deviceInfo() async {
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    var deviceData = <String, dynamic>{};
    if (Platform.isAndroid) {
      AndroidDeviceInfo build = await deviceInfo.androidInfo;
      deviceData = {
        'brand': build.brand,
        'device': build.device,
        'model': build.model,
        'product': build.product,
        'manufacturer': build.manufacturer,
        'isPhysicalDevice': build.isPhysicalDevice,
        'version.securityPatch': build.version.securityPatch,
        'version.sdkInt': build.version.sdkInt,
        'version.release': build.version.release,
        'version.previewSdkInt': build.version.previewSdkInt,
        'version.incremental': build.version.incremental,
        'version.codename': build.version.codename,
        'version.baseOS': build.version.baseOS,
        'board': build.board,
        'bootloader': build.bootloader,
        'display': build.display,
        'fingerprint': build.fingerprint,
        'hardware': build.hardware,
        'host': build.host,
        'id': build.id,
        'supported32BitAbis': build.supported32BitAbis,
        'supported64BitAbis': build.supported64BitAbis,
        'supportedAbis': build.supportedAbis,
        'tags': build.tags,
        'type': build.type,
        'androidId': build.androidId,
        'systemFeatures': build.systemFeatures,
      };
      print('Running on ${build.model}');
    } else if (Platform.isIOS) {
      IosDeviceInfo data = await deviceInfo.iosInfo;
      deviceData = {
        'name': data.name,
        'systemName': data.systemName,
        'systemVersion': data.systemVersion,
        'model': data.model,
        'localizedModel': data.localizedModel,
        'identifierForVendor': data.identifierForVendor,
        'isPhysicalDevice': data.isPhysicalDevice,
        'utsname.sysname:': data.utsname.sysname,
        'utsname.nodename:': data.utsname.nodename,
        'utsname.release:': data.utsname.release,
        'utsname.version:': data.utsname.version,
        'utsname.machine:': data.utsname.machine,
      };
      print('Running on ${data.utsname.machine}');
    }
    return deviceData;
  }
}

class Storage {
  getDownloadFolderPath() async {
    return await ExtStorage.getExternalStoragePublicDirectory(ExtStorage.DIRECTORY_DOWNLOADS);
  }

  Future<bool> requestPermission() async {
    var status = await Permission.storage.status;
    if (status.isDenied) {
      return await Permission.storage.request().isGranted;
    }
    return status.isGranted;
  }
}

enum StorageLocation { appStorage, externalStorage }

class HttpRF {
  ProgressDialog pdlg;

  Future downloadFile({
    BuildContext context,
    String url,
    String downloadFolder,
    StorageLocation downloadLocation = StorageLocation.externalStorage,
    Function onStart,
    Function onProgress,
    Function onFinish,
    Function onError,
  }) async {
    String baseLocation;
    String fileName = url.substring(url.lastIndexOf("/") + 1);
    if (downloadLocation == StorageLocation.externalStorage)
      baseLocation = await Storage().getDownloadFolderPath();
    else {
      final Directory appDir = await getApplicationDocumentsDirectory();
      baseLocation = appDir.path;
    }

    String fullPath = downloadFolder == null ? "$baseLocation/" + fileName : "$baseLocation/$downloadFolder/" + fileName;

    if (context != null) {
      pdlg = ProgressDialog(context, type: ProgressDialogType.Download, isDismissible: true);
      pdlg.update(message: 'Downloading ($fileName)...');
    }

    try {
      if (onStart == null) {
        if (pdlg != null) pdlg.show();
      } else
        onStart();

      var dio = Dio();

      var ret = await Storage().requestPermission();
      if (ret) {
        Response response = await dio.download(
          url,
          fullPath,
          onReceiveProgress: (int count, int total) {
            if (total != -1) {
              double downloaded = (count / total * 100);

              if (onProgress == null) if (pdlg != null)
                pdlg.update(progress: downloaded.roundToDouble(), maxProgress: 100);
              else
                onProgress(downloaded);

              print(downloaded.toString() + "%");
              if (downloaded.toString() == '100.0') {
                if (onFinish == null) {
                  if (pdlg.isShowing()) pdlg.hide();
                } else {
                  if (pdlg.isShowing()) pdlg.hide();
                  onFinish();
                }
              }
            }
          },
//Received data with List<int>
          options: Options(
              responseType: ResponseType.bytes,
              followRedirects: false,
              validateStatus: (status) {
                return status < 500;
              }),
        );
        print(response.headers);
      }
    } catch (e) {
      print('Error:' + e.toString());
      if (onError == null) {
        if (pdlg.isShowing()) pdlg.hide();
      } else {
        if (pdlg.isShowing()) pdlg.hide();
        onError(e);
      }
    }
  }

  Future<http.StreamedResponse> uploadFile({String url, File file, Function onComplete, Function onError}) async {
    var request = http.MultipartRequest('POST', Uri.parse(url));
    request.files.add(await http.MultipartFile.fromPath('File', file.path));

    http.StreamedResponse response = await request.send();

    if (response.statusCode == 200) {
      String resposeString = await response.stream.bytesToString();
      print(resposeString);
      onComplete(resposeString);
    } else {
      print(response.reasonPhrase);
      onError(response.reasonPhrase);
    }
    return response;
  }
}

class NetworkRF {
  Future<bool> checkWifi() async {
    bool iswifi = false;
    var connectivityResult = await (Connectivity().checkConnectivity());
    if (connectivityResult == ConnectivityResult.wifi) {
// I am connected to a wifi network.
      iswifi = true;
    }
    return iswifi;
  }

  Future<bool> hasInternet() async {
    bool isConnected = false;
    isConnected = await DataConnectionChecker().hasConnection;
/*    try {
      final result = await InternetAddress.lookup('google.com');
      if (result.isNotEmpty && result[0].rawAddress.isNotEmpty) {
        isConnected = true;
      }
    } on SocketException catch (_) {
      isConnected = false;
    }*/
    return isConnected;
  }
}

class GPSRF {
  Future<bool> gpsStatus() async {
    bool isLocationEnabled = await Geolocator.isLocationServiceEnabled();
    return isLocationEnabled;
  }
}

class RegisterJobRF {
  killJob(String jobTitle) {}

  register(JobRF1 jobRF) async {
    WidgetsFlutterBinding.ensureInitialized();
    await Workmanager.initialize(jobRF.job, isInDebugMode: false); //to true if still in testing lev turn it to false whenever you are launching the app
    await Workmanager.registerOneOffTask("6", jobRF.name);
  }

//  register(JobRF jobRF) async {
//    //Workmanager.cancelAll();
//   //WidgetsFlutterBinding.ensureInitialized();
////    Workmanager.cancelByUniqueName(jobRF.name);
////    await Workmanager.initialize(jobRF.job, isInDebugMode: false);
////    //to true if still in testing lev turn it to false whenever you are launching the app
////    await Workmanager.registerOneOffTask("6", jobRF.name);
//  }
}

class Os {
  setOsBarColors([List<Color> colors]) async {
    if (Libs.osBarColor.length == 0) {
      Libs.osBarColor.add(await FlutterStatusbarcolor.getStatusBarColor());
      Libs.osBarColor.add(await FlutterStatusbarcolor.getNavigationBarColor());
    }
    if (colors != null) {
      if (colors.length > 0)
        changeOsBarColor(colors);
      else
        changeOsBarColor(Libs.osBarColor);
    } else {
      changeOsBarColor(Libs.osBarColor);
    }
  }

  changeOsBarColor(List<Color> colors) async {
    bool _useWhiteStatusBarForeground;
    bool _useWhiteNavigationBarForeground;
    try {
      await FlutterStatusbarcolor.setStatusBarColor(colors[0], animate: true);
      await FlutterStatusbarcolor.setNavigationBarColor(colors[1], animate: true);
      if (useWhiteForeground(colors[0])) {
        await FlutterStatusbarcolor.setStatusBarWhiteForeground(true);
        await FlutterStatusbarcolor.setNavigationBarWhiteForeground(true);
        _useWhiteStatusBarForeground = true;
        _useWhiteNavigationBarForeground = true;
      } else {
        if (colors[0] == Colors.white) {
          Future.delayed(Duration(seconds: 1)).then((_) {
            FlutterStatusbarcolor.setStatusBarWhiteForeground(false);
            FlutterStatusbarcolor.setNavigationBarWhiteForeground(false);
            print('white fix');
          });
        } else {
          FlutterStatusbarcolor.setStatusBarWhiteForeground(false);
          FlutterStatusbarcolor.setNavigationBarWhiteForeground(false);
        }

        _useWhiteStatusBarForeground = false;
        _useWhiteNavigationBarForeground = false;
      }
    } on PlatformException catch (e) {
      debugPrint(e.toString());
    }
  }
}

//class JobRF {
//  String name;
//  int period;
//  Function job;
//
//  JobRF({this.name, this.period, this.job});
//}

class NotificationRF {
  notify(NotifyRF notifyRF) async {
    FlutterLocalNotificationsPlugin flp = FlutterLocalNotificationsPlugin();
    var android = AndroidInitializationSettings('@mipmap/ic_launcher');
    var iOS = IOSInitializationSettings();
    var initSetttings = InitializationSettings(android: android, iOS: iOS);
    flp.initialize(initSetttings);
    var androiddetails = AndroidNotificationDetails('channel id', 'channel NAME', 'CHANNEL DESCRIPTION', priority: Priority.high, importance: Importance.max);
    var iOSdetails = IOSNotificationDetails();
    var platform = NotificationDetails(android: androiddetails, iOS: iOSdetails);
    await flp.show(0, notifyRF.title, notifyRF.description, platform, payload: 'VIS \n notifyRF.description');
  }
}

class NotifyRF {
  String title;
  String description;
  String icon;
  dynamic data;

  NotifyRF({this.title, this.description, this.icon, this.data});
}

//region IncrementRF
enum IncrementType { horizontal, vertical }

class IncrementRF extends StatefulWidget {
  final IncrementType incrementType;

  IncrementRF({this.incrementType = IncrementType.horizontal});

  @override
  _IncrementRFState createState() => _IncrementRFState();
}

class _IncrementRFState extends State<IncrementRF> {
  int counter = 0;

  @override
  Widget build(BuildContext context) {
    switch (widget.incrementType) {
      case IncrementType.vertical:
        return Container(
          child: Row(
            children: <Widget>[
              Container(
                  width: 100,
                  height: 30,
                  margin: EdgeInsets.all(5),
                  decoration: BoxDecoration(
                    color: Colors.white,
                    border: Border.all(width: 1.0, color: Colors.blue),
                    borderRadius: BorderRadius.all(Radius.circular(5.0) //                 <--- border radius here
                        ),
                  ),
                  child: Center(
                      child: Text(
                    counter.toString(),
                    style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold, fontSize: 20),
                  ))),
              Column(
                children: <Widget>[
                  ButtonRF(
                      buttonType: ButtonType.raised,
                      buttonStyle: ButtonStyleRF.info,
                      text: Text("+"),
                      borderRadious: 50,
                      onTap: () {
                        counter++;
                        setState(() {});
                      },
                      width: 40,
                      height: 40),
                  ButtonRF(
                    buttonType: ButtonType.raised,
                    buttonStyle: ButtonStyleRF.info,
                    text: Text(
                      "-",
                      style: TextStyle(fontSize: 20),
                    ),
                    borderRadious: 50,
                    onTap: () {
                      counter--;
                      setState(() {});
                    },
                    width: 40,
                    height: 40,
                  )
                ],
              ),
            ],
          ),
        );
        break;
      case IncrementType.horizontal:
        return Container(
          child: Row(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              ButtonRF(
                  buttonType: ButtonType.raised,
                  buttonStyle: ButtonStyleRF.info,
                  text: Text("+"),
                  borderRadious: 50,
                  onTap: () {
                    counter++;
                    setState(() {});
                  },
                  width: 40,
                  height: 40),
              Container(
                  width: 100,
                  height: 40,
                  margin: EdgeInsets.all(5),
                  decoration: BoxDecoration(
                    color: 'f7f7f7'.toColor, //  border: Border.all(width: 1.0, color: Colors.blue),
                    borderRadius: BorderRadius.all(Radius.circular(5.0) //                 <--- border radius here
                        ),
                  ),
                  child: Center(
                      child: Text(
                    counter.toString(),
                    style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold, fontSize: 20),
                  ))),
              ButtonRF(
                buttonType: ButtonType.raised,
                buttonStyle: ButtonStyleRF.info,
                text: Text(
                  "-",
                  style: TextStyle(fontSize: 20),
                ),
                borderRadious: 50,
                onTap: () {
                  counter--;
                  setState(() {});
                },
                width: 40,
                height: 40,
              )
            ],
          ),
        );
        break;
    }
  }
}
//endregion

// Example
/*class Example {
  test() {
    var internet = SystemRF.network.internetStatus;
    var wifi = SystemRF.network.wifiStatus;
    var gps = SystemRF.gps.gpsStatus;
    SystemRF.job.register(
        JobRF(name: 'myJob', period: null, job: [])
    );
    SystemRF.notification.notify(NotifyRF (title: ""));
  }
}*/
