import 'dart:convert';
import 'dart:io';
import 'dart:math';
import 'package:flutter/material.dart';
import 'package:geolocator/geolocator.dart';
import 'package:intl/intl.dart';
import 'package:path_provider/path_provider.dart';
import 'package:progress_dialog/progress_dialog.dart';
import 'package:rapidfire/SystemRF.dart';
import 'package:uuid/uuid.dart';
import 'package:dio/dio.dart';
import 'package:http/http.dart' as http;
import 'package:mailer/mailer.dart' as mailer;
import 'package:mailer/smtp_server.dart' as smtp;

import 'Const.dart';
import 'GlobalScopeRF.dart';
import 'RapidFire.dart';

Type typeOfElementsInList<T>(List<T> e) => T;

class HexColor1 extends Color {
  HexColor1(final String hexColor) : super(_getColorFromHex(hexColor));

  static int _getColorFromHex(String hexColor) {
    hexColor = hexColor.toUpperCase().replaceAll('#', '');
    if (hexColor.length == 6) {
      hexColor = 'FF' + hexColor;
    }
    return int.parse(hexColor, radix: 16);
  }
}

class UtilRF {
  static TextTheme _buildTextTheme(TextTheme base) {
    const String fontName = 'WorkSans';
    return base.copyWith(
      headline1: base.headline1.copyWith(fontFamily: fontName),
      headline2: base.headline2.copyWith(fontFamily: fontName),
      headline3: base.headline3.copyWith(fontFamily: fontName),
      headline4: base.headline4.copyWith(fontFamily: fontName),
      headline5: base.headline5.copyWith(fontFamily: fontName),
      headline6: base.headline6.copyWith(fontFamily: fontName),
      button: base.button.copyWith(fontFamily: fontName),
      caption: base.caption.copyWith(fontFamily: fontName),
      bodyText1: base.bodyText1.copyWith(fontFamily: fontName),
      bodyText2: base.bodyText2.copyWith(fontFamily: fontName),
      subtitle1: base.subtitle1.copyWith(fontFamily: fontName),
      subtitle2: base.subtitle2.copyWith(fontFamily: fontName),
      overline: base.overline.copyWith(fontFamily: fontName),
    );
  }

  static ThemeData buildLightTheme() {
    final Color primaryColor = HexColor1('#54D3C2');
    final Color secondaryColor = HexColor1('#54D3C2');
    final ColorScheme colorScheme = const ColorScheme.light().copyWith(
      primary: primaryColor,
      secondary: secondaryColor,
    );
    final ThemeData base = ThemeData.light();
    return base.copyWith(
      colorScheme: colorScheme,
      primaryColor: primaryColor,
      buttonColor: primaryColor,
      indicatorColor: Colors.white,
      splashColor: Colors.white24,
      splashFactory: InkRipple.splashFactory,
      accentColor: secondaryColor,
      canvasColor: Colors.white,
      backgroundColor: const Color(0xFFFFFFFF),
      scaffoldBackgroundColor: const Color(0xFFF6F6F6),
      errorColor: const Color(0xFFB00020),
      buttonTheme: ButtonThemeData(
        colorScheme: colorScheme,
        textTheme: ButtonTextTheme.primary,
      ),
      textTheme: _buildTextTheme(base.textTheme),
      primaryTextTheme: _buildTextTheme(base.primaryTextTheme),
      accentTextTheme: _buildTextTheme(base.accentTextTheme),
      platform: TargetPlatform.iOS,
    );
  }

  static libImage(String path, String libName) {
    return Image.asset(AssetImage('${path}').assetName, package: '${libName}');
  }

  static String getDateTime() {
    DateFormat dateFormat = DateFormat("yyyy-MM-dd HH:mm:ss");
    String currentDateTime = dateFormat.format(DateTime.now());
    return currentDateTime;
  }

  static String getDate() {
    DateFormat dateFormat = DateFormat("yyyy-MM-dd");
    String currentDateTime = dateFormat.format(DateTime.now());
    return currentDateTime;
  }

  static String getTime({bool amPm = false}) {
    DateFormat dateFormat = amPm ? DateFormat("HH:mm a") : DateFormat("HH:mm:ss");
    String currentDateTime = dateFormat.format(DateTime.now());
    return currentDateTime;
  }

  static geoPosition() async {
    Position pos;
    pos = await Geolocator.getCurrentPosition();
    return pos;
  }

  static getUUID() {
    return Uuid().v1();
  }

  static String getInitials(String account_name) {
    String initials = "";
    if (account_name != null && account_name.trim().contains(" ")) {
      List<String> names = account_name.split(" ");

      int numWords = 2;

      if (numWords < names.length) {
        numWords = names.length;
      }
      for (var i = 0; i < numWords; i++) {
        initials += '${names[i][0]}';
        if (initials.length == 2) break;
      }
    } else {
      initials = account_name[0] + account_name[1];
    }

    return initials;
  }

  static dbBackUp(BuildContext context) async {
    String dbname = await Libs.pref.getString(ConstRF.dbName);
    String userName = await Libs.pref.getString('UserName');
    Map<String, dynamic> systemInfo = await SystemRF.deviceInfo();
    DialogRF(
      context,
      title: Libs.config.app.title,
      height: 120,
      avatarIcon: Icons.backup,
      avatarBackground: Colors.grey,
      blurAmount: 5,
      subTitle: "Database Backup",
      body: Container(
        margin: const EdgeInsets.symmetric(vertical: 20),
        child: Center(
            child: TextRF(
          "Do you want to backup your database ?",
          align: TextAlign.center,
          color: Colors.black,
          size: 18,
        )),
      ),
      type: DialogType.popup,
      hideDefaultControls: false,
      onAccept: () async {
        bool isInternet = await SystemRF.network.hasInternet();
        if (isInternet) {
          var pdlg = ProgressDialog(Libs.context, type: ProgressDialogType.Download, isDismissible: true);
          pdlg.update(message: 'Backuping DB ...');
          pdlg.show();
          var documentDirectory = await getApplicationDocumentsDirectory();
          String email = 'rapidfire.sci@gmail.com';
          String pass = '@abc@123@';
          String path = documentDirectory.path;
          path = path + '/' + dbname;
          print(path);

          final smtpServer = smtp.gmail(email, pass);

          String htmlData = '<h1>RF Backup Database</h1>\n<p>This is a backup database from RapidFire Mobile (a product of SCI BD IT)</p>';
          systemInfo.forEach((key, value) {
            htmlData = htmlData + '\n<p>${key} : ${value.toString()} </p>';
          });

          // Create our message.
          final message = mailer.Message()
            ..from = mailer.Address(email, Libs.config.app.title)
            ..recipients.add(Libs.config.db.backUpMail)
            ..subject = ' DB of ' + userName
            ..text = 'This is a backup database from RapidFire Mobile (a product of SCI BD IT)'
            ..attachments.add(mailer.FileAttachment(File(path)))
            ..html = htmlData ?? "<h1>RF Backup Database</h1>\n<p>This is a backup database from RapidFire Mobile (a product of SCI BD IT)</p>";

          try {
            final sendReport = await mailer.send(message, smtpServer);
            print('Message sent: ' + sendReport.toString());
            pdlg.hide();
          } on mailer.MailerException catch (e) {
            pdlg.hide();
            print('Message not sent.');
            for (var p in e.problems) {
              print('Problem: ${p.code}: ${p.msg}');
            }
          }
        } else {
          rf.msg(rf.context, 'No Internet connection');
        }
      },
    );
  }

  static getRandomColor() {
    List<Color> colors = [
      Colors.amber[700],
      Colors.green,
      Colors.purple,
      Colors.orange,
      Colors.red,
      Colors.blue,
      Colors.brown,
      Color(0xff71b4fb),
      Color(0xfffa8c73),
      Color(0xff8873f4),
      Color(0xff02c39a),
      Color(0xff70a288),
      Color(0xfff15bb5),
      Color(0xffedb183),
      Color(0xffa5668b),
      Color(0xff8873f4),
    ];
    Random random = new Random();
    int randomNumber = random.nextInt(colors.length);
    return colors[randomNumber];
  }

  static Future downloadFile(String url, String storePath, Function onStart, Function onProgress, Function onFinish, Function onError) async {
    try {
      onStart();
      var dio = Dio();
      Response response = await dio.download(
        url,
        storePath,
        onReceiveProgress: (int count, int total) {
          if (total != -1) {
            double downloaded = (count / total * 100);
            onProgress(downloaded);
            print(downloaded.toString() + "%");
            if (downloaded.toString() == '100.0') {
              onFinish();
            }
          }
        },
        //Received data with List<int>
        options: Options(
            responseType: ResponseType.bytes,
            followRedirects: false,
            validateStatus: (status) {
              return status < 500;
            }),
      );
      print(response.headers);
    } catch (e) {
      print('Error:' + e.toString());
      onError();
    }
  }

  static Future<http.StreamedResponse> uploadFile({String url, File file, Function onComplete, Function onError}) async {
    var request = http.MultipartRequest('POST', Uri.parse(url));
    request.files.add(await http.MultipartFile.fromPath('File', file.path));

    http.StreamedResponse response = await request.send();

    if (response.statusCode == 200) {
      String resposeString = await response.stream.bytesToString();
      print(resposeString);
      onComplete(resposeString);
    } else {
      print(response.reasonPhrase);
      onError(response.reasonPhrase);
    }
    return response;
  }
}
