import 'package:flutter/material.dart';
import 'package:rapidfire/Annotation.dart';
import 'Dynamic.dart';
import 'GlobalScopeRF.dart';

class Required {
  final String msg;

  const Required({this.msg});
}

class Range {
  final double start;
  final double end;
  final String msg;

  const Range({this.start, this.end, this.msg});
}

class Max {}

class Email {}

class Date {}

enum CompareType { min, max, equal }

class Compare {
  final CompareType type;
  final dynamic compareWith;
  final dynamic target;
  final dynamic value;

  const Compare({this.type, this.compareWith, this.target, this.value});
}

class Custom {
  final fn;
  final String msg;

  const Custom({this.fn, this.msg});
}

class ValidationDataRF {
  bool isValid;
  String msg;

  ValidationDataRF({this.isValid, this.msg});
}

class ValidationRF {
  static String msg(String field) {
    return Libs.modelValidationData[field].msg;
  }

  static Widget showErrorMessage(String field) {
    bool valid = false;
    if (Libs.modelValidationData[field] == null) {
      valid = true;
    } else {
      valid = Libs.modelValidationData[field].isValid;
      //Libs.modelValidationData[field].isValid = true;
    }
    if (valid) {
      return Text('');
    } else {
      return Padding(
        padding: const EdgeInsets.only(top: 5, bottom: 15),
        child: Text(
          ValidationRF.msg(field),
          style: TextStyle(color: Colors.red),
        ),
      );
    }
  }

  static List<Map> validationAnnotations;

  static ValidationDataRF validate(String field, Object model, dynamic value) {
    ValidationDataRF validationDataRF;

    var check = Dynamic(model).annotationData(Check); //validationAnnotations; //

    check.forEach((element) {
      for (var validation in element['data']['validations']) {
        if (element['field'] == field) {
          validationDataRF = validateField(element['field'], field, value, validation, model);
        } else {
          return;
        }
      }
      if (validationDataRF != null) return;
    });
    return validationDataRF;
  }

  // by imrose today
  // if (validation is Required) {
  //   if (validationDataRF != null) break; //?
  //   if (element['field'] == field) {
  //     validationDataRF = validateField(element['field'], field, value, validation);
  //   } else
  //     return;
  // }

  // Place to validation logic #############
  static ValidationDataRF validateField(
    String modelField,
    String checkField,
    dynamic checkFieldValue,
    dynamic validation,
    Object model,
  ) {
    ValidationDataRF validationDataRF;

    if (validation is Required) {
      if (modelField == checkField) {
        if (checkFieldValue == null) {
          validationDataRF = ValidationDataRF(
            isValid: false,
            msg: validation.msg ?? '${checkField} is required.',
          );
        }
        if (checkFieldValue is String) {
          if (checkFieldValue.isEmpty) {
            validationDataRF = ValidationDataRF(
              isValid: false,
              msg: validation.msg ?? '${checkField} is required.',
            );
          }
        }
      }
    } else if (validation is Range) {
      var start = validation.start;
      var end = validation.end;
      if (modelField == checkField) {
        // if (checkFieldValue >= start && checkFieldValue <= end) {
        // } else {
        //   validationDataRF = ValidationDataRF(
        //     isValid: false,
        //     msg: validation.msg ?? '${checkField} is required.',
        //   );
        // }
      }
    } else if (validation is Custom) {
      if (modelField == checkField) {
        var ret = validation.fn(model);
        if (ret != null || ret) {
          validationDataRF = ValidationDataRF(
            isValid: false,
            msg: validation.msg ?? '${checkField} has custom logic error.',
          );
        }
      }
    }

    // return validationDataRF is structure to show  error
    return validationDataRF;
  }
}

/*

CreditCard
This validates that the property has a credit card format.
Compare
This attribute validates that two property in model class match like password and compare password.
EmailAddress
This validates the property has email address format.
Phone
This validates that the property has a telephone number format.
Range
This validates that the property value within a specified range.
RegularExpression
This validates that the property value matches a specified regular expression.
Required
This validates that the field is not null
StringLength
This validates that a string property value doesn't exceed a specified length limit.
Url
This validates that the property has a URL format.
Remote
This validates input on the client by calling an action method on the server

*/
