import 'dart:convert';
import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:rapidfire/chatting/ChatGroupsPage.dart';
import 'package:rapidfire/chatting/ChatPage.dart';
import 'package:rapidfire/models/ChatModels.dart';

import '../GlobalScopeRF.dart';
import '../RapidFire.dart';

class ChatRf {
  BuildContext context;
  Color primaryColor;
  Color secondaryColor;
  Color chatHeadColor;
  double bubbleRadious;
  bool showArrow;
  bool allowModification;
  String userId;
  String jobName;
  Function dispatcher;

  start(
    BuildContext context, {
    Color primaryColor,
    Color secondaryColor,
    Color chatHeadColor,
    double bubbleRadious = 5,
    bool showArrow,
    String userId,
    String jobName,
    Function dispatcher,
    bool allowModification,
    bool gotoChatScrreen = false,
    String chatGroupId,
  }) {
    this.context = context;
    this.primaryColor = primaryColor;
    this.secondaryColor = secondaryColor;
    this.chatHeadColor = chatHeadColor;
    this.bubbleRadious = bubbleRadious;
    this.allowModification = allowModification;
    this.showArrow = showArrow;
    this.userId = userId;
    this.jobName = jobName;
    this.dispatcher = dispatcher;
    if (gotoChatScrreen) {
      if (chatGroupId != null && chatGroupId.isNotEmpty)
        callChatPage(chatGroupId: chatGroupId, userId: userId, fromGroupPage: false);
      else
        rf.msg(context, 'GroupId not Found');
    } else
      callChatgroupPage();
  }

  callChatgroupPage({bool replace = false}) {
    if (replace) {
      rf.page.replace(
          context,
          ChatGroupPage(
            primaryColor: primaryColor,
            secondaryColor: secondaryColor,
            chatHeadColor: chatHeadColor,
            bubbleRadious: bubbleRadious,
            jobName: jobName,
            dispatcher: dispatcher,
          ));
    } else {
      rf.page.call(
          context,
          ChatGroupPage(
            primaryColor: primaryColor,
            secondaryColor: secondaryColor,
            chatHeadColor: chatHeadColor,
            bubbleRadious: bubbleRadious,
            jobName: jobName,
            dispatcher: dispatcher,
          ));
    }
  }

  callChatPage({
    String chatGroupId,
    String userId,
    bool replace = false,
    bool fromGroupPage = false,
  }) {
    if (replace) {
      rf.page.replace(
          context,
          ChatPage(
            chatGroupId,
            userId,
            primaryColor: primaryColor,
            secondaryColor: secondaryColor,
            chatHeadColor: chatHeadColor,
            bubbleRadious: bubbleRadious,
            fromGroupPage: fromGroupPage,
            hasModifyPermission: allowModification,
          ));
    } else {
      rf.page.call(
          context,
          ChatPage(
            chatGroupId,
            userId,
            primaryColor: primaryColor,
            secondaryColor: secondaryColor,
            chatHeadColor: chatHeadColor,
            bubbleRadious: bubbleRadious,
            fromGroupPage: fromGroupPage,
            hasModifyPermission: allowModification,
          ));
    }
  }

  fetchChatGroups() {
    if (jobName != null)
      rf.api.fetchModel(
        context: context,
        serviceName: jobName,
        dispatcher: dispatcher,
        modelsName: 'ChatGroup',
        showDialog: false,
        showNotification: false,
      );
  }

  Future<List<ChatMessage>> getMessages(String groupId, {String sendAt = '', String userId}) async {
    var baseUrl = Libs?.config ?? new Configuration();
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Fetch/GetPacket';
    var fetchRequest = {
      "UserId": 0,
      "TableName": "ChatMessage",
      "Command": "GetData",
      "TotalRecord": 0,
      "UserGuid": userId,
      "PageNo": 1,
      "PageSize": 20,
      "ApiPacket": {},
    };

    var where = sendAt.isEmpty ? "SendTo='${groupId}'" : "SendTo='${groupId}' and SendAt<'${sendAt}'";

    fetchRequest['ApiPacket'] = {
      "Packet": {
        "ModelName": "ChatMessage",
        "Select": "",
        "Where": where,
        "OrderBy": "SendAt DESC",
        "IncludeChild": true,
      },
      "PacketList": null
    };

    print('Request: ' + json.encode(fetchRequest).toString());
    var response = await rf.api.httpPost(
      url: apiBaseURL,
      requestBody: json.encode(fetchRequest),
    );
    print(response.body);
    var jsonData = jsonDecode(response.body);
    if (jsonData['ApiPacket']['PacketList'] != null) {
      var data = jsonData['ApiPacket']['PacketList'] as List;
      print('Total Record : ${data.length}');
      List<ChatMessage> messages = [];
      List<String> seens = [];
      for (var row in data) {
        var seenData = row['ChatSeenUser'] as List;
        var m = ChatMessage();
        await Dynamic(m).dataMapToModel(row);
        m.ChatSeenUser = [];
        for (dynamic d in seenData) {
          if (!seens.contains(d['UserId'])) {
            seens.add(d['UserId']);
            m.ChatSeenUser.add(d['UserId']);
          }
        }
        messages.add(m);
      }

      if (messages.length == data.length) {
        return messages;
      } else
        return null;
    } else
      return null;
  }

  Future sendMessage(String groupId, String userId, String msg) async {
    //region RDB
    ChatMessage chatMessage = ChatMessage(
      ChatMessageId: UtilRF.getUUID(),
      Message: msg,
      SendAt: DateFormat('hh:mm a').format(DateTime.now()),
      Sender: userId,
      SendTo: groupId,
      SendAtText: UtilRF.getTime(amPm: true),
    );

    FirebaseDatabase.instance.reference().child('messages/${groupId}').set(Dynamic(chatMessage).modelToDataMap());
    //endregion

    //region Server
    var baseUrl = Libs?.config ?? new Configuration();
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Sync/SendPacket';

    var syncRequest = {
      "ApiPacket": {
        "Packet": {"ChatMessageId": chatMessage.ChatMessageId, "Message": "${msg}", "Sender": "${userId}", "SendTo": "${groupId}", "SendAtText": "${chatMessage.SendAtText}"}
      },
      "Command": "add",
      "PageNo": 0,
      "PageSize": 0,
      "TableName": "ChatMessage",
      "TotalRecord": 0,
      "UserId": 0,
      "UserGuid": "${userId}"
    };

    var response = await rf.api.httpPost(
      url: apiBaseURL,
      requestBody: json.encode(syncRequest),
    );
    var result = rf.db.execute('Update ChatGroup set Message=\'${msg}\' ,SendAt =\'${UtilRF.getDate()}\',SendAtTime =\'${UtilRF.getDateTime()}\' where ChatGroupId = \'${groupId}\'');
    print(response.body);
    //endregion
  }

  Future setSeen(String groupId, String userId, String chatMessageId) async {
    //region Server
    var baseUrl = Libs?.config ?? new Configuration();
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Sync/SendPacket';

    var syncRequest = {
      "ApiPacket": {
        "Packet": {"ChatMessageId": '${chatMessageId}', "UserId": "${userId}", "ChatGroupId": "${groupId}"}
      },
      "Command": "add",
      "PageNo": 0,
      "PageSize": 0,
      "TableName": "ChatSeenUser",
      "TotalRecord": 0,
      "UserId": 0,
      "UserGuid": "${userId}"
    };

    var response = await rf.api.httpPost(
      url: apiBaseURL,
      requestBody: json.encode(syncRequest),
    );
    print('setSeenResponse:' + response.body);
    //endregion
  }

  Future createGroup({ChatGroup user, String chatGroupId}) async {
    //region Server
    var baseUrl = Libs?.config ?? new Configuration();
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Sync/SendPacket';
    String uId = rf.pref.getString('UserGeoId');

    var syncRequest = {"ApiPacket": {}, "Command": "add", "PageNo": 0, "PageSize": 0, "TableName": "ChatGroup", "TotalRecord": 0, "UserId": 0, "UserGuid": "${uId}"};

    ChatGroup chatGroup = ChatGroup();
    //chatGroup.ChatGroupId = user.ChatGroupId;
    chatGroup.CreatedAt = UtilRF.getDateTime();
    chatGroup.ChatGroupMember = [];
    var members = await rf.db.get(ChatGroupMember, 'select * from ChatGroupMember where ChatGroupId =\'${user.ChatGroupId}\'');
    if (chatGroupId != null) chatGroup.ChatGroupId = chatGroupId;
    for (ChatGroupMember chatgoupMember in members) chatGroup.ChatGroupMember.add(ChatGroupMember(UserId: chatgoupMember.UserId, UserTable: chatgoupMember.UserTable, IsRemoved: false));
    chatGroup.ChatGroupMember.add(ChatGroupMember(UserId: uId, UserTable: rf.pref.getString('UserTable'), IsRemoved: false));

    //chatGroup.Name = user.UserName;
    var masterJson = Dynamic(chatGroup).modelToDataMap();
    var childModelDataList = [];
    for (var childRow in chatGroup.ChatGroupMember) {
      childRow = Dynamic(childRow).modelToDataMap();
      childModelDataList.add(childRow);
    }
    masterJson['ChatGroupMember'] = childModelDataList;
    syncRequest['ApiPacket'] = {"Packet": masterJson, "PacketList": null};

    print('Group Req:' + jsonEncode(syncRequest));

    var response = await rf.api.httpPost(
      url: apiBaseURL,
      requestBody: json.encode(syncRequest),
    );
    print('Group Response:' + response.body);
    chatGroup.ChatGroupId = jsonDecode(response.body)['ServerRecordId'];

    if (user.ChatGroupId != null) {
      var result = await rf.db.execute('Update ChatGroup set ChatGroupId = \'${chatGroup.ChatGroupId}\' where ChatGroupId = \'${user.ChatGroupId}\'');
      result = rf.db.execute('Update ChatGroupMember set ChatGroupId = \'${chatGroup.ChatGroupId}\' where ChatGroupId = \'${user.ChatGroupId}\'');
    }

    return chatGroup;
    //endregion
  }

  Future addGroupMember({String userId, String userTable, String chatGroupId}) async {
    //region Server
    var baseUrl = Libs?.config ?? new Configuration();
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Sync/SendPacket';
    String uId = rf.pref.getString('UserGeoId');

    var syncRequest = {"ApiPacket": {}, "Command": "add", "PageNo": 0, "PageSize": 0, "TableName": "ChatGroupMember", "TotalRecord": 0, "UserId": 0, "UserGuid": "${uId}"};

    syncRequest['ApiPacket'] = {
      "Packet": {"ChatGroupId": "${chatGroupId}", "UserId": "${userId}", "UserTable": "${userTable}"},
      "PacketList": null
    };

    print('AddMember Req:' + jsonEncode(syncRequest));

    var response = await rf.api.httpPost(
      url: apiBaseURL,
      requestBody: json.encode(syncRequest),
    );
    print('AddMember Response:' + response.body);
    //endregion
  }

  Future removeGroupMember({String userId, String userTable, String chatGroupId}) async {
    //region Server
    var baseUrl = Libs?.config ?? new Configuration();
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Sync/SendPacket';
    String uId = rf.pref.getString('UserGeoId');

    var syncRequest = {"ApiPacket": {}, "Command": "add", "PageNo": 0, "PageSize": 0, "TableName": "ChatGroupMember", "TotalRecord": 0, "UserId": 0, "UserGuid": "${uId}"};

    syncRequest['ApiPacket'] = {
      "Packet": {"ChatGroupId": "${chatGroupId}", "UserId": "${userId}", "UserTable": "${userTable}", "IsRemoved": "true"},
      "PacketList": null
    };

    print('removeMember Req:' + jsonEncode(syncRequest));

    var response = await rf.api.httpPost(
      url: apiBaseURL,
      requestBody: json.encode(syncRequest),
    );
    print('removeMember Response:' + response.body);
    //endregion
  }

  Future<ChatGroup> getChatGroup(String groupId, String userId) async {
    var baseUrl = Libs?.config ?? new Configuration();
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Fetch/GetPacket';
    var fetchRequest = {
      "UserId": 0,
      "TableName": "ChatGroup",
      "Command": "GetData",
      "TotalRecord": 0,
      "UserGuid": userId,
      "PageNo": 1,
      "PageSize": 20,
      "ApiPacket": {},
    };

    fetchRequest['ApiPacket'] = {
      "Packet": {
        "ModelName": "ChatGroup",
        "Select": "${groupId}",
        "Where": "",
        "OrderBy": "SendAt DESC",
        "IncludeChild": true,
      },
      "PacketList": null
    };

    //print('Request: ' + json.encode(fetchRequest).toString());
    var response = await rf.api.httpPost(
      url: apiBaseURL,
      requestBody: json.encode(fetchRequest),
    );
    //print(response.body);
    var jsonData = jsonDecode(response.body);
    if (jsonData['ApiPacket']['PacketList'] != null) {
      var data = jsonData['ApiPacket']['PacketList'] as List;
      print('Total Record : ${data.length}');
      ChatGroup chatgroup = ChatGroup();
      chatgroup.ChatGroupMember = [];

      var members;
      for (var row in data) {
        members = row['ChatGroupMember'] as List;
        await Dynamic(chatgroup).dataMapToModel(row);

        for (dynamic d in members) {
          ChatGroupMember cm = ChatGroupMember();
          await Dynamic(cm).dataMapToModel(d);
          if (!cm.IsRemoved) chatgroup.ChatGroupMember.add(cm);
        }
      }
      return chatgroup;
    } else
      return null;
  }
}
