import 'package:flutter/material.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/models/ChatModels.dart';

import 'ChatPage.dart';

class ChatGroupPage extends StatefulWidget {
  Color primaryColor;
  Color secondaryColor;
  Color chatHeadColor;
  double bubbleRadious;
  bool showArrow;
  String jobName;
  Function dispatcher;

  @override
  _ChatGroupPageState createState() => _ChatGroupPageState();

  ChatGroupPage({this.primaryColor, this.secondaryColor, this.bubbleRadious, this.chatHeadColor, this.showArrow, this.dispatcher, this.jobName});
}

class _ChatGroupPageState extends State<ChatGroupPage> {
  DataSourceRF dataSource;
  int listType = 1;
  var userid;
  final textController = TextEditingController();

  @override
  void initState() {
    print('CHAT GROUP PAGE INIT');
    rf.onMessageReceive = (data) {
      dataSource.modelList.clear();
      setState(() {});
    };

    Api.onFetchCompleted = (model, count) {
      if (model == 'all fetch completed') {
        dataSource.modelList.clear();
        print("Printing on Complete: " + model + " " + count.toString());
        setState(() {});
      }
    };

    dataSource = DataSourceRF(modelList: List<ChatGroup>(), type: SourceType.local, where: " GroupType isnull  ", orderField: 'SendAtTime');

    super.initState();
  }

  @override
  void didChangeDependencies() async {
    print('CHAT GROUP PAGE DIDCHANGEDIPENDENCY');
    userid = await rf.pref.getString('UserGeoId');
  }

  @override
  void dispose() {
    Api.onFetchCompleted = null;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    print('CHAT GROUP PAGE BUILD');
    return Scaffold(
      backgroundColor: const Color(0xfff5f4ec),
      appBar: AppBar(
        automaticallyImplyLeading: false,
        backgroundColor: const Color(0xfff5f4ec),
        elevation: 0.0,
        title: Container(
          child: Stack(
            children: [
              Positioned(
                left: 0,
                child: InkWell(
                  onTap: () => rf.page.back(context), //rf.page.back(_context),
                  child: Container(child: Icon(Icons.arrow_back, color: Colors.black)),
                ),
              ),
              Center(
                child: TextRF(
                  'Chat',
                  size: 20,
                ),
              ),
              /* InkWell(
                onTap: () {
                  listType = 1;
                  dataSource = DataSourceRF(
                    modelList: List<ChatGroup>(),
                    type: DataSourceType.local,
                    orderField: 'SendAtTime',
                    externalWhereExp: " GroupType isnull ",
                  );
                  setState(() {});
                },
                child: Container(
                  padding: const EdgeInsets.only(left: 12, right: 12, top: 5, bottom: 5),
                  decoration: BoxDecoration(
                    color: listType == 1 ? widget.primaryColor ?? Colors.blue.withOpacity(0.7) : Colors.white,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    border: Border.all(color: listType == 1 ? Colors.black : widget.primaryColor ?? Colors.blue),
                  ),
                  child: TextRF(
                    'ChatList',
                    size: 18,
                    overflow: TextOverflow.ellipsis,
                    color: listType == 1 ? Colors.white : Colors.black,
                    weight: FontWeight.normal,
                  ),
                ),
              ),
              SizedBox(
                width: 10,
              ),
              InkWell(
                onTap: () {
                  listType = 2;
                  dataSource = DataSourceRF(modelList: List<ChatUsers>(), type: DataSourceType.local, orderField: 'UserName');
                  setState(() {});
                },
                child: Container(
                  padding: const EdgeInsets.only(left: 12, right: 12, top: 5, bottom: 5),
                  decoration: BoxDecoration(
                    color: listType == 2 ? widget.primaryColor ?? Colors.blue.withOpacity(0.7) : Colors.white,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    border: Border.all(color: listType == 2 ? Colors.black : widget.primaryColor ?? Colors.blue),
                  ),
                  child: TextRF(
                    'Contacts',
                    size: 18,
                    overflow: TextOverflow.ellipsis,
                    color: listType == 2 ? Colors.white : Colors.black,
                    weight: FontWeight.normal,
                  ),
                ),
              ),*/
            ],
          ),
        ),
      ),
      body: body(),
    );
  }

  body() {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Container(
          height: 50,
          width: ScreenRF.width(context),
          margin: const EdgeInsets.symmetric(horizontal: 10),
          decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(5)),
            boxShadow: <BoxShadow>[
              BoxShadow(
                offset: Offset(0, 2),
                blurRadius: 2,
                color: Colors.grey.withOpacity(.2),
              )
            ],
          ),
          child: Row(
            children: [
              SizedBox(
                width: 12,
              ),
              Icon(
                Icons.search_rounded,
                color: Colors.grey[400],
              ),
              SizedBox(
                width: 15,
              ),
              Expanded(
                child: Center(
                  child: TextField(
                    autofocus: false,
                    controller: textController,
                    decoration: InputDecoration(
                        border: InputBorder.none,
                        hintText: 'Search',
                        hintStyle: TextStyle(
                          color: Colors.grey[400],
                          fontWeight: FontWeight.w300,
                        )),
                    onChanged: (input) async {
                      print('SearchText:' + input.toString());
                      dataSource = DataSourceRF(
                          modelList: List<ChatGroup>(), type: SourceType.local, where: " Name like '%${input.toString()}%' and GroupType isnull  ", orderField: 'SendAtTime');
                      setState(() {});
                    },
                  ),
                ),
              ),
              InkWell(
                onTap: () {
                  textController.clear();
                  dataSource = DataSourceRF(modelList: List<ChatGroup>(), type: SourceType.local, where: " GroupType isnull  ", orderField: 'SendAtTime');
                  setState(() {});
                },
                child: Icon(
                  Icons.refresh,
                  color: Colors.grey[400],
                ),
              ),
              SizedBox(
                width: 15,
              ),
            ],
          ),
        ),
        // SizedBox(
        //   height: 15,
        // ),
        // Center(child: Image.asset('assets/images/chatlist.PNG')),
        SizedBox(
          height: 10,
        ),
        Expanded(
          child: Container(
            key: GlobalKey(),
            width: ScreenRF.width(context),
            margin: const EdgeInsets.symmetric(horizontal: 10),
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.all(Radius.circular(5)),
              boxShadow: <BoxShadow>[
                BoxShadow(
                  offset: Offset(0, 2),
                  blurRadius: 2,
                  color: Colors.grey.withOpacity(.2),
                )
              ],
            ),
            child: ListViewRF(
              dataSource: dataSource,
              builder: (model, index) => listType == 1 ? conversationCard(model, index) : userCard(model, index),
            ),
          ),
        ),
        SizedBox(
          height: 30,
        ),
      ],
    );
  }

  conversationCard(modelm, index) {
    ChatGroup model = modelm;
    return InkWell(
      onTap: () async {
        rf.chat.createGroup(user: model).then((value) {
          rf.onMessageReceive = null;
          rf.chat.callChatPage(chatGroupId: value.ChatGroupId, userId: userid, replace: true, fromGroupPage: true);
        });

        /* if (model.ChatGroupId == null || model.ChatGroupId.isEmpty) {

        } else {
          rf.onMessageReceive = null;
          rf.chat.callChatPage(chatGroupId: model.ChatGroupId, userId: userid, replace: true, fromGroupPage: true);
        }*/
      },
      child: Container(
        width: ScreenRF.width(context),
        margin: const EdgeInsets.only(top: 15, left: 15, right: 15),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(0)),
        ),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              mainAxisSize: MainAxisSize.min,
              children: [
                Container(
                  height: 45,
                  width: 45,
                  margin: const EdgeInsets.all(12),
                  decoration: BoxDecoration(
                    color: UtilRF.getRandomColor(),
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                  ),
                  child: Center(
                      child: Icon(
                    Icons.person,
                    color: Colors.white,
                  )),
                ),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      GapRF(5),
                      Row(
                        children: [
                          Expanded(
                            child: Container(
                              child: TextRF(
                                model.Name,
                                size: 18,
                                overflow: TextOverflow.ellipsis,
                                color: Colors.black,
                                weight: FontWeight.bold,
                              ),
                            ),
                          ),
                          Container(
                            child: TextRF(
                              model.SendAt ?? '',
                              size: 10,
                              overflow: TextOverflow.ellipsis,
                              color: Colors.grey,
                              weight: FontWeight.w400,
                            ),
                          ),
                        ],
                      ),
                      GapRF(5),
                      TextRF(
                        model.Message ?? 'no message sent yet',
                        size: 13,
                        color: Colors.grey,
                        weight: FontWeight.w400,
                      )
                    ],
                  ),
                ),
              ],
            ),
            SizedBox(
              height: 15,
            ),
            Container(
              width: ScreenRF.width(context),
              color: Colors.blue.shade50,
              height: 1.0,
            ),
          ],
        ),
      ),
    );
  }

  userCard(modelm, index) {
    ChatUsers model = modelm;
    return InkWell(
      onTap: () {
        /*rf.chat.createGroup(user: model).then((value) {
          rf.chat.callChatPage(chatGroupId: value.ChatGroupId, userId: userid, replace: true, fromGroupPage: true);
        });*/
      },
      child: Container(
        width: ScreenRF.width(context),
        margin: const EdgeInsets.all(10),
        padding: const EdgeInsets.all(10),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(0)),
          boxShadow: <BoxShadow>[
            BoxShadow(
              offset: Offset(0, 0),
              blurRadius: 5,
              color: Colors.grey.withOpacity(.15),
            )
          ],
        ),
        child: Stack(
          children: [
            Row(
              mainAxisSize: MainAxisSize.max,
              children: [
                Container(
                  height: 60,
                  width: 60,
                  margin: const EdgeInsets.all(12),
                  decoration: BoxDecoration(
                    color: widget.chatHeadColor ?? Colors.deepPurple,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    boxShadow: <BoxShadow>[
                      BoxShadow(
                        offset: Offset(0, 0),
                        blurRadius: 5,
                        color: widget.chatHeadColor.withOpacity(0.5) ?? Colors.deepPurple.withOpacity(.5),
                      )
                    ],
                  ),
                  child: Center(
                      child: TextRF(
                    UtilRF.getInitials(model.UserName),
                    size: 20,
                    color: Colors.white,
                    weight: FontWeight.bold,
                  )),
                ),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      GapRF(5),
                      Container(
                        width: 250,
                        child: TextRF(
                          model.UserName,
                          size: 18,
                          overflow: TextOverflow.ellipsis,
                          color: Colors.black,
                          weight: FontWeight.bold,
                        ),
                      ),
                      GapRF(5),
                      TextRF(
                        model.UserTable ?? 'no message sent yet',
                        size: 13,
                        color: Colors.grey,
                        weight: FontWeight.w400,
                      )
                    ],
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }
}
