import 'dart:async';
import 'dart:io';
import 'package:flutter/services.dart';
import 'package:http_server/http_server.dart';
import 'package:path_provider/path_provider.dart';
import 'package:rapidfire/util/ZipFileRF.dart';

class HttpServerRF {
  HttpServer _server;
  Directory _httpDirectory;

  String host = '';
  bool active = false;

  deploy(String zipFileName, Function callback) async {
    Directory appDir = await getApplicationDocumentsDirectory(); /// External Store ???
    String contentPath = appDir.path + '/scorm_store';


    final Directory httpDirectory = await Directory('${appDir.path}/www').create(recursive: true);
    final VirtualDirectory staticFiles = VirtualDirectory(httpDirectory.path)..allowDirectoryListing = true;
    _httpDirectory = httpDirectory;


    await ZipFileRF().unzip('$contentPath/$zipFileName', '${_httpDirectory.path}', 'content', false);
    await ZipFileRF().unzip('lib.zip', '${_httpDirectory.path}/', '', true);

    File indexFile = File('${_httpDirectory.path}/scorm12.html');
    String indexContent = await rootBundle.loadString('assets/scorm12.txt');
    await indexFile.writeAsString(indexContent);

    callback(host);
  }

  Future start([Function callback]) async {
    final Directory appDir = await getApplicationDocumentsDirectory();
    final Directory httpDirectory = await Directory('${appDir.path}/www').create(recursive: true);
    final VirtualDirectory staticFiles = VirtualDirectory(httpDirectory.path)..allowDirectoryListing = true;
    _httpDirectory = httpDirectory;

    await runZoned(() async {
      if (_server == null) {
        final HttpServer server = await HttpServer.bind(InternetAddress.loopbackIPv4, 3000);

        server.listen(staticFiles.serveRequest);

        _server = server;
        host = 'http://${server.address.host}:${server.port}';
        active = true;
        if (callback != null) callback(true);
      }
    }, onError: (e, stackTrace) {
      print('Error occured : $e $stackTrace');
    });
  }

  Future stop() async {
    await _server?.close();
    await _httpDirectory?.delete(recursive: true);
    active = false;
  }

  purgeContent() async {
    await _httpDirectory?.delete(recursive: true);
  }
}
