import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:rapidfire/http/HttpServerRF.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:webview_cookie_manager/webview_cookie_manager.dart';
import 'package:rapidfire/models/CMI.dart';

import '../RapidFire.dart';

class scorm {
  static CoreCMI coreCMI = CoreCMI();
  static HttpServerRF server = HttpServerRF();
}

enum ScormType { s12, s2004, aci }
enum PlayType { online, offline }

var _initialUrl;
final cookieManager = WebviewCookieManager();

var loadingTextStyle = TextStyle(
  fontSize: 15,
  color: Colors.black54,
  fontWeight: FontWeight.normal,
  decoration: TextDecoration.none,
);

var contentTextStyle = TextStyle(
  fontSize: 16,
  color: Colors.black54,
  fontWeight: FontWeight.bold,
  decoration: TextDecoration.none,
);

class ScormPlayerRF extends StatefulWidget {
  String studentId;
  String studentName;
  int port;
  String content;
  String contentId;
  Function onInteractions;
  ScormType scormType;
  PlayType playType;
  bool displayHorizontal;

  ScormPlayerRF({
    this.content,
    this.contentId,
    this.onInteractions,
    this.port = 3000,
    this.scormType = ScormType.s12,
    this.playType = PlayType.offline,
    this.studentId = '3001',
    this.studentName = '',
    this.displayHorizontal = true,
  });

  @override
  _ScormPlayerRFState createState() => _ScormPlayerRFState();

  static downloadContent(String remoteUrl) {
    //rf.downloadManager.download(remoteUrl, path-to-save:scorm-store);
  }
}

class _ScormPlayerRFState extends State<ScormPlayerRF> {
  WebViewController _controller;

  @override
  void initState() {
    super.initState();
    if (widget.displayHorizontal) {
      SystemChrome.setPreferredOrientations([
        DeviceOrientation.landscapeRight,
        DeviceOrientation.landscapeLeft,
      ]);
    }

    _initialUrl = null;
    if (Platform.isAndroid) WebView.platform = SurfaceAndroidWebView();
    scorm.server.active = false;
  }

  runHttpServer() async {
    if (!scorm.server.active) await scorm.server.start();
  }

  @override
  void dispose() async {
    // SharedPreferences prefs = await SharedPreferences.getInstance(); ????????
    // String suspend_data = prefs.getString('scorm_suspend_data');
    // await prefs.setString('scorm_suspend_data', CMIUtil.masterCMI.suspend_data);
    //server.stop();
    super.dispose();
  }

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();

    if (widget.onInteractions == null) {
      widget.onInteractions = (cmi) {
        print((cmi as CoreCMI).toJson());
        /*var master = cmi as MasterCMI;
        var child = (cmi as MasterCMI).interactions;

        rf.db.save(master);
        rf.db.save(child);
        rf.db.commit();*/
      };
    }

    await runHttpServer();

    await scorm.server.purgeContent();
    scorm.server.deploy(widget.content, (host) {
      setState(() {
        _initialUrl = '${host}/scorm12.html?content=${widget.content}';
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        await SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
        return true;
      },
      child: (_initialUrl == null)
          ? Container(
              height: MediaQuery.of(context).size.height,
              color: Colors.white,
              child: Center(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    CircularProgressIndicator(),
                    SizedBox(height: 10),
                    Text('Loading content...', style: loadingTextStyle),
                    SizedBox(height: 10),
                    Text(widget.content, style: contentTextStyle),
                  ],
                ),
              ),
            )
          : Scaffold(
              body: SafeArea(
                child: Container(
                  height: MediaQuery.of(context).size.height,
                  color: Colors.black,
                  child: WebView(
                    javascriptMode: JavascriptMode.unrestricted,
                    initialUrl: _initialUrl,
                    javascriptChannels: <JavascriptChannel>[
                      _cmiDataChannel(
                          context, widget.onInteractions, widget.content, widget.studentId, widget.studentName, widget.contentId),
                    ].toSet(),
                    onWebViewCreated: (WebViewController webViewController) => _controller = webViewController,
                    onPageFinished: (finish) {
                      var func = "loadSCORM('${widget.studentId}', '${widget.studentName}');";
                      _controller.evaluateJavascript(func);
                    },
                  ),
                ),
              ),
            ),
    );
  }
}

DataCMI dataCMI = DataCMI();
List<DataCMI> dataCMIList = [];

Map<String, dynamic> mapMasterCMI = {};
Map<String, dynamic> mapInteractCMI = {};
var pendingIndex = '';
List<InteractionsCMI> _interactions = [];

JavascriptChannel _cmiDataChannel(
    BuildContext context, Function onInteractions, String contentName, String studentId, String studentName, String contentId) {
  return JavascriptChannel(
    name: 'cmidata',
    onMessageReceived: (JavascriptMessage message) async {
      var cmiMsg = message.message.split('##')[0];
      var cmiVal = message.message.split('##')[1];

      if (cmiMsg != 'null') {
        if (cmiMsg.contains('cmi.suspend_data')) {
          var _suspend_data = cmiVal.replaceAll(':', '').replaceAll('result true', '').replaceAll('returned', '').trim();
          var cookieName = '${contentName.replaceAll(RegExp(r"\s+"), '').trim()}_${studentId}_SD';

          await cookieManager.setCookies([
            Cookie(cookieName, _suspend_data)
              ..domain = '127.0.0.1'
              ..expires = DateTime.now().add(Duration(days: 365))
              ..httpOnly = false
          ]);

          if (pendingIndex.isNotEmpty) {
            scorm.coreCMI = CoreCMI.fromJson(mapMasterCMI);
            scorm.coreCMI.student_id = studentId;
            scorm.coreCMI.content_id = contentId ?? '';
            scorm.coreCMI.student_name = studentName;
            scorm.coreCMI.suspend_data = cmiVal;
            scorm.coreCMI.DataStatus = 0;

            InteractionsCMI interaction = InteractionsCMI.fromJson(mapInteractCMI);
            interaction.DataStatus = 0;
            _interactions.add(interaction);
            scorm.coreCMI.InteractionsCMI.addAll(_interactions);

            mapInteractCMI.clear();
            pendingIndex = '';

            onInteractions(scorm.coreCMI);

            CoreCMI mc = await rf.db.save(scorm.coreCMI);
            var item = scorm.coreCMI.InteractionsCMI.last;
            item.row_id = UtilRF.getUUID();
            item.master_id = mc.master_id;
            await rf.db.save(item);
          }
        } else {
          dataCMI.key = cmiMsg ?? 'empty';
          dataCMI.value = message.message
                  .split('##')[1]
                  .replaceAll('returned: true', '')
                  .replaceAll('result: true', '')
                  .replaceAll('returned:', '')
                  .replaceAll(':', '')
                  .trim() ??
              'empty';

          dataCMIList.add(dataCMI);

          if (dataCMI.key.contains('cmi.interactions')) {
            if (dataCMI.key.contains('cmi.interactions._count')) {
              pendingIndex = dataCMI.value;
            } else {
              var spitedCMI = dataCMI.key.split('.');
              var index = spitedCMI[2];
              var param = spitedCMI[3];
              mapInteractCMI['interaction'] = index;
              if (spitedCMI.length > 4 && dataCMI.key.contains('pattern')) param = param + '_pattern';
              mapInteractCMI['cmi.interactions.${param}'] = dataCMI.value;
            }
          } else {
            mapMasterCMI[dataCMI.key] = dataCMI.value;
          }
        }
      }
      print('RF SCORM: ${message.message}');
    },
  );
}
