import 'package:flutter/material.dart';
import 'package:rapidfire/GlobalScopeRF.dart';
import 'package:rapidfire/store/modern/onboarding/InternetUsage.dart';
import 'package:rapidfire/store/modern/onboarding/Language.dart';
import 'package:rapidfire/store/modern/onboarding/auth_page.dart';
import 'package:rapidfire/store/modern/setting/ChangePassword.dart';
import 'package:rapidfire/uiux/DrawerRF.dart';

import '../../../../GlobalRF.dart';
import '../../../../uiux/WidgetsRF.dart';

class StoreOutlineDefaults {
  TemplateRF _template;

  StoreOutlineHeaderDefaults storeOutline() {
    return StoreOutlineHeaderDefaults(_template);
  }

  StoreOutlineDefaults([TemplateRF template]) {
    template = TemplateRF.get();
    _template = template;
  }
}

class StoreOutlineFooterDefaults {
  TemplateRF _template;

  StoreOutlineFooterDefaults(TemplateRF template) {
    _template = template;
  }

  bottomBar(int _selectedIndex, Function _onItemTapped, {TemplateRF template}) {
    return BottomNavigationBar(
      items: const <BottomNavigationBarItem>[
        BottomNavigationBarItem(
          icon: Icon(Icons.info),
          title: Text('Why RF'),
        ),
        BottomNavigationBarItem(
          icon: Icon(Icons.home),
          title: Text('Home'),
        ),
        BottomNavigationBarItem(
          icon: Icon(Icons.widgets),
          title: Text('How To'),
        ),
      ],
      currentIndex: _selectedIndex,
      selectedItemColor: Colors.blue,
      onTap: _onItemTapped,
    );
  }
}

class StoreOutlineHeaderDefaults {
  TemplateRF _template;
  StoreOutlineHeaderDrawerDefaults header;
  StoreOutlineFooterDefaults footer;

  StoreOutlineHeaderDefaults(this._template) {
    header = StoreOutlineHeaderDrawerDefaults(_template);
    footer = StoreOutlineFooterDefaults(_template);
  }
}

class StoreOutlineHeaderDrawerDefaults {
  TemplateRF _template;
  StoreOutlineHeaderDrawerItems drawer;

  pageHeaderActions(List<ActionItemsRF> actionItems,
      {List<PopupMenuItemsRF> popupMenuItems, TemplateRF template}) {
    template = TemplateRF.get(template);

    popupMenu(BuildContext context) {
      List<Widget> items = List();
      for (var item in popupMenuItems) {
        items.add(
          InkWell(
            onTap: () {
              item.action.call();
            },
            child: ListTile(
              leading: IconButton(icon: Icon(item.icon, color: Colors.blue, size: 26)),
              title: Text(item.label, style: TextStyle(fontSize: 18)),
            ),
          ),
        );
      }
      return items;
    }

    if (template.name == TemplateNameRF.modern) {
      List<Widget> widgets = List();
      for (var item in actionItems) {
        if (item.actionType == ActionItemTypeRF.geo) {
          widgets.addAll([
            IconButton(
                icon: Icon(Icons.my_location, color: Colors.black45), onPressed: item.action),
          ]);
        }
        if (item.actionType == null) {
          widgets.addAll([
            IconButton(icon: Icon(item.icon, color: Colors.black45), onPressed: item.action),
          ]);
        }
      }
      return widgets;
    }
    if (template.name == TemplateNameRF.material) {
      List<Widget> widgets = List();
      for (var item in actionItems) {
        if (item.actionType == ActionItemTypeRF.geo) {
          widgets.addAll([
            IconButton(icon: Icon(Icons.my_location, color: Colors.white), onPressed: item.action),
          ]);
        } else if (item.actionType == ActionItemTypeRF.sci) {
          widgets.addAll([
            IconButton(icon: Icon(Icons.my_location, color: Colors.white), onPressed: item.action),
            PopupMenuButton(
              itemBuilder: (BuildContext context) {
                return [PopupMenuItem(child: Column(children: popupMenu(context)))];
              },
            )
          ]);
        }
      }
      return widgets;
    }
  }

  StoreOutlineHeaderDrawerDefaults(this._template) {
    drawer = StoreOutlineHeaderDrawerItems(_template);
  }
}

class StoreOutlineHeaderDrawerItems {
  TemplateRF _template;

  StoreOutlineHeaderDrawerItems(this._template);

  drawerMenuButton(Function onTap, {dynamic template}) {
    return IconButton(
      icon: Icon(Icons.menu),
      onPressed: () => onTap(),
      color: Colors.black54,
    );
  }

  drawerHeader(
    String title,
    String subTitle,
    String imagePath,
    Color circleColor, {
    TemplateRF template,
    BuildContext context,
  }) {
    var modernHeader = Container(
      width: 250,
      child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(title, style: WidgetStyleRF.theme.appTitle(template)),
            TextRF(
              subTitle ?? '',
              style: WidgetStyleRF.theme.appSubTitle(template),
            )
          ]),
    );

    var modernOvalHeader = Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Container(
          color: Colors.transparent,
          height: 240,
          padding: EdgeInsets.only(top: 15),
          child: Container(
            padding: EdgeInsets.zero,
            child: Stack(
              alignment: AlignmentDirectional.center,
              children: <Widget>[
                Positioned(
                  child: FaIcon(FontAwesomeIcons.powerOff, color: Colors.red),
                  right: 50,
                  top: 10,
                ),
                Positioned(
                    top: 30.0,
                    left: 105,
                    child: Column(
                      children: <Widget>[
                        SizedBox(height: 20),
                        CircleAvatar(
                          radius: 55,
                          backgroundColor: Colors.redAccent,
                          child: CircleAvatar(
                            radius: 50,
                            backgroundImage: AssetImage(imagePath), //
                          ),
                        ),
                        Padding(
                          padding: EdgeInsets.only(top: 5),
                          child: Text(
                            title,
                            style: TextStyle(
                              fontWeight: FontWeight.bold,
                              color: Colors.black,
                              fontSize: 18,
                            ),
                          ),
                        ),
                        Padding(
                          padding: const EdgeInsets.only(top: 0),
                          child: Text(
                            subTitle ?? '',
                            style: TextStyle(color: Colors.black),
                          ),
                        ),
                      ],
                    )),
              ],
            ),
          ),
        )
      ],
    );

    var materialHeader = Container(
      height: 250,
      color: Colors.blue,
      alignment: Alignment(0, 0),
      child: Column(
        children: <Widget>[
          SizedBox(height: 60),
          CircleAvatar(
            radius: 55,
            backgroundColor: Colors.redAccent,
            child: CircleAvatar(
              radius: 50, backgroundImage: AssetImage(imagePath), //
            ),
          ),
          SizedBox(height: 10),
          Text(
            title,
            style: WidgetStyleRF.theme
                .pageTitle(TemplateRF(null, style: WidgetStyleRF(textColor: Colors.white))),
          ),
          //.theme.pageTitle()
          Text(
            subTitle ?? '',
            style: WidgetStyleRF.theme.pageSubTitle(
              TemplateRF(null, style: WidgetStyleRF(textColor: Colors.white, fontSize: 16)),
            ),
          )
        ],
      ),
    );

    template = TemplateRF.get(template);
    if (template.name == TemplateNameRF.modern && template.version == 'Oval')
      return modernOvalHeader;
    else if (template.name == TemplateNameRF.modern)
      return modernHeader;
    else if (template.name == TemplateNameRF.material) return materialHeader;
  }

  //scaff - page. outline . drawer . drawerItems
  menuBody(
    BuildContext context, {
    bool separator = false,
    FontWeight textWeight = FontWeight.w500,
    Color color = Colors.black,
    double padding = 12.0,
    TemplateRF template,
    Function onTap,
  }) {
    //template = TemplateRF.get();
    if (template.name == TemplateNameRF.modern && template.version == 'Oval') {
      separator = true;
      textWeight = FontWeight.w500;
      color = Colors.black;
      padding = 12.0;
    } else if (template.name == TemplateNameRF.modern) {
      separator = false;
      textWeight = FontWeight.w500;
      color = Colors.white;
      padding = 6.0;
    } else if (template.name == TemplateNameRF.material) {
      separator = true;
      textWeight = FontWeight.w500;
      color = Colors.black;
      padding = 12.0;
    }

    if (Libs.config.app.outline?.customOutline()?.drawerItems == null) {
      var sciStandard = [
        DrawerItemRF(
          icon: FontAwesomeIcons.userAlt,
          text: 'My Profile',
          separator: separator,
          textWeight: textWeight,
          color: Color(0xFFf77f00),
          padding: padding,
          template: _template,
          onTap: () {
            return onTap("My Profile");
          },
        ),
        DrawerItemRF(
          icon: FontAwesomeIcons.userLock,
          text: 'Change Password',
          separator: separator,
          textWeight: textWeight,
          color: Color(0xFF2a9d8f),
          padding: padding,
          template: _template,
          onTap: () {
            rf.page.call(context, ChangePass(), refreshCaller: true);
            return onTap("Change Password");
          },
        ),
        DrawerItemRF(
          icon: FontAwesomeIcons.cloudMeatball,
          text: 'Internet Usage',
          separator: separator,
          textWeight: textWeight,
          color: Color(0xFFe63946),
          padding: padding,
          template: _template,
          onTap: () {
            rf.page.call(context, InternetUsage(), refreshCaller: true);
          },
        ),
        DrawerItemRF(
          icon: FontAwesomeIcons.database,
          text: 'Email Data Backup',
          separator: separator,
          textWeight: textWeight,
          color: Color(0xFF7209b7),
          padding: padding,
          template: _template,
          onTap: () {
            return onTap("Email Data Backup");
          },
        ),
        DrawerItemRF(
          icon: FontAwesomeIcons.microphone,
          text: 'Language Setting',
          separator: separator,
          textWeight: textWeight,
          color: Color(0xFF27AE60),
          padding: padding,
          template: _template,
          onTap: () {
            rf.page.call(
                context,
                LanguageSelection(
                    title: Libs.config.app.title,
                    languages: [
                      new LanguageItems(
                        isOnboarding: false,
                        name: "English",
                        text: "This is English Language",
                        img: 'assets/images/lan-en.png',
                        locale: 'en',
                      ),
                      new LanguageItems(
                        isOnboarding: false,
                        name: "Bangla",
                        text: "এটি বাংলা ভাষা",
                        img: "assets/images/lan-bg.png",
                        locale: 'bn',
                      )
                    ],
                    isOnboarding: false),
                refreshCaller: true);
          },
        ),
        DrawerItemRF(
          icon: FontAwesomeIcons.cog,
          text: 'Data Download',
          separator: separator,
          textWeight: textWeight,
          color: Color(0xFF01baef),
          padding: padding,
          template: _template,
          onTap: () {
            return onTap("Data Setting");
          },
        ),
        DrawerItemRF(
          icon: FontAwesomeIcons.signOutAlt,
          text: 'LogOut',
          separator: separator,
          textWeight: textWeight,
          color: Color(0xFFffc300),
          padding: padding,
          template: _template,
          onTap: () {
            return onTap("logout");
          },
        ),
      ];

      if (template.directives != null) if (template.directives['DrawerMenuBodySetVersion'] ==
          'SCI') {
        return Libs.config.app.outline.customOutline().drawerItems ?? sciStandard;
      }
      if (template.name == TemplateNameRF.modern && template.version == 'SCI') {
        return Libs.config.app.outline.customOutline().drawerItems ?? sciStandard;
      }
      if (template.name == TemplateNameRF.modern || template.name == TemplateNameRF.material) {
        return Libs.config.app.outline.customOutline().drawerItems ?? sciStandard;
      }
    } else {
      return Libs.config.app.outline.customOutline(onDrawerItemClick: onTap).drawerItems;
    }
  }

  //scaff - page. outline . drawer . drawerFooter
  drawerFooter(
    BuildContext context, {
    bool separator = false,
    FontWeight textWeight = FontWeight.w500,
    Color color = Colors.black,
    double padding = 12.0,
    TemplateRF template,
  }) {
    template = TemplateRF.get();
    if (template.name == TemplateNameRF.modern && template.version == 'Oval') {
      separator = true;
      textWeight = FontWeight.w500;
      color = Colors.black;
      padding = 12.0;
    } else if (template.name == TemplateNameRF.modern) {
      separator = false;
      textWeight = FontWeight.w500;
      color = Colors.white;
      padding = 10.0;
    } else if (template.name == TemplateNameRF.material) {
      separator = false;
      textWeight = FontWeight.w500;
      color = Colors.pink;
      padding = 12.0;
    }
    return DrawerItemRF(
      icon: FontAwesomeIcons.headphonesAlt,
      text: 'SUPPORT',
      separator: separator,
      textWeight: textWeight,
      color: color,
      template: _template,
      padding: padding,
      onTap: () => rf.page.call(context, AuthPage()),
    );
  }
}

enum ActionItemTypeRF { geo, floating, sci }

class ActionItemsRF {
  ActionItemTypeRF actionType;
  Function action;
  IconData icon;

  ActionItemsRF({this.actionType, this.action, this.icon});
}

class PopupMenuItemsRF {
  IconData icon;
  String label;
  Function action;
  bool separator;

  PopupMenuItemsRF({this.icon, this.label, this.action, this.separator = true});
}
