import 'dart:async';

import 'package:carousel_slider/carousel_slider.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:rapidfire/Api.dart';
import 'package:rapidfire/Config.dart';
import 'package:rapidfire/GlobalRF.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/uiux/RipplesRF.dart';

class InitialDownload extends StatefulWidget {
  bool initialCall = true;
  String fetchDataJobName;
  Color backgroundColor;
  Function fetchDataService;

  InitialDownload({this.initialCall = true, this.fetchDataJobName, this.backgroundColor, this.fetchDataService});

  @override
  _InitialDownloadState createState() => _InitialDownloadState();
}

class _InitialDownloadState extends State<InitialDownload> with TickerProviderStateMixin {
  int _currentStep = 0;
  ScrollController _scrollController = new ScrollController();
  AnimationController animationController, animController;
  Animation<double> animation, reverseAnimation, grow;
  List<initialItem> items = [];
  List<String> fetchItems = [];
  List<String> errorItems = [];
  bool fabVisible = false;
  bool show = false;
  bool singleDownload = false;
  Timer timer;
  List<String> images = [
    "assets/images/imagesix.png",
    "assets/images/mmg.png",
    "assets/images/imageone.png",
    "assets/images/imagefour.png",
    "assets/images/imagefive.png",
  ];

  @override
  void initState() {
    Api.onFetchCompleted = (model, count) {
      print("Printing on Complete: " + model + " " + count.toString());
      if (model.toString() != "all fetch completed") {
        rf.pref.setInt('in' + model.toString(), count);
        if (singleDownload) {
          addInitialItems();
        } else {
          nextStep(count, 2);
        }
      }
    };

    Api.onFetchFailed = (model) {
      print("Printing on Failure: " + model + " ");
      rf.pref.setInt('in' + model.toString(), -1);
      errorItems.add(model);
      if (singleDownload) {
        addInitialItems();
      } else {
        nextStep(0, 3);
      }
    };

    if (!widget.initialCall) fabVisible = true;

    animationController = AnimationController(duration: const Duration(milliseconds: 400), vsync: this);
    animController = AnimationController(duration: const Duration(milliseconds: 1300), vsync: this);
    animation = Tween<double>(begin: 0.2, end: 1.0).animate(CurvedAnimation(
      parent: animationController,
      curve: Curves.elasticInOut,
    ));
    reverseAnimation = Tween<double>(begin: 300, end: 0).animate(CurvedAnimation(
      parent: animController,
      curve: Curves.decelerate,
    ));

    grow = Tween<double>(begin: 0, end: 1).animate(CurvedAnimation(
      parent: animController,
      curve: Curves.ease,
    ));
  }

  @override
  void dispose() {
    timer.cancel();
    animationController.dispose();
    animController.dispose();
    _scrollController.dispose();
    Api.onFetchCompleted = null;
    super.dispose();
  }

  @override
  void didChangeDependencies() {}

  void nextStep(int count, int error) {
    _currentStep += 1;
    updateSteps(_currentStep, count, error);

    if (_currentStep > 2) {
      if (_scrollController.position.pixels != _scrollController.position.maxScrollExtent) {
        _scrollController.animateTo(
          (_currentStep * 100).toDouble(),
          curve: Curves.ease,
          duration: const Duration(milliseconds: 800),
        );
      }
    }
    print(_currentStep.toString() + "--" + fetchItems.length.toString());
    if (_currentStep == fetchItems.length) {
      if (widget.initialCall) {
        rf.pref.setBool("IsInitDone", true);
        rf.page.replace(context, rf.onboard.home);
      } else
        fabVisible = true;
    }

    setState(() {
      animationController.reset();
      animationController.forward();
    });
  }

  addInitialItems({bool startDownload = false}) async {
    _currentStep = 0;

    fetchItems.clear();
    items.clear();
    print("Data Count:" + fetchItems.length.toString());
    Models models = Models();
    for (var item in models.get().entries) {
      var modelRF = item.value as ModelRF;
      if (widget.initialCall) {
        if (modelRF.initialFetch && !modelRF.alreadyFetched) {
          fetchItems.add(item.key.toString());
          int prefData = rf.pref.getInt('in' + modelRF.model.runtimeType.toString()) ?? 0;

          items.add(initialItem(
              Title: modelRF.displayName,
              Name: modelRF.model.runtimeType.toString(),
              ComplitionTime: "",
              image: modelRF.displayImage,
              Status: prefData < 0
                  ? 3
                  : prefData > 0
                      ? 2
                      : 0,
              DataCount: prefData < 0 ? 0 : prefData));
        }
      } else {
        if (modelRF.initialFetch || modelRF.fetchBeforeSignUp) {
          fetchItems.add(item.key.toString());
          int prefData = rf.pref.getInt('in' + modelRF.model.runtimeType.toString()) ?? 0;

          items.add(initialItem(
              Title: modelRF.displayName,
              Name: modelRF.model.runtimeType.toString(),
              ComplitionTime: "",
              image: modelRF.displayImage,
              Status: prefData < 0
                  ? 3
                  : prefData > 0
                      ? 2
                      : 0,
              DataCount: prefData < 0 ? 0 : prefData));
        }
      }
    }

    setState(() {});
    if (widget.initialCall || startDownload) {
      Api().fetchInitials(context, widget.fetchDataJobName, widget.fetchDataService, fetchItems);
      updateSteps(_currentStep, 0, 0);
    }
    animController.forward();
    animationController.forward();
  }

  updateSteps(int stepcount, int count, int error) {
    for (int i = 0; i < items.length; i++) {
      print("Item Name: " + items[i].Name + " ");
      if (i < stepcount) {
        items[i].Status = 2;
        if (rf.pref.getInt('in' + items[i].Name) < 0) items[i].Status = 3;
        if (items[i].ComplitionTime.isEmpty) {
          items[i].ComplitionTime = Jiffy().format("h:mm:ss a");
          items[i].DataCount = count;
        }
      } else if (i == stepcount)
        items[i].Status = 1;
      else
        items[i].Status = 0;
    }
  }

  getbarColor(int status) {
    if (status == 2)
      return Colors.pinkAccent;
    else if (status == 3)
      return Colors.red;
    else
      return Colors.grey;
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () {},
      child: Scaffold(
        backgroundColor: widget.backgroundColor ?? Colors.lightBlue,
        body: Stack(
          children: [
            Container(
              //top & bottom padding, image in middle
              margin: EdgeInsets.only(top: 25),
              child: CarouselSlider(
                options: CarouselOptions(
                  height: 160,
                  aspectRatio: 16 / 9,
                  viewportFraction: 1,
                  initialPage: 0,
                  enableInfiniteScroll: true,
                  reverse: false,
                  autoPlay: true,
                  autoPlayInterval: Duration(seconds: 5),
                  autoPlayAnimationDuration: Duration(milliseconds: 500),
                  autoPlayCurve: Curves.easeInOut,
                  enlargeCenterPage: true,
                  scrollDirection: Axis.horizontal,
                ),
                items: images.map((i) {
                  return Builder(
                    builder: (BuildContext context) {
                      return Container(
                        width: MediaQuery.of(context).size.width,
                        margin: EdgeInsets.symmetric(horizontal: 5.0),
                        child: Image.asset(i, height: 300),
                      );
                    },
                  );
                }).toList(),
              ),
            ),
            Container(
              margin: EdgeInsets.only(top: 200),
              padding: EdgeInsets.only(top: 30, bottom: 30),
              decoration: new BoxDecoration(
                  boxShadow: <BoxShadow>[
                    BoxShadow(
                      offset: Offset(0, 0),
                      blurRadius: 10,
                      color: Colors.black45.withOpacity(.5),
                    )
                  ],
                  color: Colors.white,
                  borderRadius: new BorderRadius.only(topLeft: Radius.circular(40), topRight: Radius.circular(40))),
              child: show
                  ? Stack(
                      children: [
                        AnimatedBuilder(
                          animation: animController,
                          builder: (context, child) {
                            return Transform.translate(
                              offset: Offset(0.0, reverseAnimation.value),
                              child: ListView.builder(
                                  padding: EdgeInsets.all(0),
                                  controller: _scrollController,
                                  itemCount: items.length,
                                  itemBuilder: (BuildContext context, int index) {
                                    return InkWell(
                                      onTap: () {
                                        if (!widget.initialCall) {
                                          print("fetch button clicked");
                                          singleDownload = true;
                                          rf.api.fetchModel(
                                              modelsName: items[index].Name,
                                              context: context,
                                              serviceName: widget.fetchDataJobName,
                                              dispatcher: widget.fetchDataService);
                                        }
                                      },
                                      child: Stack(
                                        children: [
                                          Positioned(
                                            left: 50.0,
                                            top: index == 0 ? 40 : 0,
                                            child: Container(
                                              height: index == items.length - 1 ? 70 : 120,
                                              width: 1,
                                              color: getbarColor(items[index].Status),
                                            ),
                                          ),
                                          Container(
                                            width: 500,
                                            height: 90,
                                            margin: EdgeInsets.only(left: 10, right: 10, bottom: 25),
                                            child: Row(
                                              children: [
                                                getLeadingView(index, items[index].Status, items[index].image),
                                                SizedBox(
                                                  width: 10,
                                                ),
                                                Expanded(
                                                  child: AnimatedBuilder(
                                                    animation: animController,
                                                    builder: (context, child) {
                                                      return Transform.translate(
                                                        offset: Offset(reverseAnimation.value * index * index, 0),
                                                        child: Transform.scale(
                                                          scale: grow.value,
                                                          child: Column(
                                                            crossAxisAlignment: CrossAxisAlignment.start,
                                                            mainAxisAlignment: MainAxisAlignment.center,
                                                            children: [
                                                              Row(
                                                                children: [
                                                                  TextRF(
                                                                    items[index].Title,
                                                                    style: TextStyle(fontSize: 18),
                                                                  ),
                                                                  SizedBox(
                                                                    width: 8,
                                                                  ),
                                                                  items[index].DataCount != 0
                                                                      ? TextRF(
                                                                          items[index].DataCount.toString(),
                                                                          style: TextStyle(fontSize: 22, color: Colors.grey[300]),
                                                                        )
                                                                      : Container(),
                                                                ],
                                                              ),
                                                              SizedBox(
                                                                height: 8,
                                                              ),
                                                              if (items[index].Status == 2)
                                                                TextRF(
                                                                  "DownloadSuccess",
                                                                  style: TextStyle(fontSize: 13, color: Color(0Xff52b788)),
                                                                )
                                                              else if (items[index].Status == 1)
                                                                TextRF(
                                                                  "DownloadStart",
                                                                  style: TextStyle(fontSize: 13, color: Colors.pink),
                                                                )
                                                              else if (items[index].Status == 3)
                                                                TextRF(
                                                                  "DownloadError",
                                                                  style: TextStyle(
                                                                      fontSize: 13,
                                                                      color: Colors.red,
                                                                      fontWeight: FontWeight.bold),
                                                                )
                                                              else
                                                                TextRF(
                                                                  "WaitingForDownload",
                                                                  style: TextStyle(fontSize: 13, color: Colors.grey),
                                                                )
                                                            ],
                                                          ),
                                                        ),
                                                      );
                                                    },
                                                  ),
                                                ),
                                                Column(
                                                  crossAxisAlignment: CrossAxisAlignment.center,
                                                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                                  children: [
                                                    Text(
                                                      items[index].ComplitionTime,
                                                      style: TextStyle(fontSize: 12, color: Colors.grey),
                                                    ),
                                                    if (items[index].Status == 2)
                                                      Image.asset(
                                                        "assets/images/checked.png",
                                                        width: 25,
                                                      )
                                                    else
                                                      Container(
                                                        width: 25,
                                                        height: 25,
                                                      ),
                                                  ],
                                                ),
                                                SizedBox(
                                                  width: 10,
                                                ),
                                              ],
                                            ),
                                          ),
                                        ],
                                      ),
                                    );
                                  }),
                            );
                          },
                        ),
                        fabVisible
                            ? SpeedDial(
                                // both default to 16
                                marginRight: 30,
                                marginBottom: 0,
                                animatedIcon: AnimatedIcons.menu_close,
                                animatedIconTheme: IconThemeData(size: 22.0),
                                // this is ignored if animatedIcon is non null
                                // child: Icon(Icons.add),
                                // If true user is forced to close dial manually
                                // by tapping main button and overlay is not rendered.
                                closeManually: false,
                                curve: Curves.bounceIn,

                                overlayOpacity: 0.5,
                                onOpen: () => print('OPENING DIAL'),
                                onClose: () => print('DIAL CLOSED'),
                                tooltip: 'menu',
                                heroTag: 'speed-dial-hero-tag',
                                backgroundColor: Colors.blue,
                                foregroundColor: Colors.white,
                                elevation: 8.0,
                                shape: CircleBorder(),
                                children: [
                                  SpeedDialChild(
                                      child: Icon(
                                        Icons.cloud_download_outlined,
                                        color: Colors.white,
                                      ),
                                      backgroundColor: Color(0Xff0096c7),
                                      labelBackgroundColor: Color(0Xff0096c7),
                                      label: getString('Download Updates'),
                                      labelStyle: TextStyle(fontSize: 18.0, color: Colors.white),
                                      onTap: () {
                                        animController.reset();
                                        //rf.msg(context, "Coming Soon . . .");
                                        animController.forward();
                                      }),
                                  SpeedDialChild(
                                    child: Icon(
                                      Icons.download_rounded,
                                      color: Colors.white,
                                    ),
                                    backgroundColor: Color(0Xff00b4d8),
                                    labelBackgroundColor: Color(0Xff00b4d8),
                                    label: getString('Download All'),
                                    labelStyle: TextStyle(fontSize: 18.0, color: Colors.white),
                                    onTap: () {
                                      _scrollController.animateTo(
                                        _scrollController.position.minScrollExtent,
                                        curve: Curves.easeOut,
                                        duration: const Duration(milliseconds: 300),
                                      );
                                      setState(() {
                                        items.clear();
                                        singleDownload = false;
                                        addInitialItems(startDownload: true);
                                      });
                                    },
                                  ),
                                  SpeedDialChild(
                                    child: Icon(
                                      Icons.home,
                                      color: Colors.white,
                                    ),
                                    backgroundColor: Color(0Xff48cae4),
                                    labelBackgroundColor: Color(0Xff48cae4),
                                    label: getString('Back to Home'),
                                    labelStyle: TextStyle(fontSize: 18.0, color: Colors.white),
                                    onTap: () => rf.page.replace(context, rf.onboard.home),
                                  ),
                                ],
                              )
                            : Container(),
                      ],
                    )
                  : Container(),
            ),
          ],
        ),
      ),
    );
  }

  String getString(String text) {
    var _text = '';
    _text = rf.lang.text('System', text).replaceAll('not found', '').trim();

    if (_text == text || _text == '') {
      _text = rf.lang.text('Common', text).replaceAll('not found', '').trim();
    }
    return _text;
  }

  getLeadingView(int index, int status, String img) {
    if (status == 1) {
      //running state
      return AnimatedBuilder(
        animation: animationController,
        builder: (context, child) {
          return Transform.scale(
            scale: animation.value,
            child: RipplesRF(
                size: 26,
                color: Colors.pinkAccent,
                child: Container(
                  decoration: BoxDecoration(
                    color: Colors.transparent,
                  ),
                  width: 60,
                  child: CircleAvatar(
                    backgroundColor: Colors.pinkAccent,
                    child: Image.asset(
                      img,
                      alignment: Alignment.center,
                      width: 40,
                    ),
                  ),
                )),
          );
        },
      );
    } else if (status == 2) {
      //completed state
      return Container(
        width: 50,
        height: 50,
        margin: EdgeInsets.only(left: 15, right: 10),
        padding: EdgeInsets.all(10),
        // margin: EdgeInsets.only(left: 15, right: 15, bottom: 20, top: 10),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(100)),
          border: Border.all(width: 1, color: Colors.green),
          boxShadow: <BoxShadow>[
            BoxShadow(
              offset: Offset(0, 0),
              blurRadius: 3,
              color: Colors.black.withOpacity(.2),
            )
          ],
        ),
        child: Center(
          child: Image.asset(
            img,
            width: 35,
          ),
        ),
      );
    } else if (status == 3) {
      //completed state
      return Container(
        width: 50,
        height: 50,
        margin: EdgeInsets.only(left: 15, right: 10),
        padding: EdgeInsets.all(10),
        // margin: EdgeInsets.only(left: 15, right: 15, bottom: 20, top: 10),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(100)),
          border: Border.all(width: 1, color: Colors.red),
          boxShadow: <BoxShadow>[
            BoxShadow(
              offset: Offset(0, 0),
              blurRadius: 3,
              color: Colors.black.withOpacity(.2),
            )
          ],
        ),
        child: Center(
          child: Image.asset(
            img,
            width: 35,
          ),
        ),
      );
    } else {
      return AnimatedBuilder(
        animation: animController,
        builder: (context, child) {
          return Transform.scale(
            scale: index == 0 ? 1 : grow.value,
            child: Container(
              width: 50,
              height: 50,
              margin: EdgeInsets.only(left: 15, right: 10),
              padding: EdgeInsets.all(8),
              // margin: EdgeInsets.only(left: 15, right: 15, bottom: 20, top: 10),
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.all(Radius.circular(100)),
                border: Border.all(width: 1, color: Colors.grey),
                /*  boxShadow: <BoxShadow>[
                BoxShadow(
                  offset: Offset(0, 0),
                  blurRadius: 3,
                  color: Colors.black.withOpacity(.2),
                )
              ],*/
              ),
              child: Center(
                child: Image.asset(
                  img,
                  width: 35,
                ),
              ),
            ),
          );
        },
      );
    }
  }

  _InitialDownloadState() {
    timer = new Timer(const Duration(milliseconds: 500), () {
      setState(() {
        show = true;
        if (items.isEmpty) addInitialItems();
      });
    });
  }
}

//items model
class initialItem {
  String Title;
  String Name;
  String ComplitionTime;
  String image;
  int Status; //0=waiting,1=downloading,2=completed,3=failed
  int DataCount;

  initialItem({this.Title, this.Name, this.ComplitionTime, this.image, this.Status, this.DataCount = 0});
}
