import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:rapidfire/GlobalScopeRF.dart';
import 'package:rapidfire/UtilRF.dart';
import 'package:rapidfire/uiux/RipplesRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import '../../../GlobalRF.dart';

class Splash extends StatefulWidget {
  SplashRF splash;

  Splash({this.splash});

  @override
  _SplashState createState() => _SplashState();
}

class _SplashState extends State<Splash> {
  @override
  Widget build(BuildContext context) {
    var title = widget.splash?.title ?? Libs.config.app.title;
    var subTitle = widget.splash?.subTitle ?? Libs.config.app.subTitle;
    var color = widget.splash?.color ?? Colors.pink;
    var colorTitle = widget.splash?.titleColor ?? Colors.pink;
    var font = widget.splash?.font ?? 'Raleway';
    var colorSubTitle = widget.splash?.subTitleColor ?? Colors.black;
    var rippleImagePath =
        widget.splash?.rippleImagePath ?? 'assets/images/child.png';
    var rippleImageSize = widget.splash?.rippleImageSize ?? 0.2;
    var rippleColor =
        widget.splash?.rippleColor ?? widget.splash?.color ?? Colors.pink;
    var orgLogoPath = widget.splash?.orgLogoPath ?? 'assets/images/sci.png';

    var loggedin = rf.pref.getBool("IsLoggedIn");
    var initialDownloadDone = rf.pref.getBool("IsInitDone");
    Timer(Duration(seconds: widget.splash?.duration ?? 4), () {
      if (loggedin != null && loggedin) {
        if (initialDownloadDone != null && initialDownloadDone)
          rf.page.replace(context, rf.onboard.home);
        else
          rf.page.replace(context, rf.onboard.initialSetup);
      } else {
        rf.page.replace(context, rf.onboard.next());
      }
    }); //Intro()
    return Material(
      color: color ?? Colors.white, //alignment: Alignment(0, 0),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.start,
        children: <Widget>[
          Expanded(child: SizedBox(height: 50), flex: 8),
          Expanded(
            child: TextRF(title,
                align: TextAlign.center,
                style: TextStyle(
                  fontFamily: 'Berlin Sans FB Demi Bold',
                  color: colorTitle,
                  fontSize: 38,
                  fontWeight: FontWeight.bold,
                )),
            flex: MediaQuery.of(context).devicePixelRatio >= 2.5 ? 6 : 8,
          ),
          Expanded(
            child: Center(
              child: TextRF(
                subTitle, //'Development framework for mobile',
                align: TextAlign.center,
                style: TextStyle(
                  fontFamily: font,
                  fontSize: 18,
                  color: colorSubTitle,
                  fontWeight: FontWeight.w500,
                ),
              ),
            ),
            flex: 3,
          ),
          Expanded(
            child: Align(
              alignment: Alignment.topCenter,
              child: Padding(
                padding: EdgeInsets.only(top: 12.0.h),
                child: ripple(
                    context, rippleColor, rippleImagePath, rippleImageSize),
              ),
            ),
            flex: 75,
          ),
          Expanded(
            child: Padding(
              padding: EdgeInsets.all(2.0.h),
              child: widget.splash?.orgLogoPath == null
                  ? UtilRF.libImage(orgLogoPath, 'rapidfire')
                  : Image.asset(orgLogoPath),
            ),
            flex: 10,
          ),
        ],
      ),
    );
  }
}

Widget ripple(
    BuildContext context, Color color, String imagePath, double imageSize) {
  return RipplesRF(
    color: color,
    child: Container(
      decoration: BoxDecoration(
        color: Colors.transparent,
      ),
      width: MediaQuery.of(context).size.width * imageSize,
      child: Image.asset(
        imagePath,
        alignment: Alignment.centerLeft,
      ),
    ),
  );
}
