import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/models/Quiz.dart';

enum LoadType { offline, online }

class QuizEngine {
  LoadType loadType;

  //region Veriables
  Quiz quiz = Quiz();
  String quizId;
  double singleQuestionMark = 0;
  double totalEarnedMark = 0;
  List<Question> questions = [];
  List<Options> options = [];
  int currentQuestin = -1;
  Question activeQuestion;
  List<Options> activeOptions = [];
  List<Answer> answerList = [];
  int answered = 0;
  String sessionId = UtilRF.getUUID();
  //for multiple choice
  int isAnswerRight = 0;
  static List<Options> selectedOptions = [];
//endregion

  QuizEngine({this.quizId, this.loadType});

  Future<void> load() async {
    var quizData = await rf.db.get(quiz, 'select * from Quiz where QuizId = ?', ['${quizId}']);
    quiz = quizData.first;

    Question question = Question();
    var questionData = await rf.db.get(question, 'select * from Question where QuizId = ?', ['${quizId}']);
    for (Question q in questionData) {
      Options option = Options();
      List<Options> ops = [];
      var OptionData = await rf.db.get(option, 'select * from Options where QuestionId = ?', ['${q.QuestionId}']);
      for (Options option in OptionData) {
        ops.add(option);
      }
      q.Options = ops;
      questions.add(q);
    }
    return quiz;
  }

  next() {
    currentQuestin++;
    if (activeQuestion != null && activeQuestion.QuestionTypeId != 1) {
      if (selectedOptions.length > 0) {
        answered++;
        if (isAnswerRight == 1) totalEarnedMark = totalEarnedMark + singleQuestionMark;
        answerList.add(Answer(AnswerId: UtilRF.getUUID(), QuestionId: activeQuestion.QuestionId, QuizId: quiz.QuizId, SessionId: sessionId, IsCorrect: isAnswerRight));
      } else {
        answerList.add(Answer(AnswerId: UtilRF.getUUID(), QuestionId: activeQuestion.QuestionId, QuizId: quiz.QuizId, SessionId: sessionId, IsCorrect: 0));
      }
    } else {
      if (activeQuestion != null) {
        var isExist = answerList.where((element) => element.QuestionId == activeQuestion.QuestionId).isNotEmpty;
        if (!isExist) answerList.add(Answer(AnswerId: UtilRF.getUUID(), QuestionId: activeQuestion.QuestionId, QuizId: quiz.QuizId, SessionId: sessionId, IsCorrect: 0));
      }
    }
    selectedOptions.clear();
    isAnswerRight = 0;
    if (questions.length > currentQuestin) {
      activeQuestion = questions[currentQuestin];
      activeOptions = activeQuestion.Options;
      singleQuestionMark = activeQuestion.Marks.toDouble();
      return activeQuestion;
    } else
      return null;
  }

  submitAnswer(Options selectedOption) async {
    if (activeQuestion != null && activeQuestion.QuestionTypeId == 1) {
      answered++;
      int isCorrect = 0;
      if (selectedOption != null && selectedOption.IsCorrect == 1) {
        isCorrect = 1;
        totalEarnedMark = totalEarnedMark + singleQuestionMark;
      }
      answerList.add(Answer(AnswerId: UtilRF.getUUID(), QuestionId: activeQuestion.QuestionId, QuizId: quiz.QuizId, SessionId: sessionId, IsCorrect: isCorrect));
    } else {
      if (selectedOption != null) selectedOptions.add(selectedOption);

      int totalCorrect = 0;
      int selectedCorrect = 0;

      for (Options a in activeOptions) {
        if (a.IsCorrect == 1) totalCorrect++;
      }
      for (Options a in selectedOptions) {
        if (a.IsCorrect == 1) selectedCorrect++;
      }
      if (totalCorrect == selectedCorrect)
        isAnswerRight = 1;
      else
        isAnswerRight = 0;

      for (Options a in selectedOptions) {
        if (a.IsCorrect == 0) isAnswerRight = 0;
      }
    }
  }

  getProgressDouble() {
    if (questions.length == 0)
      return 0.toDouble();
    else
      return (currentQuestin / questions.length).toDouble() < 0 ? 0.toDouble() : (currentQuestin / questions.length).toDouble();
  }

  getProgressTxt() {
    return "$currentQuestin/" + questions.length.toString();
  }

  void showResult() {
    DialogRF(rf.context,
        height: 440,
        avatarIcon: FontAwesomeIcons.bookOpen,
        hideAvatar: true,
        avatarIconSize: 25,
        hideDefaultControls: true,
        type: DialogType.popup,
        title: 'Quiz Result',
        subTitle: quiz.Title,
        body: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Center(
              child: Container(
                height: 80,
                width: 80,
                margin: const EdgeInsets.all(15),
                decoration: BoxDecoration(
                  boxShadow: [BoxShadow(offset: Offset(6, 6), color: Colors.black38, blurRadius: 20), BoxShadow(offset: Offset(-6, -6), color: Colors.white.withOpacity(0.85), blurRadius: 20)],
                  color: Colors.white38,
                  borderRadius: BorderRadius.circular(20.0),
                ),
                child: Center(
                    child: TextRF(
                  totalEarnedMark.toString(),
                  color: Colors.cyan,
                  size: 30,
                  weight: FontWeight.bold,
                )),
              ),
            ),
            GapRF(15),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: [
                Container(
                  height: 30,
                  padding: const EdgeInsets.symmetric(horizontal: 6),
                  decoration: new BoxDecoration(
                    color: Colors.deepPurpleAccent[100].withOpacity(0.2),
                    borderRadius: new BorderRadius.all(Radius.circular(4)),
                  ),
                  child: Center(
                    child: TextRF(
                      questions.length.toString() + " Questions",
                      size: 15,
                      color: Colors.deepPurple,
                      weight: FontWeight.w300,
                    ),
                  ),
                ),
                Container(
                  height: 30,
                  padding: const EdgeInsets.symmetric(horizontal: 6),
                  decoration: new BoxDecoration(
                    color: Colors.blue[200].withOpacity(0.3),
                    borderRadius: new BorderRadius.all(Radius.circular(4)),
                  ),
                  child: Center(
                    child: TextRF(
                      answered.toString() + " Answered",
                      size: 15,
                      color: Colors.blue,
                      weight: FontWeight.w300,
                    ),
                  ),
                ),
                Container(
                  height: 30,
                  padding: const EdgeInsets.symmetric(horizontal: 6),
                  decoration: new BoxDecoration(
                    color: Colors.deepOrange[200].withOpacity(0.3),
                    borderRadius: new BorderRadius.all(Radius.circular(4)),
                  ),
                  child: Center(
                    child: TextRF(
                      (questions.length - answered).toString() + " Skipped",
                      size: 15,
                      color: Colors.deepOrange,
                      weight: FontWeight.w300,
                    ),
                  ),
                )
              ],
            ),
            GapRF(50),
            TextRF(
              "Here is your answers :",
              size: 16,
              color: Colors.black87,
              weight: FontWeight.w300,
            ),
            GapRF(10),
            Container(
              height: 60,
              child: ListView.builder(
                scrollDirection: Axis.horizontal,
                itemCount: answerList.length,
                itemBuilder: (context, i) {
                  return TweenAnimationBuilder(
                    tween: Tween<double>(begin: 0, end: 1),
                    curve: Curves.elasticOut,
                    duration: Duration(milliseconds: 800 * i),
                    builder: (BuildContext context, double size, Widget child) {
                      return Transform.scale(
                        scale: size,
                        child: Center(
                          child: Container(
                            width: 40,
                            height: 40,
                            margin: const EdgeInsets.all(5),
                            decoration: new BoxDecoration(
                              gradient: LinearGradient(
                                  colors: answerList[i].IsCorrect != 1 ? [Colors.deepOrange[700], Colors.red[200]] : [Colors.green[700], Colors.green[200]],
                                  begin: Alignment.topRight,
                                  end: Alignment.bottomLeft),
                              borderRadius: new BorderRadius.all(Radius.circular(100)),
                            ),
                            color: answerList[i].IsCorrect == 1 ? Colors.green : Colors.red,
                            child: Center(
                              child: TextRF(
                                (i + 1).toString(),
                                size: 18,
                                color: Colors.black,
                              ),
                            ),
                          ),
                        ),
                      );
                    },
                  );
                },
              ),
            ),
          ],
        ),
        onAccept: () {
          rf.page.back(rf.context);
        },
        onCancel: () => rf.page.back(rf.context));
  }

  insertAnswers() async {
    for (Answer ans in answerList) {
      await rf.db.save(ans);
    }
  }
}
