import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:percent_indicator/percent_indicator.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/models/Quiz.dart';
import 'QuizEngine.dart';

class QuizRF extends StatefulWidget {
  Quiz quiz;
  Color indexColor;
  Color questionTextColor;
  Color optionTextColor;
  Color optionBorderColor;
  Color optionBackgroundColor;
  double optionBorderRadious;
  double optionBorderWidth;
  FontWeight optionFontWeight;
  Color buttonColor;
  int optionsPerRow;
  FontWeight questionFontWeight;
  FontWeight indexFontWeight;
  Function onComplete;
  @override
  _QuizRFState createState() => _QuizRFState();

  QuizRF(
      {this.quiz,
      this.buttonColor,
      this.indexColor,
      this.optionBackgroundColor,
      this.optionBorderColor,
      this.optionBorderRadious,
      this.optionBorderWidth,
      this.optionTextColor,
      this.questionTextColor,
      this.indexFontWeight,
      this.optionFontWeight,
      this.questionFontWeight,
      this.optionsPerRow=1,
      this.onComplete});
}

class _QuizRFState extends State<QuizRF> with TickerProviderStateMixin {
  //region Variables
  QuizEngine quizR;
  Timer _timer;
  int _start = 30;
  Question activeQuestion = Question();
  AnimationController animationController;
  Animation<double> animation, animation2;

  //endregion

  @override
  void initState() {
    animationController = AnimationController(duration: const Duration(milliseconds: 500), vsync: this);
    animation = Tween<double>(begin: 0.0, end: 1.0).animate(animationController);
    animation2 = Tween<double>(begin: -200, end: 0.0).animate(animationController);
    quizR = QuizEngine(quizId: widget.quiz.QuizId, loadType: LoadType.offline);

    test();
  }

  void test() async {
    await quizR.load().then((value) => nextQuestion());
  }

  @override
  void dispose() {
    _timer.cancel();
    animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        decoration: BoxDecoration(
          // Box decoration takes a gradient
          gradient: LinearGradient(
            // Where the linear gradient begins and ends
            begin: Alignment.topLeft,
            end: Alignment.bottomCenter,
            // Add one stop for each color. Stops should increase from 0 to 1
            stops: [0.0, 0.6],
            colors: [
              // Colors are easy thanks to Flutter's Colors class.
              Colors.deepPurple[100].withOpacity(0.4),
              Colors.white,
            ],
          ),
        ),
        child: Stack(
          children: [
            CustomPaint(
              size: Size(ScreenRF.width(context), ScreenRF.height(context)), //You can Replace [WIDTH] with your desired width for Custom Paint and height will be calculated automatically
              painter: paining(),
            ),
            Padding(
              padding: const EdgeInsets.all(15.0),
              child: Center(
                child: Container(
                    child: Column(
                  children: [
                    GapRF(50),
                    Container(
                      padding: const EdgeInsets.all(15),
                      decoration: new BoxDecoration(
                        color: Colors.white,
                        borderRadius: new BorderRadius.all(Radius.circular(12)),
                        boxShadow: <BoxShadow>[
                          BoxShadow(
                            offset: Offset(0, 0),
                            blurRadius: 1,
                            color: Colors.black.withOpacity(.1),
                          )
                        ],
                      ),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.start,
                        children: [
                          Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Container(
                                width: 200,
                                child: TextRF(
                                  widget.quiz.Title,
                                  size: 18,
                                  overflow: TextOverflow.ellipsis,
                                  color: Colors.black87,
                                  weight: FontWeight.bold,
                                ),
                              ),
                              TextRF(
                                widget.quiz.Description,
                                size: 16,
                                color: Colors.grey[700],
                                weight: FontWeight.normal,
                              ),
                              TextRF(
                                "Total Marks :" + widget.quiz.Marks.toString(),
                                size: 15,
                                color: Colors.grey[700],
                                weight: FontWeight.normal,
                              ),
                              Container(
                                  height: 35,
                                  margin: const EdgeInsets.all(10),
                                  width: 90,
                                  decoration: new BoxDecoration(
                                    color: Colors.blue,
                                    borderRadius: new BorderRadius.all(Radius.circular(50)),
                                  ),
                                  child: Center(
                                      child: TextRF(
                                    "Time : " + _start.toString(),
                                    color: Colors.white,
                                    size: 15,
                                  ))),
                            ],
                          ),
                          Expanded(child: SizedBox()),
                          CircularPercentIndicator(
                            radius: 120.0,
                            lineWidth: 8.0,
                            startAngle: 210,
                            animation: true,
                            animationDuration: 800,
                            percent: quizR.getProgressDouble() > 1.0 ? 1.0 : quizR.getProgressDouble(),
                            center: new Text(
                              quizR.getProgressTxt(),
                              style: new TextStyle(fontWeight: FontWeight.bold, fontSize: 35.0),
                            ),
                            backgroundColor: Colors.grey.withOpacity(0.2),
                            circularStrokeCap: CircularStrokeCap.round,
                            rotateLinearGradient: true,
                            linearGradient: new LinearGradient(colors: [Colors.purpleAccent, Colors.orange]),
                            // progressColor: Colors.purple,
                          ),
                        ],
                      ),
                    ),
                    Container(
                      margin: const EdgeInsets.all(10),
                      padding: const EdgeInsets.all(10),
                      width: ScreenRF.width(context) - 50,
                      decoration: new BoxDecoration(
                        color: Colors.white,
                        borderRadius: new BorderRadius.all(Radius.circular(5)),
                        boxShadow: <BoxShadow>[
                          BoxShadow(
                            offset: Offset(0, 0),
                            blurRadius: 1,
                            color: Colors.black.withOpacity(.2),
                          )
                        ],
                      ),
                      child: Column(
                        children: [
                          activeQuestion.ImageUrl == null
                              ? activeQuestion.VideoUrl == null
                                  ? Container()
                                  : Container(
                                      height: 200,
                                      width: ScreenRF.width(context),
                                      color: Colors.black87,
                                      child: VideoPlayerRF(
                                        fromNetwork: true,
                                        url: activeQuestion.VideoUrl,
                                      ),
                                    )
                              : Container(
                                  height: 150,
                                  width: 350,
                                  child: Image.network(activeQuestion.ImageUrl),
                                ),
                          Center(
                              child: TextRF(
                            activeQuestion.Title ?? "no active question",
                            color: Colors.black,
                          )),
                        ],
                      ),
                    ),
                    Expanded(
                      child: AnimatedBuilder(
                          animation: animationController,
                          builder: (context, child) {
                            return Transform.translate(
                              offset: Offset(0, animation2.value),
                              child: Transform.scale(
                                scale: animation.value,
                                child: activeQuestion.Options == null
                                    ? Container()
                                    : QuizOptions(
                                        activeOptions: activeQuestion.Options,
                                        questionType: activeQuestion.QuestionTypeId,
                                  optionsPerRow: widget.optionsPerRow,
                                        onItemTap: onAnswerSelect,
                                      ),
                              ),
                            );
                          }),
                    ),
                    InkWell(
                      onTap: () {
                        nextQuestion();
                      },
                      child: Container(
                          height: 45,
                          margin: const EdgeInsets.all(10),
                          width: 100,
                          decoration: BoxDecoration(
                              borderRadius: BorderRadius.all(
                                Radius.circular(50.0),
                              ),
                              color: Colors.orange,
                              boxShadow: [
                                BoxShadow(
                                  color: Colors.orangeAccent,
                                  spreadRadius: 2,
                                  blurRadius: 10,
                                  offset: Offset(0, 0),
                                ),
                              ]),
                          child: Center(
                              child: TextRF(
                            "NEXT",
                            color: Colors.white,
                          ))),
                    ),
                  ],
                )),
              ),
            ),
          ],
        ),
      ),
    );
  }

  onAnswerSelect(Options selectedOption) async {
    await quizR.submitAnswer(selectedOption);
    if (quizR.activeQuestion.QuestionTypeId == 1) nextQuestion();
  }

  void nextQuestion() {
    animationController.reset();
    animationController.forward();

    var question = quizR.next();
    if (question != null) {
      activeQuestion = question;
      setState(() {
        startTimer();
      });
    } else {
      _timer.cancel();
      showResult();
    }
  }

  void showResult() {
    quizR.insertAnswers();
    DialogRF(context,
        height: 340,
        avatarIcon: FontAwesomeIcons.bookOpen,
        hideAvatar: true,
        avatarIconSize: 25,
        hideDefaultControls: true,
        type: DialogType.popup,
        title: 'Quiz Result',
        subTitle: quizR.quiz.Title,
        body: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Center(
              child: Container(
                height: 80,
                width: 80,
                margin: const EdgeInsets.all(15),
                decoration: BoxDecoration(
                  boxShadow: [BoxShadow(offset: Offset(6, 6), color: Colors.black38, blurRadius: 20), BoxShadow(offset: Offset(-6, -6), color: Colors.white.withOpacity(0.85), blurRadius: 20)],
                  color: Colors.white38,
                  borderRadius: BorderRadius.circular(20.0),
                ),
                child: Center(
                    child: TextRF(
                  quizR.totalEarnedMark.toString(),
                  color: Colors.cyan,
                  size: 30,
                  weight: FontWeight.bold,
                )),
              ),
            ),
            GapRF(15),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: [
                Container(
                  height: 30,
                  padding: const EdgeInsets.symmetric(horizontal: 6),
                  decoration: new BoxDecoration(
                    color: Colors.deepPurpleAccent[100].withOpacity(0.2),
                    borderRadius: new BorderRadius.all(Radius.circular(4)),
                  ),
                  child: Center(
                    child: TextRF(
                      quizR.questions.length.toString() + " Questions",
                      size: 15,
                      color: Colors.deepPurple,
                      weight: FontWeight.w300,
                    ),
                  ),
                ),
                Container(
                  height: 30,
                  padding: const EdgeInsets.symmetric(horizontal: 6),
                  decoration: new BoxDecoration(
                    color: Colors.blue[200].withOpacity(0.3),
                    borderRadius: new BorderRadius.all(Radius.circular(4)),
                  ),
                  child: Center(
                    child: TextRF(
                      quizR.answered.toString() + " Answered",
                      size: 15,
                      color: Colors.blue,
                      weight: FontWeight.w300,
                    ),
                  ),
                ),
                Container(
                  height: 30,
                  padding: const EdgeInsets.symmetric(horizontal: 6),
                  decoration: new BoxDecoration(
                    color: Colors.deepOrange[200].withOpacity(0.3),
                    borderRadius: new BorderRadius.all(Radius.circular(4)),
                  ),
                  child: Center(
                    child: TextRF(
                      (quizR.questions.length - quizR.answered).toString() + " Skipped",
                      size: 15,
                      color: Colors.deepOrange,
                      weight: FontWeight.w300,
                    ),
                  ),
                )
              ],
            ),
            GapRF(50),
            TextRF(
              "Here is your answers :",
              size: 16,
              color: Colors.black87,
              weight: FontWeight.w300,
            ),
            GapRF(10),
            Container(
              height: 60,
              child: ListView.builder(
                scrollDirection: Axis.horizontal,
                itemCount: quizR.answerList.length,
                itemBuilder: (context, i) {
                  return TweenAnimationBuilder(
                    tween: Tween<double>(begin: 0, end: 1),
                    curve: Curves.elasticOut,
                    duration: Duration(milliseconds: 800 * i),
                    builder: (BuildContext context, double size, Widget child) {
                      return Transform.scale(
                        scale: size,
                        child: Center(
                          child: Container(
                            width: 40,
                            height: 40,
                            margin: const EdgeInsets.all(5),
                            decoration: new BoxDecoration(
                              gradient: LinearGradient(
                                  colors: quizR.answerList[i].IsCorrect != 1 ? [Colors.deepOrange[700], Colors.red[200]] : [Colors.green[700], Colors.green[200]],
                                  begin: Alignment.topRight,
                                  end: Alignment.bottomLeft),
                              borderRadius: new BorderRadius.all(Radius.circular(100)),
                            ),
                            //color: quizR.answerList[i].IsCorrect == 1 ? Colors.green : Colors.red,
                            child: Center(
                              child: TextRF(
                                (i + 1).toString(),
                                size: 18,
                                color: Colors.black,
                              ),
                            ),
                          ),
                        ),
                      );
                    },
                  );
                },
              ),
            ),
          ],
        ),
        onAccept: () {
      if(widget.onComplete==null)
          rf.page.back(context);
      else widget.onComplete.call();
        },
        onCancel: ()  {
          if(widget.onComplete==null)
            rf.page.back(context);
          else widget.onComplete.call();
    });
  }

  void startTimer() {
    if (_timer != null) _timer.cancel();
    _start = activeQuestion.Duration??100;
    const oneSec = const Duration(seconds: 1);
    _timer = new Timer.periodic(
      oneSec,
      (Timer timer) {
        if (_start == 0) {
          setState(() {
            timer.cancel();
            onAnswerSelect(null);
            nextQuestion();
          });
        } else {
          setState(() {
            _start--;
          });
        }
      },
    );
  }
}

class QuizOptions extends StatefulWidget {
  List<Options> activeOptions = [];
  int questionType = 0;
  Color optionTextColor;
  Color optionBorderColor;
  Color optionBackgroundColor;
  Color indexColor;
  double optionBorderRadious;
  int optionsPerRow;
  double optionBorderWidth;
  FontWeight optionFontWeight;
  Function onItemTap;

  @override
  _QuizOptionsState createState() => _QuizOptionsState();

  QuizOptions(
      {this.activeOptions,
      this.questionType,
      this.optionTextColor,
      this.indexColor,
      this.optionBorderColor,
      this.optionBackgroundColor,
      this.optionBorderRadious = 8,
      this.optionBorderWidth = 0,
      this.optionsPerRow = 1,
      this.optionFontWeight = FontWeight.normal,
      this.onItemTap});
}

class _QuizOptionsState extends State<QuizOptions> {
  Timer _timer;
  String temp = "ssdf";
  List<String> indexAlphabet = ["A", "B", "C", "D", "E", "F", "G", "H"];
  List<String> indexNumber = ["1", "2", "3", "4", "5", "6", "7", "8"];
  String selectedoption = "";
  List<Options> markedOptions = [];

  @override
  void dispose() {
    if (_timer != null) _timer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (temp != widget.activeOptions[1].OptionsId) {
      selectedoption = "";
      temp = widget.activeOptions[1].OptionsId;
    }

    return widget.questionType==5?GridView.builder(
      shrinkWrap: true,
      itemCount: widget.activeOptions.length,
      gridDelegate: new SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: widget.optionsPerRow),
      itemBuilder: (context, i) {
        return InkWell(
          onTap: () {
            markedOptions.add(widget.activeOptions[i]);
            setState(() {

            });
          },
          onLongPress: (){
            DialogRF(
              context,
              title: 'QuizRF',
              height: 120,
              avatarIcon: Icons.backup,
              avatarBackground: Colors.grey,
              blurAmount: 5,
              subTitle: "",
              body: Container(
                margin: const EdgeInsets.symmetric(vertical: 20),
                child: Center(
                    child: TextRF(
                      "Is this the last word ?",
                      align: TextAlign.center,
                      color: Colors.black,
                      size: 18,
                    )),
              ),
              type: DialogType.popup,
              hideDefaultControls: false,
              onAccept: () async {

              },
            );
          },
          child: Container(
            width: 100,
            height:100,
            margin: const EdgeInsets.all(10),
            decoration: new BoxDecoration(
                color: markedOptions.contains(widget.activeOptions[i])?Colors.red:Colors.white,
                borderRadius: new BorderRadius.all(Radius.circular(widget.optionBorderRadious ?? 30)),
            border: Border.all(color: Colors.cyan,width: 1)),
            child: Center(child:  TextRF(
              widget.activeOptions[i].Title ?? "n/a",
              color: widget.optionTextColor ?? Colors.black87,
            ),),
          ),
        );
      },
    ):widget.activeOptions[1].Image == null
        ? ListView.builder(
            itemCount: widget.activeOptions.length,
            itemBuilder: (context, i) {
              return Column(
                children: [
                  GapRF(5),
                  InkWell(
                    onTap: () {
                      if (widget.questionType == 1) {
                        if (selectedoption.isEmpty) {
                          setState(() {});
                          selectedoption = i.toString();
                          startTimer(widget.activeOptions[i]);
                        }
                      } else {
                        if (!selectedoption.contains(i.toString())) {
                          setState(() {});
                          selectedoption = selectedoption + i.toString();
                          widget.onItemTap(widget.activeOptions[i]);
                        }
                      }
                    },
                    child: Container(
                      height: 45,
                      margin: const EdgeInsets.all(10),
                      width: ScreenRF.width(context) - 100,
                      decoration: new BoxDecoration(
                          color: selectedoption.contains(i.toString()) ? Colors.blue[300].withOpacity(0.4) : Colors.blueGrey[100].withOpacity(0.4),
                          borderRadius: new BorderRadius.all(Radius.circular(25))),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Container(
                            height: 40,
                            width: 40,
                            margin: const EdgeInsets.only(left: 2),
                            decoration: new BoxDecoration(color: selectedoption == i ? Colors.blue[300] : Colors.white, borderRadius: new BorderRadius.all(Radius.circular(25))),
                            child: Center(
                                child: TextRF(
                              indexAlphabet[i],
                              color: Colors.purple,
                            )),
                          ),
                          GapRF(10),
                          TextRF(
                            widget.activeOptions[i].Title ?? "no active question",
                            color: Colors.grey[600],
                          ),
                        ],
                      ),
                    ),
                  ),
                  GapRF(2),
                ],
              );
            },
          )
        : GridView.builder(
            shrinkWrap: true,
            itemCount: widget.activeOptions.length,
            gridDelegate: new SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 2),
            itemBuilder: (context, i) {
              return InkWell(
                onTap: () {
                  if (widget.questionType == 1) {
                    if (selectedoption.isEmpty) {
                      setState(() {});
                      selectedoption = i.toString();
                      startTimer(widget.activeOptions[i]);
                    }
                  } else {
                    if (!selectedoption.contains(i.toString())) {
                      setState(() {});
                      selectedoption = selectedoption + i.toString();
                      widget.onItemTap(widget.activeOptions[i]);
                    }
                  }
                },
                child: Stack(
                  children: [
                    Container(
                      width: ScreenRF.width(context),
                      height: (ScreenRF.width(context) / 2) + 100,
                      margin: const EdgeInsets.all(10),
                      decoration: new BoxDecoration(
                          color: selectedoption.contains(i.toString()) ? Colors.blue[300].withOpacity(0.4) : widget.optionBackgroundColor ?? Colors.blueGrey[100].withOpacity(0.4),
                          borderRadius: new BorderRadius.all(Radius.circular(widget.optionBorderRadious ?? 25))),
                      child: Column(
                        children: [
                          Container(
                            width: (ScreenRF.width(context) / 2) - 90,
                            height: (ScreenRF.width(context) / 2) - 90,
                            margin: const EdgeInsets.all(10),
                            child: Image.network(
                              widget.activeOptions[i].Image,
                              fit: BoxFit.contain,
                            ),
                          ),
                          GapRF(5),
                          TextRF(
                            widget.activeOptions[i].Title ?? "no active question",
                            color: widget.optionTextColor ?? Colors.grey[600],
                          ),
                        ],
                      ),
                    ),
                    Positioned(
                      top: 12,
                      left: 12,
                      child: Container(
                        height: 40,
                        width: 40,
                        margin: const EdgeInsets.only(left: 2),
                        decoration: new BoxDecoration(color: selectedoption == i ? Colors.blue[300] : Colors.white, borderRadius: new BorderRadius.all(Radius.circular(25))),
                        child: Center(
                            child: TextRF(
                          indexAlphabet[i],
                          color: widget.indexColor ?? Colors.purple,
                        )),
                      ),
                    ),
                  ],
                ),
              );
            },
          );
  }

  getOptionsView(){

  }

  void startTimer(Options option) {
    if (_timer != null) _timer.cancel();
    int _start = 1;
    const oneSec = const Duration(milliseconds: 200);
    _timer = new Timer.periodic(
      oneSec,
      (Timer timer) {
        if (_start == 0) {
          setState(() {
            timer.cancel();
            selectedoption = "";
            widget.onItemTap(option);
          });
        } else {
          _start--;
        }
      },
    );
  }
}

class paining extends CustomPainter {
  @override
  void paint(Canvas canvas, Size size) {
    Paint paint_0 = new Paint()
      ..color = Colors.blueGrey[50].withOpacity(0.0)
      ..style = PaintingStyle.fill
      ..strokeWidth = 1;

    Path path_0 = Path();
    path_0.moveTo(0, 0);
    path_0.lineTo(size.width * 0.8272750, size.height * 0.0825457);
    path_0.lineTo(size.width, size.height);
    path_0.lineTo(size.width, size.height * 0.3500844);
    path_0.lineTo(size.width, 0);
    path_0.lineTo(0, 0);
    path_0.close();

    canvas.drawPath(path_0, paint_0);

    Paint paint_1 = new Paint()
      ..color = Colors.blueGrey[50].withOpacity(0.1)
      ..style = PaintingStyle.fill
      ..strokeWidth = 1;

    Path path_1 = Path();
    path_1.moveTo(size.width * 0.4661500, size.height * 0.0986357);
    path_1.lineTo(size.width * 0.7433750, size.height * 0.1262869);
    path_1.lineTo(size.width * 0.8012500, size.height * 0.4336709);
    path_1.lineTo(size.width * 0.8012500, size.height * 0.2159072);
    path_1.lineTo(size.width * 0.8012500, size.height * 0.0986357);
    path_1.lineTo(size.width * 0.4661500, size.height * 0.0986357);
    path_1.close();

    canvas.drawPath(path_1, paint_1);

    Paint paint_2 = new Paint()
      ..color = Colors.purple.withOpacity(0.05)
      ..style = PaintingStyle.fill
      ..strokeWidth = 1;

    Path path_2 = Path();
    path_2.moveTo(0, size.height * 0.1993249);
    path_2.quadraticBezierTo(size.width * 0.0452500, size.height * 0.4534599, size.width * 0.3940000, size.height * 0.5104501);
    path_2.quadraticBezierTo(size.width * 0.9246250, size.height * 0.5624051, size.width * 0.2574000, size.height * 0.8945570);
    path_2.lineTo(0, size.height);
    path_2.lineTo(0, size.height * 0.7489873);
    path_2.lineTo(0, size.height * 0.3789592);

    canvas.drawPath(path_2, paint_2);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
