import 'dart:async';
import 'dart:ui';
import 'package:flutter/rendering.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:flutter/services.dart';
import '../Db.dart';

enum DialogType { full, bottom, popup }

class DialogRF {
  final BuildContext context;
  final String title;
  final String subTitle;
  final Widget body;
  final DialogType type;
  final bool hideDefaultControls;
  final Function onAccept;
  final Function onCancel;
  final double height;
  final IconData avatarIcon;
  final String avatarImage;
  final double avatarIconSize;
  final double blurAmount;
  final Color avatarBackground;
  final bool hideAvatar;
  final fullWindowDialogScrollEffect;

  DialogRF(
    this.context, {
    this.title = "Missing Title",
    this.subTitle = "Missing Sub Title",
    this.body,
    this.type = DialogType.full,
    this.hideDefaultControls = false,
    this.onAccept,
    this.onCancel,
    this.height = 300,
    this.avatarIcon = FontAwesomeIcons.search,
    this.avatarImage,
    this.avatarIconSize,
    this.blurAmount = 3,
    this.avatarBackground,
    this.hideAvatar = false,
    this.fullWindowDialogScrollEffect = ScrollPhysicsRF.bounce,
  }) {
    if (type == DialogType.full) {
      var content = body ?? Text('Missing Content');
      var fullDlg = Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => FullWindowDialog(
            title: title,
            body: (fullWindowDialogScrollEffect == ScrollPhysicsRF.bounce)
                ? ScrollConfiguration(
                    behavior: BounceScrollBehavior(),
                    child: content,
                  )
                : content,
            hidePageHeader: hideDefaultControls,
            onAccept: onAccept,
            onCancel: onCancel,
          ),
          fullscreenDialog: false,
        ),
      );
      fullDlg.whenComplete(() {});
    } else if (type == DialogType.popup) {
      showDialog<String>(
        context: context, barrierDismissible: false, // user must tap button!
        builder: (BuildContext context) {
          return PopupWindowDialogRF(
            title: title,
            subTitle: subTitle,
            avatarIcon: avatarIcon,
            avatarImage: avatarImage,
            avatarSize: avatarIconSize,
            height: height,
            avatarBackground: avatarBackground,
            blurAmount: blurAmount,
            body: body ?? Text('Missing Content'),
            onAccept: onAccept,
            onCancel: onCancel,
          );
        },
      );
    } else if (type == DialogType.bottom) {
      BottomSheetRF(
        title: title,
        subTitle: subTitle,
        context: context,
        body: body,
        onAccept: onAccept,
        onCancel: onCancel,
        height: height,
        hideDefaultControls: hideDefaultControls,
        avatarIcon: avatarIcon,
        avatarImage: avatarImage,
        avatarSize: avatarIconSize,
        hideAvatar: hideAvatar,
      );
    }
  }

  static refresh() {
    (PageRF.currentDialogContext as Element).reassemble();
  }
}

//Type 1
class PopupWindowDialogRF extends StatefulWidget {
  final String title;
  final String subTitle;
  final IconData avatarIcon;
  final String avatarImage;
  final double avatarSize;
  final double height;
  final Color avatarBackground;
  final Widget body;
  final double blurAmount;
  final Function onAccept;
  final Function onCancel;

  const PopupWindowDialogRF(
      {Key key,
      this.title,
      this.subTitle,
      this.avatarIcon,
      this.avatarImage,
      this.avatarSize = 24,
      this.height = 100,
      this.blurAmount = 3,
      this.avatarBackground = Colors.blue,
      this.body,
      this.onCancel,
      this.onAccept})
      : super(key: key);

  @override
  PopupWindowDialogRFState createState() {
    return new PopupWindowDialogRFState();
  }
}

class PopupWindowDialogRFState extends State<PopupWindowDialogRF> {
  final _resetKey = GlobalKey<FormState>();
  bool _resetValidate = false;
  StreamController<bool> rebuild = StreamController<bool>();

  dialogActions() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.end,
      crossAxisAlignment: CrossAxisAlignment.end,
      children: <Widget>[
        FlatButton(
          child: Text(
            'CANCEL',
            style: TextStyle(color: Colors.red, fontWeight: FontWeight.w500),
          ),
          onPressed: () {
            Navigator.of(context).pop("");
            if (widget.onCancel != null) widget.onCancel.call();
          },
        ),
        widget.onAccept == null
            ? Container()
            : FlatButton(
                child: Text(
                  'ACCEPT',
                  style: TextStyle(color: Colors.blue, fontWeight: FontWeight.w500),
                ),
                onPressed: () async {
                  Navigator.of(context).pop("");
                  if (widget.onAccept != null) {
                    var ret = await widget.onAccept();
                    /*if (ret) {
                  Navigator.of(context).pop("");
                }*/
                  }
                },
              ),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return BackdropFilter(
      filter: ImageFilter.blur(sigmaX: widget.blurAmount, sigmaY: widget.blurAmount),
      child: Container(
        decoration: BoxDecoration(shape: BoxShape.rectangle, boxShadow: [
          BoxShadow(color: Colors.black.withOpacity(0.0), offset: Offset(0, 0), spreadRadius: 0, blurRadius: 0),
        ]),
        child: Dialog(
          shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(20))),
          backgroundColor: Colors.white.withOpacity(0.82),
          child: Form(
            key: _resetKey,
            //autovalidate: _resetValidate, //Blocked by Imrose
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                SizedBox(
                  height: 10,
                ),
                Padding(
                  padding: const EdgeInsets.only(left: 12.0),
                  child: Row(
                    children: <Widget>[
                      CircleAvatar(
                        radius: 22,
                        backgroundColor: widget.avatarBackground,
                        child: widget.avatarImage == null
                            ? Icon(widget.avatarIcon, size: widget.avatarSize, color: Colors.white)
                            : Image.asset(widget.avatarImage, height: widget.avatarSize, width: widget.avatarSize),
                      ),
                      SizedBox(
                        width: 10,
                      ),
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          Text(widget.title, style: TextStyle(fontSize: 20.0, fontWeight: FontWeight.bold)),
                          Text(widget.subTitle, style: TextStyle(fontSize: 16.0, fontWeight: FontWeight.w400, color: Colors.black54)),
                        ],
                      )
                    ],
                  ),
                ),
                SizedBox(
                  height: 5,
                ),
                Container(
                    padding: EdgeInsets.only(left: 20, top: 10, bottom: 10, right: 20),
                    height: widget.height,
                    child: SingleChildScrollView(
                      child: widget.body,
                    )),
                SizedBox(
                  height: 10,
                ),
                Container(
                  margin: const EdgeInsets.only(bottom: 10),
                  child: dialogActions(),
                  color: Colors.transparent,
                )
              ],
            ),
          ),
        ),
      ),
    );
  }
}

//Type 2

class FullWindowDialog extends StatefulWidget {
  final String title;
  final Widget body;
  final bool hidePageHeader;
  final Function onAccept;
  final Function onCancel;
  final key;

  FullWindowDialog({
    this.key,
    this.title,
    this.body,
    this.hidePageHeader = false,
    this.onAccept,
    this.onCancel,
  }) : super(key: key);

  @override
  FullWindowDialogState createState() => new FullWindowDialogState();
}

class FullWindowDialogState extends State<FullWindowDialog> {
  @override
  void didChangeDependencies() {
    PageRF.currentDialogContext = context;
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: widget.hidePageHeader
          ? null
          : AppBar(
              backgroundColor: Colors.white,
              elevation: 0,
              leading: GestureDetector(
                onTap: () {
                  if (widget.onCancel != null) widget.onCancel.call();
                  Navigator.of(context).pop("");
                },
                child: Container(
                  margin: EdgeInsets.all(10),
                  width: 30,
                  height: 30,
                  decoration: BoxDecoration(boxShadow: [
                    BoxShadow(color: Colors.grey.withOpacity(0.5), blurRadius: 6.0, spreadRadius: 3.0),
                    //BoxShadow(offset: Offset(0, 0), color: Colors.black38, blurRadius: 8),
                  ], color: Colors.white, borderRadius: new BorderRadius.all(Radius.circular(100))),
                  child: Icon(
                    Icons.close,
                    size: 25,
                    color: Colors.red,
                  ),
                ),
              ),
              title: TextRF(
                widget.title,
                color: Colors.black,
                weight: FontWeight.w500,
              ),
              actions: [
                new FlatButton(
                  onPressed: () {
                    if (widget.onAccept != null) {
                      SystemChannels.textInput.invokeMethod('TextInput.hide');
                      var ret = widget.onAccept();

                      if (ret == null) {
                        assert(ret == null, 'Expected a boolean return here!');
                      }
                      if (ret) {
                        Navigator.of(context).pop(true);
                      }
                    }
                  },
                  child: Container(
                    padding: EdgeInsets.only(left: 20, right: 20, top: 8, bottom: 8),
                    decoration: new BoxDecoration(boxShadow: [
                      BoxShadow(color: Colors.grey.withOpacity(0.5), blurRadius: 6.0, spreadRadius: 3.0),
                      //BoxShadow(offset: Offset(0, 0), color: Colors.blue[300], blurRadius: 5),
                    ], color: Colors.white, borderRadius: new BorderRadius.all(Radius.circular(100))),
                    child: TextRF(
                      "ACCEPT",
                      size: 15,
                      color: Colors.blue[800],
                    ),
                  ),
                ),
              ],
            ),
      body: Container(
        margin: const EdgeInsets.only(top: 10),
        //padding: EdgeInsets.only(left: 20, top: 20, bottom: 20, right: 20),
        alignment: Alignment(0.0, 0.0), child: widget.body, color: Colors.white,
      ),
    );
  }
}

// Type 3
class BottomSheetRF {
  String title;
  String subTitle;
  BuildContext context;
  Widget body;
  Function onAccept;
  Function onCancel;
  double height;
  bool hideDefaultControls;
  final IconData avatarIcon;
  final double avatarSize;
  final String avatarImage;
  final bool hideAvatar;

  BottomSheetRF({
    this.title,
    this.subTitle,
    this.context,
    this.body,
    this.onAccept,
    this.onCancel,
    this.height = 300,
    this.hideDefaultControls,
    this.avatarIcon = Icons.search,
    this.avatarImage,
    this.avatarSize = 32,
    this.hideAvatar = false,
  }) {
    showModalBottomSheet<void>(
        isScrollControlled: true,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(0.0),
        ),
        backgroundColor: Colors.transparent,
        context: context,
        builder: (BuildContext context) {
          return Padding(
            padding: EdgeInsets.only(bottom: MediaQuery.of(context).viewInsets.bottom),
            child: Container(
              height: height,
              //color: Colors.white,
              child: Padding(
                padding: const EdgeInsets.all(12.0),
                child: Container(
                  color: Colors.transparent,
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Row(
                        children: <Widget>[
                          hideAvatar
                              ? Container()
                              : CircleAvatar(
                                  radius: 24,
                                  backgroundColor: avatarImage == null ? Colors.blue : Colors.white,
                                  child: avatarImage == null ? Icon(avatarIcon, size: avatarSize, color: Colors.white) : Image.asset(avatarImage, height: avatarSize, width: avatarSize),
                                ),
                          SizedBox(
                            width: 10,
                          ),
                          Expanded(
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                TextRF(
                                  title,
                                  style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
                                ),
                                TextRF(
                                  subTitle,
                                  style: TextStyle(fontSize: 16, fontWeight: FontWeight.w600, color: Colors.black54),
                                ),
                              ],
                            ),
                          ),
                          Container(
                            color: Colors.transparent,
                            child: IconButton(
                              padding: EdgeInsets.zero,
                              icon: Icon(Icons.clear, size: 34, color: Colors.black45),
                              color: Colors.black,
                              onPressed: () {
                                Navigator.of(context).pop("");
                              },
                            ),
                          )
                        ],
                      ),
                      SizedBox(height: 15),
                      Expanded(child: body),
                      if (!hideDefaultControls) Divider(color: Colors.black54),
                      if (!hideDefaultControls) dialogActions(onAccept, onCancel),
                    ],
                  ),
                ),
              ),
              decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.only(topLeft: Radius.circular(12), topRight: Radius.circular(12))),
            ),
          );
        });
  }

  dialogActions(onAccept, onCancel) {
    return Container(
      height: 40,
      color: Colors.transparent,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.end,
        children: <Widget>[
          FlatButton(
            padding: EdgeInsets.zero,
            child: Text(
              'CANCEL',
              style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold),
            ),
            onPressed: () {
              Navigator.of(context).pop("");
              if (onCancel != null) onCancel.call();
            },
          ),
          FlatButton(
              padding: EdgeInsets.zero,
              child: Text(
                'ACCEPT',
                style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold),
              ),
              onPressed: () {
                Navigator.of(context).pop("");
                if (onAccept != null) onAccept.call();
              })
        ],
      ),
    );
  }
}

class DialogItemListTileRF extends StatefulWidget {
  String title;
  String subTitle;
  IconData icon;
  double size;
  Function onTap;

  DialogItemListTileRF({
    this.title,
    this.subTitle,
    this.icon,
    this.size,
    this.onTap,
  });

  @override
  _DialogItemListTileRFState createState() => _DialogItemListTileRFState();
}

class _DialogItemListTileRFState extends State<DialogItemListTileRF> {
  @override
  Widget build(BuildContext context) {
    return FlatButton(
      color: Colors.black12.withOpacity(0.05),
      child: Padding(
        padding: EdgeInsets.only(top: 10, bottom: 10, left: 0, right: 0),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Icon(widget.icon, size: widget.size ?? 50, color: Colors.black45),
            SizedBox(width: 15),
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Text(
                    widget.title,
                    style: TextStyle(fontWeight: FontWeight.bold, fontSize: 20),
                  ),
                  Text(widget.subTitle),
                ],
              ),
            )
          ],
        ),
      ),
      onPressed: () => widget.onTap.call(),
    );
  }
}
