import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_custom_clippers/flutter_custom_clippers.dart';
import 'package:rapidfire/RapidFire.dart';

import '../GlobalRF.dart';
import '../GlobalScopeRF.dart';
import '../store/StoreRF.dart';
import 'StyleRF.dart';
import 'WidgetsRF.dart';

class DrawerHeaderItemsRF {
  String title;
  String subTitle;
  String avatarImagePath;
  Color avatarBorderColor;
  double avatarBorderSize;
  TemplateRF template;

  DrawerHeaderItemsRF(
    this.template, {
    this.title,
    this.subTitle,
    this.avatarImagePath,
    this.avatarBorderColor,
    this.avatarBorderSize,
  });
}

class DrawerAncestorRF extends InheritedWidget {
  final Function onMenu;

  const DrawerAncestorRF({Key key, @required Widget child, this.onMenu})
      : assert(child != null),
        super(key: key, child: child);

  @override
  bool updateShouldNotify(DrawerAncestorRF old) {
    return true;
  }
}

enum DrawerHeaderInfoDefaultType { app, user }

class DrawerRF extends StatefulWidget {
  Widget menuHeader;
  List<DrawerItemRF> menuBody;
  Widget menuFooter; //DrawerItemRF
  Function pageBuilder;
  Widget background;
  CustomClipper clipper;
  DrawerHeaderInfoDefaultType headerInfoDefaultType;
  TemplateRF template;
  Function onDrawerOpen;
  Function onDrawerClose;
  Function onMenuClick;

  DrawerRF({
    this.menuHeader,
    this.menuBody,
    this.menuFooter,
    this.pageBuilder,
    this.background,
    this.clipper,
    this.headerInfoDefaultType = DrawerHeaderInfoDefaultType.user,
    this.template,
    this.onDrawerOpen,
    this.onDrawerClose,
    this.onMenuClick,
  });

  @override
  State<StatefulWidget> createState() {
    return _DrawerPageRF();
  }
}

class _DrawerPageRF extends State<DrawerRF> with SingleTickerProviderStateMixin {
  bool _menuOpened = false;
  bool _isDraggingMenu = false;

  double _drawerWidth = 0.66;
  double _minScale = 0.86;

  Animation<double> animation, scaleAnimation;
  Animation<BorderRadius> radiusAnimation;
  AnimationController animationController;

  _open() {
    animationController.forward();
    setState(() {
      if (widget.onDrawerOpen != null) widget.onDrawerOpen();
      _menuOpened = true;
    });
  }

  _close() {
    animationController.reverse();
    setState(() {
      if (widget.onDrawerClose != null) widget.onDrawerClose();
      _menuOpened = false;
    });
  }

  _onMenuPressed() {
    _menuOpened ? _close() : _open();
  }

  _finishDrawerAnimation() {
    if (_isDraggingMenu) {
      var opened = false;
      setState(() {
        _isDraggingMenu = false;
      });
      if (animationController.value >= 0.4) {
        animationController.forward();
        opened = true;
      } else {
        animationController.reverse();
      }
      setState(() {
        _menuOpened = opened;
      });
    }
  }

  @override
  void initState() {
    super.initState();

    animationController = AnimationController(duration: const Duration(milliseconds: 500), vsync: this);

    animation = Tween<double>(begin: 0.0, end: 1.0).animate(animationController);

    scaleAnimation = Tween<double>(begin: 1.0, end: _minScale).animate(animationController);
    radiusAnimation = BorderRadiusTween(begin: BorderRadius.circular(0.0), end: BorderRadius.circular(32.0))
        .animate(CurvedAnimation(parent: animationController, curve: Curves.ease));
  }

  getHeader(dynamic item) {
    if (item is Widget) {
      return item;
    } else if (item is DrawerHeaderItemsRF) {
      return StoreRF.scaffold().outline.storeOutline().header.drawer.drawerHeader(
            item.title,
            item.subTitle,
            item.avatarImagePath,
            item.avatarBorderColor,
            template: item.template,
            context: context,
          );
    }
    return null;
  }

  @override
  Widget build(BuildContext context) {
//    var clintTemplate =
//        Libs.config?.app?.pageOutline?.customOutline()?.pageHeader?.drawer?.template;
//    var b = StoreRF.scaffold().outline.drawerBodyItems(context, template: widget.template);
    var header = getHeader(widget.menuHeader) ??
        getHeader(Libs.config.app?.outline?.customOutline()?.appBar?.drawer?.menuHeader) ??
        StoreRF.scaffold().outline.storeOutline().header.drawer.drawerHeader(
              widget.headerInfoDefaultType == DrawerHeaderInfoDefaultType.user ? Libs.appUser.Name : Libs.config.app.title,
              widget.headerInfoDefaultType == DrawerHeaderInfoDefaultType.user
                  ? Libs.appUser.Designation
                  : Libs.config.app.subTitle,
              widget.headerInfoDefaultType == DrawerHeaderInfoDefaultType.user
                  ? Libs.appUser.ProfileImagePath
                  : 'assets/images/userImage.webp',
              null,
              template: widget.template ?? TemplateRF(TemplateNameRF.modern),
              context: context,
            );

    var body = WidgetsRF.selectWidget(
      widget.menuBody,
      Libs.config?.app?.outline?.customOutline()?.appBar?.drawer?.menuBody,
      StoreRF.scaffold().outline.storeOutline().header.drawer.menuBody(
            context,
            template: widget.template ?? TemplateRF(TemplateNameRF.modern),
            onTap: widget.onMenuClick,
          ),
    );

    var footer = WidgetsRF.selectWidget(
      widget.menuFooter,
      Libs.config.app?.outline?.customOutline()?.appBar?.drawer?.menuFooter,
      StoreRF.scaffold().outline.storeOutline().header.drawer.drawerFooter(
            context,
            template: widget.template ?? TemplateRF(TemplateNameRF.modern),
          ),
    );
//        widget.menuFooter ??
//        Libs.config.app?.pageOutline?.customOutline()?.drawer?.menuFooter ??
//        StoreRF.scaffold().outline.drawerFooter(context, template: widget.template);
    //TemplateRF(TemplateNameRF.modern, 'Oval')

    if (widget.template != null) {
      if (widget.template.name == TemplateNameRF.modern && widget.template.version == 'Oval') {
        widget.clipper = OvalRightBorderClipper();
      }
    }

    var drawerMenuButton = ClipPath(
      clipper: widget.clipper,
      child: Container(
        color: Colors.red,
        child: SingleChildScrollView(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              header,
              AnimatedBuilder(
                animation: animationController,
                builder: (context, child) {
                  return Transform.scale(
                    scale: animation.value,
                    child: Center(
                      child: body,
                    ),
                  );
                },
              ),
              if (footer != null) footer,
            ],
          ),
        ),
      ),
    );

    var standalone = DrawerAncestorRF(
      onMenu: _onMenuPressed,
      child: GestureDetector(
        onHorizontalDragStart: (DragStartDetails details) async {
          setState(() {
            _isDraggingMenu = (_menuOpened && details.globalPosition.dx / MediaQuery.of(context).size.width >= _drawerWidth) ||
                (!_menuOpened && details.globalPosition.dx <= 8.0);
          });
        },
        onHorizontalDragUpdate: (DragUpdateDetails details) async {
          if (_isDraggingMenu) {
            animationController.value = details.globalPosition.dx / MediaQuery.of(context).size.width;
            if (animationController.value < 0.3) {
              widget.onDrawerClose();
            } else
              widget.onDrawerOpen();
          }
        },
        onHorizontalDragEnd: (DragEndDetails details) async {
          _finishDrawerAnimation();
        },
        onHorizontalDragCancel: () async {
          _finishDrawerAnimation();
        },
        child: Stack(
          children: <Widget>[
            drawerMenu(animation, animationController, context, header, body, footer), //widget.menuFooter
            drawerContentPage(
              animation,
              scaleAnimation,
              radiusAnimation,
              animationController,
              context,
              widget.pageBuilder == null ? null : widget.pageBuilder(_onMenuPressed),
            ),
          ],
        ),
      ),
    );

    if (widget.pageBuilder == null)
      return drawerMenuButton;
    else
      return standalone;
  }
}

Widget drawerContentPage(Animation<double> animation, scaleAnimation, Animation<BorderRadius> radiusAnimation,
    AnimationController animationController, BuildContext context, Widget mainPage) {
  double _drawerWidth = 0.66;
  double _minScale = 0.86;

  return AnimatedBuilder(
    animation: animationController,
    builder: (context, child) {
      return Transform.scale(
        scale: scaleAnimation.value,
        child: Transform.translate(
          offset: Offset((MediaQuery.of(context).size.width * _drawerWidth) * animation.value, 0.0),
          child: Stack(
            children: <Widget>[
              Column(
                children: <Widget>[
                  Expanded(
                    child: Container(
                      padding: EdgeInsets.symmetric(vertical: 32.0),
                      child: ClipRRect(
                        borderRadius: BorderRadius.all(Radius.circular(44.0)),
                        child: Container(
                          decoration: BoxDecoration(
                            color: Colors.white.withAlpha(128),
                          ),
                        ),
                      ),
                    ),
                  ),
                ],
              ),
              Padding(
                padding: EdgeInsets.only(left: animation.value * 16.0),
                child: ClipRRect(
                  borderRadius: radiusAnimation.value,
                  child: Container(
                    color: Colors.red,
                    child: Scaffold(
                      body: mainPage, //?? page(_onMenuPressed, context),
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      );
    },
  );
}

Widget drawerMenu(Animation<double> animation, AnimationController animationController, BuildContext context, Widget header,
    List<DrawerItemRF> body, dynamic footer) {
  var menu = Container(
    decoration: rf.style.gradient.lefttLightRighrDark('ffffff', '8e9aaf'),
    child: AnimatedBuilder(
      animation: animationController,
      builder: (context, child) {
        return Transform.scale(
          scale: animation.value,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Padding(
                padding: const EdgeInsets.only(top: 55.0, left: 18),
                child: header,
              ),
              Expanded(child: SizedBox()),
              //header
              Container(
                color: Colors.transparent,
                height: MediaQuery.of(context).size.height - 200,
                child: SingleChildScrollView(
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    mainAxisSize: MainAxisSize.min,
                    children: body,
                  ),
                ),
              ),
              Expanded(child: SizedBox()),
              Padding(
                padding: const EdgeInsets.only(left: 18, bottom: 15),
                child: footer,
              )
            ],
          ),
        );
      },
    ),
  );
  return menu;
}

//??
class DrawerItemRF extends StatefulWidget {
  String text;
  double textSize;
  FontWeight textWeight;
  IconData icon;
  Color color;
  double iconSize;
  bool separator;
  bool badge;
  dynamic route;
  Function onTap;
  double padding;
  TemplateRF template;

  DrawerItemRF({
    this.icon,
    this.text,
    this.color = Colors.black54,
    this.iconSize = 20,
    this.textSize = 16,
    this.textWeight = FontWeight.w500,
    this.route,
    this.separator = false,
    this.badge = false,
    this.padding = 12.0,
    this.onTap,
    this.template,
  });

  @override
  _DrawerItemRFState createState() => _DrawerItemRFState();
}

class _DrawerItemRFState extends State<DrawerItemRF> with SingleTickerProviderStateMixin {
  @override
  Widget build(BuildContext context) {
    const kPrimaryColor = Color(0xFF457b9d);
    const kTextColor = Color(0xFF1d3557);
    const kBackgroundColor = Color(0xFFF9F8FD);
    return Material(
      color: Colors.transparent,
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Padding(
          padding: const EdgeInsets.only(left: 10),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              InkWell(
                onTap: () {
                  widget.route == null
                      ? widget.onTap == null
                          ? null
                          : widget.onTap()
                      : Navigator.of(context).push(CupertinoPageRoute(
                          fullscreenDialog: true,
                          builder: (BuildContext context) {
                            return widget.route;
                          },
                        ));
                  //rf.call(context, widget.route);
                },
                child: Padding(
                  padding: EdgeInsets.only(bottom: widget.padding, top: widget.padding),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Row(
                        children: <Widget>[
                          Container(
                            margin: EdgeInsets.only(left: 0),
                            width: 40,
                            height: 40,
                            decoration: BoxDecoration(
                              color: kBackgroundColor,
                              borderRadius: BorderRadius.circular(100),
                              boxShadow: [
                                BoxShadow(
                                  blurRadius: 8,
                                  color: Color(0Xff8e9aaf),
                                ),
                              ],
                            ),
                            child: Center(
                              child: Icon(
                                widget.icon,
                                color: widget.color,
                                size: widget.iconSize,
                              ),
                            ),
                          ),
                          SizedBox(width: 15),
                          Expanded(
                            flex: 60,
                            child: Padding(
                              padding: const EdgeInsets.only(top: 4.0),
                              child: TextRF(widget.text,
                                  style: TextStyle(
                                    color: kTextColor,
                                    fontWeight: widget.textWeight,
                                    fontSize: widget.textSize,
                                  )),
                            ),
                          ),
                          if (widget.badge)
                            Expanded(
                              flex: 8,
                              child: Container(
                                width: 25,
                                height: 20,
                                child: Text(
                                  '5',
                                  style: TextStyle(
                                    fontSize: 16,
                                    color: kPrimaryColor,
                                    fontWeight: FontWeight.bold,
                                  ),
                                ),
                                alignment: Alignment(0, 0),
                                decoration: BoxDecoration(
                                  color: Colors.red,
                                  borderRadius: BorderRadius.all(Radius.circular(4)),
                                  boxShadow: [
                                    BoxShadow(
                                      color: Colors.grey.withOpacity(0.6),
                                      spreadRadius: 1,
                                      blurRadius: 1,
                                      offset: Offset(0, 1), // changes position of shadow
                                    ),
                                  ],
                                ),
                              ),
                            )
                          else
                            Expanded(child: Container()),
                          Expanded(
                            child: Container(),
                            flex: 20,
                          )
                        ],
                      ),
                    ],
                  ),
                ),
              ), //SizedBox(height: 10),
              widget.separator ? Container(width: 300, height: 1.5, color: Colors.grey) : Container()
            ],
          ),
        ),
      ),
    );
  }
}
