import 'package:flutter/material.dart';

//import 'package:searchable_dropdown/searchable_dropdown.dart';
import 'package:rapidfire/uiux/searchable_dropdown.dart';

import '../Db.dart';
import '../Dynamic.dart';
import '../GlobalScopeRF.dart';
import '../RapidFire.dart';
import '../ValidationRF.dart';
import 'WidgetStylesRF.dart';
import 'WidgetsRF.dart';
import 'StyleRF.dart';

enum DropdownType { simple, searchable }
enum DropdownStyleRF { normal, fluid }

class DropdownRF extends StatefulWidget {
  const DropdownRF({
    this.items,
    this.model,
    this.field,
    this.itemsOLD,
    this.value,
    this.bind,
    this.dropdownType = DropdownType.simple,
    this.maxLength,
    this.hintText,
    this.hint,
    this.caption,
    this.onSaved,
    this.validator,
    this.onFieldSubmitted,
    this.onChange,
    this.backColor = const Color(0x77f7f7f7),
    this.backgroundColor,
    this.dropdownStyle = DropdownStyleRF.normal,
    this.textColor,
    this.fontSize,
    this.autoLoadable = false,
    this.enabled = true,
    this.borderColor,
    this.borderWidth,
    this.borderRadius,
    this.fontWeight,
    this.decoration,
    this.placeholder,
    this.widgetStyle,
    //  this.width,
    this.height = 50,
    this.isExpanded = false,
  });

  final items;
  final field;
  final model;
  final List<DropdownMenuItem> itemsOLD;
  final String value;
  final dynamic bind;
  final DropdownType dropdownType;
  final int maxLength;
  final String hintText;
  final String hint;
  final String caption;
  final FormFieldSetter<String> onSaved;
  final FormFieldValidator<String> validator;
  final ValueChanged<String> onFieldSubmitted;
  final Function onChange;
  final backColor;
  final Color backgroundColor;
  final DropdownStyleRF dropdownStyle;
  final WidgetStyleRF widgetStyle;
  final textColor;
  final fontSize;
  final bool autoLoadable;
  final bool enabled;
  final Color borderColor;
  final double borderWidth;
  final double borderRadius;
  final FontWeight fontWeight;
  final BoxDecoration decoration;
  final Widget placeholder;

  //final double width;
  final double height;
  final bool isExpanded;

  @override
  _DropdownListRF createState() => _DropdownListRF();
}

class _DropdownListRF extends State<DropdownRF> {
  String _selectedValue = "";
  int _selectedIndex = 0;
  bool _showPlaceholder = true;

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;

    IStyleRF style = Libs.config.app.widgetStyle.all;
    WidgetStyleRF cws = widget.widgetStyle ??
        style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);

    if (widget.model != null) model = widget.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        try {
          setState(() {
            Libs.modelValidationData[widget.field] = data;
          });
        } catch (w) {}

        return data?.isValid;
      }
      return false;
    };

    selectedValue(snapshot) {
      var defaultValue = "0";

      if (widget.autoLoadable && _selectedValue.isNotEmpty) {
        defaultValue = _selectedValue;
      }
      if (widget.value != null && widget.value != "0")
        defaultValue = widget.value;
      else if (widget.field != null) {
        if (model != null) {
          var val = Dynamic(model).get(widget.field);
          defaultValue = (val == null) ? "0" : Dynamic(model).get(widget.field).toString();
        }
      } else {
        defaultValue = widget.value;
      }
      if (!widget.autoLoadable && _selectedValue.isNotEmpty) {
        defaultValue = _selectedValue;
      }

      var pkValue = Db().getPrimaryKeyValue(model);
      if (pkValue != null) _showPlaceholder = false; //defaultValue != "0"

      print('idx:$defaultValue');
      return defaultValue.toString();
    }

    simpleDropdown(snapshot) {
      var currentSelected = selectedValue(snapshot);
      print('current:selected : $currentSelected');
/*      if (currentSelected !="0" && widget.field != null) {
        if (model != null) Dynamic(model).set(widget.field, currentSelected);
        if (Libs.modelValidationData[widget.field] != null) Libs.modelValidationData[widget.field].isValid = true;
      }*/
      return IgnorePointer(
        ignoring: !widget.enabled,
        child: DropdownButton(
          focusColor: Colors.green,
          onTap: () => FocusScope.of(context).requestFocus(FocusNode()),
          underline: Padding(padding: EdgeInsets.all(0)),
          isExpanded: widget.isExpanded,
          items: snapshot as List<DropdownMenuItem<String>>,
          value: selectedValue(snapshot),
          style: cws.textStyle ??
              TextStyle(
                fontSize: widget.fontSize ?? cws.fontSize,
                color: widget.textColor ?? cws.textColor,
                fontWeight: widget.fontWeight ?? cws.fontWeight,
              ),
          onChanged: (value) {
            var index = (snapshot as List).indexWhere((element) => (element.value == value));
            if (widget.onChange != null) widget.onChange(index, value);
            if (widget.field != null) {
              if (model != null) Dynamic(model).set(widget.field, value);
              if (Libs.modelValidationData[widget.field] != null)
                Libs.modelValidationData[widget.field].isValid = true;
            }
            _selectedValue = value;
            setState(() {});
          },
        ),
      );
    }

    var templateDefaultControlBG; // future link with stock object
    //WidgetStyleRF styleRF = Libs.config.app?.theme?.controlBackground ?? templateDefaultControlBG ?? widget.backColor;

    searchableDropdown(snapshot) {
      return Container(
        //color: widget.backgroundColor ?? styleRF.backgroundColor,
        decoration: BoxDecoration(
            color: widget.backgroundColor ?? cws.backColor,
            /*    border: Border.all(
                color: widget.borderColor ?? Colors.transparent,
                width: widget.borderSize ?? 1), */ //shape: BoxShape.circle,
            borderRadius:
                BorderRadius.all(Radius.circular(widget.borderRadius ?? cws.borderRadius))),
        child: Stack(
          children: [
            SearchableDropdown.single(
              onSearchDialogOpen: () {},
              style: cws.textStyle ??
                  TextStyle(
                      fontSize: widget.fontSize ?? cws.fontSize,
                      color: widget.textColor ?? cws.textColor,
                      fontWeight: widget.fontWeight ?? cws.fontWeight),
              underline: Padding(padding: EdgeInsets.all(0)),
              items: snapshot as List<DropdownMenuItem<String>>,
              value: selectedValue(snapshot),
              isCaseSensitiveSearch: false,
              searchFn: (String keyword, items) {
                List<int> ret = <int>[];
                if (keyword != null && items != null && keyword.isNotEmpty) {
                  keyword.split(" ").forEach((k) {
                    int i = 0;
                    items.forEach((item) {
                      if (k.isNotEmpty &&
                          (item.child.data.toString().toLowerCase().contains(k.toLowerCase()))) {
                        ret.add(i);
                      }
                      i++;
                    });
                  });
                }
                if (keyword.isEmpty) {
                  ret = Iterable<int>.generate(items.length).toList();
                }
                return (ret);
              },
              hint: "Select an Item",
              searchHint: "Select an Item",
              onChanged: (value) {
                var index = (snapshot as List).indexWhere((element) => (element.value == value));
                if (widget.onChange != null) widget.onChange(index, value);
                Dynamic(model).set(widget.field, value);
                if (Libs.modelValidationData[widget.field] != null) {
                  Libs.modelValidationData[widget.field].isValid = true;
                }
                setState(() {});
              },
              isExpanded: true,
            ),
          ],
        ),
      );
    }

    dropdownPlaceholder(Widget ui) {
      if (showPlaceholder != null) cws.placeholderVisible = showPlaceholder;
      var _placeholder = widget.placeholder ?? cws.placeholder;
      if (_placeholder is EmptyRF) {
        return ui;
      } else {
        if (cws.placeholderVisible == null) cws.placeholderVisible = true;

        if (_selectedValue == null || _selectedValue.isEmpty) {
          return Stack(
            children: [
              ui,
              if (_showPlaceholder && cws.placeholderVisible)
                PlaceholderRF(
                  height: widget.height,
                  // set it for textbox
                  type: PlaceholderTypeRF.Inline,
                  backColor: widget.widgetStyle?.placeholderBackColor ?? Colors.white,
                  lineColor: widget.widgetStyle?.placeholderLineColor ?? Colors.grey,
                  lines: widget.widgetStyle?.placeholderLines ?? 2,
                  lineHeight: widget.widgetStyle?.placeholderLineHeight ?? 10,
                  border: widget.widgetStyle?.placeholderBorder,
                  borderRadius: widget.widgetStyle?.placeholderRadius ?? 4,
                ),
            ],
          );
        } else {
          return ui;
        }
      }
    }

    var futureDropdown = FutureBuilder(
      future: (widget.items is Future) ? widget.items : null,
      builder: (context, snapshot) {
        switch (snapshot.connectionState) {
          case ConnectionState.waiting:
            return Text('Loading....');
          default:
            if (snapshot.hasError) {
              return Text('Error: ${snapshot.error}');
            } else {
              _selectedIndex = 0;
              return (widget.dropdownType == DropdownType.simple)
                  ? simpleDropdown(snapshot.data)
                  : searchableDropdown(snapshot.data);
            }
        }
      },
    );

    //Color color = Libs.config.app?.theme?.controlBackground?.backgroundColor ?? widget.backColor;

    switch (widget.dropdownStyle) {
      case DropdownStyleRF.normal:
        var _decoration = widget.decoration ?? cws.decoration;
        return Column(
          children: <Widget>[
            dropdownPlaceholder(
              Container(
                height: widget.height,
                padding: EdgeInsets.only(left: 10, right: 10),
                decoration: _decoration ??
                    BoxDecoration(
                      color: widget.backgroundColor ?? cws.backColor,
                      border: Border.all(
                          color: widget.borderColor ?? cws.borderColor,
                          width: widget.borderWidth ?? cws.borderWidth),
                      borderRadius: BorderRadius.all(
                        Radius.circular(widget.borderRadius ?? cws.borderRadius),
                      ),
                      boxShadow: cws.shadow,
                    ),
                child: (widget.items is Future)
                    ? futureDropdown
                    : (widget.dropdownType == DropdownType.simple)
                        ? simpleDropdown(widget.items)
                        : searchableDropdown(widget.items),
              ),
            ),
            ValidationRF.showErrorMessage(widget.field)
          ],
        );
        break;

      case DropdownStyleRF.fluid:
        return Container(
          margin: EdgeInsets.only(left: 5, right: 5),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Container(
                padding: EdgeInsets.only(left: 10, right: 10),
                decoration: BoxDecoration(
                    color: Colors.white, //shape: BoxShape.circle,
                    boxShadow: [
                      BoxShadow(
                        color: Colors.grey.withOpacity(0.4),
                        blurRadius: 3.0,
                        spreadRadius: 2.0,
                      ),
                    ],
                    border: Border.all(
                        color: widget.borderColor ?? cws.borderColor,
                        width: widget.borderWidth ?? cws.borderWidth),
                    borderRadius:
                        BorderRadius.all(Radius.circular(widget.borderRadius ?? cws.borderRadius))),
                child: (widget.items is Future)
                    ? futureDropdown
                    : (widget.dropdownType == DropdownType.simple)
                        ? simpleDropdown(widget.items)
                        : searchableDropdown(widget.items),
              ),
              ValidationRF.showErrorMessage(widget.field)
            ],
          ),
        );
        break;
    }
    return Container();
  }
}
