import 'dart:io';
import 'dart:ui';

import 'package:catcher/catcher.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:flutter_rating_bar/flutter_rating_bar.dart';
import 'package:flutter_sticky_header/flutter_sticky_header.dart';
import 'package:flutter_xlider/flutter_xlider.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:intl/intl.dart';
import 'package:rapidfire/Annotation.dart' as annotation;
import 'package:rapidfire/Config.dart';
import 'package:rapidfire/LocalizationProviderRF.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/SystemRF.dart';
import 'package:rapidfire/ValidationRF.dart';
import 'package:rapidfire/store/StoreRF.dart';
import 'package:rapidfire/store/modern/onboarding/Intro.dart';
import 'package:rapidfire/store/modern/onboarding/Splash.dart';
import 'package:rapidfire/uiux/PlaceHolderRF.dart';
import 'package:rapidfire/uiux/intro/Models/page_view_model.dart';
import 'package:sizer/sizer.dart';
import 'package:video_player/video_player.dart' as vp;

// import 'package:keyboard_visibility/keyboard_visibility.dart';

import '../Db.dart';
import '../Dynamic.dart';
import '../Extensions.dart';
import '../GlobalRF.dart';
import '../GlobalScopeRF.dart';
import '../LocalizationRF.dart';
import '../RapidFire.dart';
import '../uiux/StyleRF.dart';
import 'AlertRF.dart';
import 'DrawerRF.dart';
import 'WidgetStylesRF.dart';

export 'package:font_awesome_flutter/font_awesome_flutter.dart';
export 'package:rapidfire/store/StoreRF.dart';
export 'package:rapidfire/uiux/DropdownListRF.dart';
export 'package:rapidfire/uiux/widgets/AnimatedContainerRF.dart';
export 'package:rapidfire/uiux/widgets/ButtonRF.dart';
export 'package:rapidfire/uiux/widgets/GridViewRF1.dart';

//export 'package:rapidfire/uiux/widgets/ListViewRFOLD.dart';
export 'package:rapidfire/uiux/widgets/ListViewRF.dart';
export 'package:rapidfire/uiux/widgets/SyncFetchRF.dart';
export 'package:sizer/sizer.dart';

export '../uiux/DialogRF.dart';
export 'StyleRF.dart';
export 'package:rapidfire/uiux/PlaceHolderRF.dart';
export 'package:rapidfire/uiux/widgets/GridViewRF.dart';
export 'package:rapidfire/uiux/widgets/FieldRowRF.dart';
export 'package:rapidfire/uiux/widgets/SocialRF.dart';
export 'package:rapidfire/uiux/widgets/TabRF.dart';
export 'package:rapidfire/uiux/widgets/PageViewSliderRF.dart';
export 'package:rapidfire/uiux/widgets/ListViewSliderRF.dart';

//#region RapidFireApp
class RapidFireAppParentRF extends InheritedWidget {
  Object initialModel;
  DataSourceRF dataSource;
  bool keyboardVisible;

  RapidFireAppParentRF({
    Key key,
    this.dataSource,
    this.keyboardVisible = false,
    this.initialModel,
    Widget child,
  }) : super(key: key, child: child);

  @override
  bool updateShouldNotify(RapidFireAppParentRF oldWidget) {
    return oldWidget.dataSource != dataSource || oldWidget.keyboardVisible != keyboardVisible;
  }
}

class RapidFireAppRF extends StatefulWidget {
  final dynamic child;

  RapidFireAppRF(this.child);

  static RapidFireAppParentRF of(BuildContext context) =>
      context.dependOnInheritedWidgetOfExactType<RapidFireAppParentRF>();

  static dispose() {}

  @override
  State<StatefulWidget> createState() {
    return _RapidFireAppRF();
  }
}

class _RapidFireAppRF extends State<RapidFireAppRF> {
  LanguageDelegate _localeLanguageOverrideDelegate;

  @override
  void initState() {
    super.initState();
    var lang = rf.pref.getString("lang") ?? 'en';
    _localeLanguageOverrideDelegate = new LanguageDelegate(Locale(lang, ''));
    LanguageRF.loadLanguage = onSetLanguage;
  }

  onSetLanguage(Locale locale, Function onChanged) {
    setState(() {
      _localeLanguageOverrideDelegate = new LanguageDelegate(locale);
      rf.pref.setString('lang', locale.languageCode);
      LanguageRF.onLoadComplete = onChanged;
    });
  }

  @override
  Widget build(BuildContext context) {
    return RapidFireAppParentRF(
      child: LayoutBuilder(builder: (context, constraints) {
        return OrientationBuilder(
          builder: (context, orientation) {
            SizerUtil().init(constraints, orientation);
            return MaterialApp(
              navigatorKey: Catcher.navigatorKey,
              debugShowCheckedModeBanner: false,
              theme: Libs.config.app.themeData.customThemeData(ThemeDataTypeRF.light),
              darkTheme: Libs.config.app.themeData.customThemeData(ThemeDataTypeRF.dark),
              localizationsDelegates: [
                _localeLanguageOverrideDelegate,
                GlobalMaterialLocalizations.delegate,
                GlobalWidgetsLocalizations.delegate,
              ],
              supportedLocales: LanguageRF.supportedLocales(),
              home: widget.child,
            );
          },
        );
      }),
    );
  }
}
//#endregion RapidFireApp

//#region Onboard
enum OnboardItemsRF { splash, language, intro, login, otp, setup, home }

class AppOnboardRF {
  Widget language;
  dynamic splash;
  dynamic intro;
  Widget login;
  Widget otp;
  Widget initialSetup;
  dynamic home;

  // dynamic drawer;

  AppOnboardRF({
    this.language,
    this.splash,
    @required this.intro,
    this.login,
    this.otp,
    this.initialSetup,
    @required this.home, //   this.drawer,
  });

  static int currentStep = 0;
  static AppOnboardRF onboardItems;
  static List<dynamic> onboardItemList = List();
  static bool isLanguageLoaded = false;

  static loadOnboardItems(AppOnboardRF items) {
    onboardItems = items;
    onboardItemList.add(onboardItems.splash);
    onboardItemList.add(onboardItems.language);
    onboardItemList.add(onboardItems.intro);
    onboardItemList.add(onboardItems.login);
    onboardItemList.add(onboardItems.otp);
    onboardItemList.add(onboardItems.initialSetup);
    onboardItemList.add(onboardItems.home);
  }

  next() {
    for (int i = currentStep; i < onboardItemList.length; i++) {
      if (onboardItemList[i] != null) {
        currentStep++;
        return onboardItemList[i];
      } else {
        currentStep++;
      }
    }
  }

  logout() {
    currentStep = 0;
  }

  static onboardWidget(source) {
    if (source is SplashRF)
      return Splash(splash: source);
    else if (source is List<IntroPageViewRF>)
      return Intro(pages: source);
    else
      return source;
  }

  static addItem(
    Configuration _configuration,
    OnboardItemsRF item,
  ) {
    TemplateRF clientTemplate = _configuration.app.template;
    if (_configuration.app.onboard is AppOnboardRF) {
      switch (item) {
        case OnboardItemsRF.splash:
          return onboardWidget(_configuration.app.onboard.splash) ??
              StoreRF.onboard.splash(_configuration, template: clientTemplate);
        case OnboardItemsRF.language:
          return !(_configuration.app.onboard.language is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.language) ??
                  StoreRF.onboard.language(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.intro:
          return !(_configuration.app.onboard.intro is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.intro)
              : null;
        case OnboardItemsRF.login:
          return !(_configuration.app.onboard.login is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.login) ??
                  StoreRF.onboard.login(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.otp:
          return !(_configuration.app.onboard.otp is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.otp) ??
                  StoreRF.onboard.otp(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.setup:
          return !(_configuration.app.onboard.initialSetup is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.initialSetup) ??
                  StoreRF.onboard.setup(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.home:
          assert(_configuration.app.onboard.home != null, 'AppOnboardRF requires Home');
          return _configuration.app.onboard.home;
      }
    } else {
      switch (item) {
        case OnboardItemsRF.splash:
          return onboardWidget(_configuration.app.onboard.customOnboard().splash) ??
              StoreRF.onboard.splash(_configuration, template: clientTemplate);
        case OnboardItemsRF.language:
          return !(_configuration.app.onboard.customOnboard().language is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().language) ??
                  StoreRF.onboard.language(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.intro:
          return !(_configuration.app.onboard.customOnboard().intro is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().intro)
              : null;
        case OnboardItemsRF.login:
          return !(_configuration.app.onboard.customOnboard().login is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().login) ??
                  StoreRF.onboard.login(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.otp:
          return !(_configuration.app.onboard.customOnboard().otp is EmptyRF)
              ? (onboardWidget(_configuration.app.onboard.customOnboard().otp) ??
                  StoreRF.onboard.otp(_configuration, template: clientTemplate))
              : null;
        case OnboardItemsRF.setup:
          return !(_configuration.app.onboard.customOnboard().initialSetup is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().initialSetup) ??
                  StoreRF.onboard.setup(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.home:
          assert(_configuration.app.onboard.customOnboard().home != null,
              'AppOnboardRF requires Home');
          return _configuration.app.onboard.customOnboard().home;
      }
    }
  }
}
//#endregion Onboard

//#region PageRF
class PageParentRF extends InheritedWidget {
  final Object model;
  final dynamic pageData;
  final dynamic showPlaceholder;
  final dynamic instance;
  FlexBarRF flex;

  PageParentRF({
    Key key,
    this.model,
    this.pageData,
    this.showPlaceholder,
    this.instance,
    this.flex,
    Widget child,
  }) : super(key: key, child: child);

  @override
  bool updateShouldNotify(PageParentRF oldWidget) {
    return oldWidget.model != model ||
        oldWidget.pageData != pageData ||
        oldWidget.showPlaceholder != showPlaceholder ||
        oldWidget.instance != instance ||
        oldWidget.flex != flex;
  }
}

enum PageType { none, home, entry, list, detail, sync, fetch }

class PageRF extends StatefulWidget {
  final String title;
  final bool titleVisible;
  final String flexTitle;
  final bool flexTitleVisible;
  final dynamic appBar;
  final Widget appBarBottom;
  final Widget drawer;
  final Widget drawerIcon;
  final AppBar footer;
  final PageType pageType;
  final OutlineRF outline;
  final Object model;
  final dynamic pageData;
  final bool showPlaceholder;
  final Function onCallerBack;
  final FlexBarRF flex;
  final stickyHeader;
  final Widget body;
  final Widget fab; //fabEx + on click
  final FloatingActionButtonAnimator fabEx; //fabEx + on click
  final Widget filter;
  final Function onInit;
  final Function onBack;
  final Color backgroundColor;
  final bool disabledPageScroll;
  final List<Color> osBarColors;
  List<Widget> backgroundPositionedImages;
  List<int> hidePositionedImages;
  final bool normalPage;
  TemplateRF template;

  PageRF({
    @required this.title,
    this.titleVisible = false,
    this.flexTitle = '', //will go to flex
    this.flexTitleVisible = true,
    this.pageType,
    this.appBar, //New
    this.appBarBottom, //New
    this.drawer,
    this.drawerIcon,
    this.footer, //new
    this.outline, //?? no need
    this.model,
    this.pageData,
    this.showPlaceholder,
    this.onCallerBack,
    this.flex,
    this.stickyHeader,
    this.body,
    this.fab,
    this.fabEx,
    this.filter, // no need , can go to sticket header
    this.onInit,
    this.onBack,
    this.normalPage = false,
    this.backgroundColor = Colors.white,
    this.disabledPageScroll = false,
    this.backgroundPositionedImages,
    this.hidePositionedImages,
    this.osBarColors,
    this.template,
  }) : assert(title != null);

  static DataSourceRF dataSourceRefreshOnBack; // remove
  static Function onCallerPageBack; // remove
  //static Map<String, Function> onCallerRefreshMap = Map(); // remove
  //static Map<String, List<DataSourceRF>> dataSourcesCallerRefreshMap = Map(); // remove
  static List<Function> pageScrollListener = [];

  //static dynamic currentPageClassName;
  static dynamic currentDialogContext;
  static bool keyboardOpened;
  static double pageViewInsets;

  static PageParentRF of(BuildContext context) {
    return context.dependOnInheritedWidgetOfExactType<PageParentRF>();
  }

  // static refresh(BuildContext context) => (context as Element).reassemble();
  static refresh(dynamic instance) {
    if (instance is StatefulElement) {
      instance.reassemble();
    }
  }

  static bool isValidModel(BuildContext context, dynamic model,
      {String message, Function validator}) {
    bool valid = true;

    var modelMembers = Dynamic(model).modelMembers;

    // Annotation validations
    for (var item in Libs.modelValidationFunctions.entries) {
      if (modelMembers[item.key] != null) {
        var ret = item.value.call();
        if (ret != null) {
          valid = ret;
          if (!valid) Libs.modelValidationData[item.key].isValid = false;
        }
      }
    }
    if (!valid) {
      rf.msg(context, message ?? 'Data save was unsuccessful due to missing mandatory fields!');
      return valid;
    }

    // Global config validations
    for (var element in Libs.config?.model?.validators) {
      valid = element(context);
      var methodName = Dynamic.methodName(element);
      if (!valid) {
        return valid;
      }
    }

    // Local config validations
    if (validator != null) {
      String msg = validator.call() ?? '';
      if (msg.isNotEmpty) {
        rf.msg(context, msg ?? 'Data save was unsuccessful due some error!');
        return false;
      }
    }

    return valid;
  }

  static bool executeValidators(BuildContext context, String methodName,
      {Function validateHandler}) {
    bool valid = true;

    for (var element in Libs.config?.model?.validators) {
      var functionName = Dynamic.methodName(element);

      if (methodName == null || methodName.isEmpty) {
        valid = element(context);
      } else {
        if (functionName == methodName) {
          valid = element(context);
        }
      }

      if (!valid) {
        if (validateHandler != null) validateHandler(functionName);
      }
    }
    return valid;
  }

  static willPop(context, {model}) {
    if (model != null) {
      if (Dynamic(model).changed(RapidFireAppRF.of(context).initialModel)) {
        rf.msg(
          context,
          'Save change pending, do you want to quite anyway?',
          icon: Icons.error,
          actionText: 'Quit',
          action: () => Navigator.pop(context),
        );
      } else {
        if (Navigator.canPop(context)) {
          Navigator.pop(context);
        } else {
          //SystemChannels.platform.invokeMethod('SystemNavigator.pop');
          SystemNavigator.pop();
        }
      }
    } else {
      if (Navigator.canPop(context)) {
        Navigator.pop(context);
      } else {
        //SystemChannels.platform.invokeMethod('SystemNavigator.pop');
        SystemNavigator.pop();
      }
    }
    return Future.value(true);
  }

  @override
  State<StatefulWidget> createState() => _PageRF(osBarColors);
}

class _PageRF extends State<PageRF> {
  List<Color> _colors;

  dynamic modelPKValue, initialModelPKValue;
  List<Widget> widgetStack = [];

  _PageRF(List<Color> colors) {
    _colors = colors;
  }

  @override
  void initState() {
    widget.template = widget.template ?? TemplateRF(TemplateNameRF.modern);

    PageRF.dataSourceRefreshOnBack = null;
    Libs.modelValidationData.clear();
    Libs.modelValidationFunctions.clear();

    SystemRF.os.setOsBarColors(_colors);

    WidgetsBinding.instance.addPostFrameCallback((_) {
      if (widget.onInit != null) widget.onInit(context);
    });
    RapidFire.updateGlobalErrorLanguage();

    _controller.addListener(_scrollListener);

    super.initState();
  }

  @override
  void didChangeDependencies() async {
    if (widget.model != null) {
      ValidationRF.validationAnnotations = Dynamic(widget.model).annotationData(annotation.Check);

      var _initialModel = Dynamic(widget.model).copy();

      if (RapidFireAppRF.of(context).initialModel == null) {
        RapidFireAppRF.of(context).initialModel = _initialModel;
        initialModelPKValue = Db().getPrimaryKeyValue(_initialModel);
      } else {
        if (widget.model.runtimeType.toString() !=
            RapidFireAppRF.of(context).initialModel.runtimeType.toString()) {
          RapidFireAppRF.of(context).initialModel = _initialModel;
          initialModelPKValue = Db().getPrimaryKeyValue(_initialModel);
        }
      }
    }
    super.didChangeDependencies();
  }

  @override
  void dispose() {
    //if (PageRF.callerStack.length == 1) PageRF.callerStack.clearStack();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    widgetStack.clear();
    PageRF.pageViewInsets = MediaQuery.of(context).viewInsets.bottom;
    Libs.context = context;

    if (widget.model != null) {
      if (initialModelPKValue == null) {
        modelPKValue = Db().getPrimaryKeyValue(widget.model);
        if (modelPKValue != null && modelPKValue != "") {
          RapidFireAppRF.of(context).initialModel = Dynamic(widget.model).copy();
        }
      }
    }
    if (widget.backgroundPositionedImages == null) {
      IStyleRF style = Libs.config.app.widgetStyle.all;
      WidgetStyleRF cws = style.customStyle(
        context,
        widget.runtimeType,
        widget.pageData,
        widget.showPlaceholder,
      );

      widget.hidePositionedImages?.forEach((element) {
        cws.backgroundPositionedImages.removeAt(element);
      });

      widgetStack = cws.backgroundPositionedImages ?? [];
      //(context as Element).reassemble();
    } else {
      widget.backgroundPositionedImages?.forEach((element) {
        widgetStack.add(element);
      });
    }
    widgetStack.add(pageContent(
      widget.flex,
      widget.title,
      widget.titleVisible,
      widget.flexTitle,
      widget.flexTitleVisible,
      widget.stickyHeader,
      widget.disabledPageScroll,
    ));

    // OutlineRF.loadPageOutline(widget.template);

    var page = PageParentRF(
      instance: this,
      model: widget.model,
      pageData: widget.pageData,
      showPlaceholder: widget.showPlaceholder,
      flex: widget.flex,
      child: SafeArea(
        child: Scaffold(
          backgroundColor: widget.backgroundColor ?? Colors.white,
          //appBar: Libs.config.app.outline.customOutline().appBar ?? StoreRF.implementation(template).page.outline.customOutline().appBar,
          //appBar: appBar,
          // OutlineRF.getAppBar(
          //   context,
          //   widget.outline,
          //   widget.title,
          //   widget.flex,
          //   widget.outline?.stickyHeaderPadding,
          //   widget.template,
          // ),
          drawer: widget.drawer,
          //OutlineRF.getAppDrawer(widget.outline, widget.template),
          body: WillPopScope(
            onWillPop: widget.onBack ??
                () {
                  PageRF.willPop(context, model: widget.model);
                  return Future.value(true);
                },
            child: Libs.config.app.forceAppUpgrade
                ? UpgradeAlert(
                    debugLogging: true,
                    child: Stack(
                      children: widgetStack,
                    ),
                  )
                : Stack(
                    children: widgetStack,
                  ),
          ),
          floatingActionButton: widget.fab,
          floatingActionButtonAnimator: widget.fabEx,
          bottomNavigationBar: widget.outline?.footer,
          //PageOutlineRF.getAppBottomBar(widget.outline, widget.theme),
        ),
      ),
    );

    return page;
  }

  sliverAppBar(BuildContext context, Widget leading, FlexBarRF flex, String title) {
    double _expandedHeight = flex?.height ?? 0.0;
    var _marginTop = flex?.marginTop ?? 56;
    var titleCenter = true;
    var top = 0.0;

    var appBar = widget.appBar ??
        Libs.config.page.appBar
            .customAppBar(widget.pageType, context, widget.model, widget.pageData) ??
        StoreRF.implementation(widget.template)
            .page
            .appBar
            .customAppBar(widget.pageType, context, widget.model, widget.pageData);

    flexPanel() {
      return LayoutBuilder(builder: (BuildContext context, BoxConstraints constraints) {
        top = constraints.biggest.height;
        if (top <= 80) {
          titleCenter = false;
        } else {
          titleCenter = true;
        }

        return FlexibleSpaceBar(
          centerTitle: titleCenter,
          collapseMode: CollapseMode.parallax,
          stretchModes: [StretchMode.fadeTitle],
          title: AnimatedOpacity(
            duration: Duration(milliseconds: 100),
            opacity: top == 80.0 ? 1.0 : 0.0,
            child: Text(
              title, //widget.flex.title,
              style: TextStyle(fontSize: 20.0, color: Colors.black),
            ),
          ),
          background: Container(
            margin: EdgeInsets.only(top: _marginTop, left: 0),
            color: flex.color,
            child: Stack(
              children: <Widget>[
                flex.background,
                Positioned(
                  left: 15,
                  bottom: 15,
                  child: Text(
                    flex.label ?? '',
                    style: TextStyle(
                      fontSize: 20.0,
                      color: Colors.black,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ),
              ],
            ),
          ),
        );
        //Image.network(widget.flex.background, fit: BoxFit.cover));
      });
    }

    return SliverAppBar(
      iconTheme: IconThemeData(color: Colors.black),
      leading: appBar.leading,
      title: flex == null ? TextRF(title, size: 18, color: Colors.black87) : TextRF(''),
      expandedHeight: _expandedHeight,
      floating: true,
      backgroundColor: appBar.backgroundColor,
      actions: appBar.actions,
      pinned: true,
      elevation: appBar.elevation ?? 0.0,
      forceElevated: (appBar.elevation == null) ? false : true,
      bottom: widget.appBarBottom,
      flexibleSpace: flex != null ? flexPanel() : null,
    );
  }

  ScrollController _controller = ScrollController();
  bool sliverActionsHidden = true;

  void _scrollListener() {
    if (_controller.position.atEdge) {
      // Reach the top of the list
      if (_controller.position.pixels == 0) {
        print('top from parent');
      }
      // Reach the bottom of the list
      else {
        PageRF.pageScrollListener.forEach((element) {
          element.call();
        });
        print('bottom from parent');
      }
    }
  }

  pageContent(FlexBarRF flex, String title, bool titleVisible, String flexTitle,
      bool flexTitleVisible, Widget stickyHeader, bool disablePageScroll) {
    var content = CustomScrollView(
      controller: _controller,
      physics: disablePageScroll ? NeverScrollableScrollPhysics() : null,
      slivers: <Widget>[
        if (widget.appBar.runtimeType != EmptyRF) ...[
          sliverAppBar(context, widget?.outline?.appBar?.leading, flex, title),
        ],
        // SliverFixedExtentList(
        //   delegate: SliverChildListDelegate([
        //     // Container(color: Colors.blue, child: Text('FixedExtent')),
        //     // Container(color: Colors.blue, child: Text('FixedExtent')),
        //   ]),
        //   itemExtent: 20,
        // ),
        SliverStickyHeader(
          header: Container(
            //height: stickyHeader == null ? 0.0 : (flex == null ? 64 : 56), // Removed for PP Proj
            color: Colors.transparent,
            padding: EdgeInsets.only(top: 0.0),
            alignment: Alignment.center,
            child:
                Padding(padding: EdgeInsets.only(top: (flex == null ? 0 : 0)), child: stickyHeader),
          ),
          sliver: SliverList(
            delegate: SliverChildListDelegate(
              [widget.body],
            ),
          ),
        ),
      ],
    );

    if (!widget.normalPage) {
      if (Libs.config.page.scrollPhysics == null) {
        return ScrollConfiguration(
          behavior: BounceScrollBehavior(),
          child: content,
        );
      } else {
        if (Libs.config.page.scrollPhysics == ScrollPhysicsRF.bounce) {
          return ScrollConfiguration(
            behavior: BounceScrollBehavior(),
            child: content,
          );
        } else {
          return content;
        }
      }
    } else
      return widget.body ?? Container();
  }
}

class FlexBarRF {
  var label;
  var background;
  var backgroundHeight;
  var color;
  var height;
  double marginTop;

  // var stickyHeader;
  FlexBarRF({
    this.label,
    this.background,
    this.color = Colors.white,
    this.height = 250.0,
    this.backgroundHeight = 300,
    this.marginTop = 56,
  });
}
//#endregion PageRF

//#region PageOutline
enum OutlineDisableItems { header, headerPanel, pageFooter }

class OutlineRF {
  AppBarRF appBar;
  Widget stickyHeader;
  Widget footer;
  EdgeInsets stickyHeaderPadding;
  TemplateRF template;
  List<DrawerItemRF> drawerItems;

//  style -> bar ->back color, font, elevation
  //pageBar sub class=> leading, actions, drawer, , , ?style?, -disabled, -title?

  OutlineRF({
    this.appBar,
    this.stickyHeader,
    this.footer,
    this.stickyHeaderPadding,
    this.template,
    this.drawerItems,
  });

  static OutlineRF implemented = OutlineRF(); //client, stock,

  static loadPageOutline(template) {
    implemented.appBar = Libs.config.app.outline.customOutline().appBar ??
        StoreRF.implementation(template).page.outline.customOutline().appBar;

    implemented.appBar?.drawer = Libs.config.app.outline.customOutline().appBar?.drawer ??
        StoreRF.implementation(template).page.outline.customOutline().appBar?.drawer;

    implemented.appBar?.actions = Libs.config.app.outline.customOutline().appBar?.actions ??
        StoreRF.implementation(template).page.outline.customOutline().appBar?.actions;

    implemented.stickyHeader = Libs.config.app.outline.customOutline().stickyHeader ??
        StoreRF.implementation(template).page.outline.customOutline().stickyHeader;

    implemented.footer = Libs.config.app.outline.customOutline().footer ??
        StoreRF.implementation(template).page.outline.customOutline().footer;
  }

  static getAppBar(
    BuildContext context,
    OutlineRF local,
    String pageTitle,
    FlexBarRF flex,
    EdgeInsets pageHeaderPanelPadding,
    TemplateRF template,
  ) {
    OutlineRF _source = OutlineRF();
    var appBar;

    // _source.pageHeader = local?.pageHeader ?? implemented.pageHeader;
    _source.appBar = AppBarRF(
      leading: local?.appBar?.leading ?? implemented.appBar.leading,
      drawer: local?.appBar?.drawer ?? implemented.appBar.drawer,
      actions: local?.appBar?.actions ?? implemented.appBar.actions,
      backgroundColor: local?.appBar?.backgroundColor ?? implemented.appBar.backgroundColor,
      foregroundColor: local?.appBar?.foregroundColor ?? implemented.appBar.foregroundColor,
      elevation: local?.appBar?.elevation ?? implemented.appBar.elevation,
    );
    _source.stickyHeader = local?.stickyHeader ?? implemented.stickyHeader;

    assert(pageTitle != null, "Page title is missing.");

    customAppBar() {
      AppBar pageHeader;
      var _headerPanel = _source?.stickyHeader;
      var pageBottom = PreferredSize(
        child: Container(
          color: Colors.transparent,
          height: 50.0,
          width: ScreenRF.width(context),
          child: Padding(
            padding: EdgeInsets.only(
                top: pageHeaderPanelPadding?.top ?? 5, bottom: pageHeaderPanelPadding?.bottom ?? 5),
            child: _headerPanel is Widget ? _headerPanel : (_headerPanel as TabBarRF).toTabBar,
          ),
        ),
        preferredSize: Size.fromHeight(50.0),
      );

      if (_source?.appBar is AppBarRF) {
        pageHeader = AppBar(
          titleSpacing: 0,
          actionsIconTheme:
              IconThemeData(color: (_source?.appBar as AppBarRF).foregroundColor ?? Colors.white),
          elevation: (_source?.appBar as AppBarRF).elevation,
          backgroundColor: (_source?.appBar as AppBarRF).backgroundColor,
          leading: (_source?.appBar as AppBarRF).leading is EmptyRF
              ? null
              : (_source?.appBar as AppBarRF).leading,
          title: Text(pageTitle,
              style: TextStyle(color: (_source?.appBar as AppBarRF).foregroundColor)),
          actions: (_source?.appBar as AppBarRF).actions,
          bottom: _headerPanel == null ? _headerPanel : pageBottom,
        );
        return pageHeader;
      } else if (_source?.appBar is AppBar) {
        pageHeader = _source?.appBar as AppBar; //ImroseX
      } else if (_source?.appBar is EmptyRF) {
        return null;
      }

      return AppBar(
        leading: (pageHeader)?.leading is EmptyRF ? null : (pageHeader)?.leading,
        automaticallyImplyLeading: (pageHeader)?.automaticallyImplyLeading ?? true,
        title: Text(pageTitle),
        actions: (pageHeader)?.actions,
        flexibleSpace: (pageHeader)?.flexibleSpace,
        bottom: PreferredSize(
            child: Container(
              color: Colors.transparent,
              height: 50.0,
              width: 800,
              child: _headerPanel is Widget ? _headerPanel : (_headerPanel as TabBarRF).toTabBar,
            ),
            preferredSize: Size.fromHeight(50.0)),
        elevation: (pageHeader)?.elevation,
        shape: (pageHeader)?.shape,
        //backgroundColor: (pageHeader)?.backgroundColor,
        brightness: (pageHeader)?.brightness,
        iconTheme: (pageHeader)?.iconTheme,
        actionsIconTheme: (pageHeader)?.actionsIconTheme,
        textTheme: (pageHeader)?.textTheme,
        primary: (pageHeader)?.primary ?? true,
        centerTitle: (pageHeader)?.centerTitle,
        excludeHeaderSemantics: (pageHeader)?.centerTitle ?? false,
        titleSpacing: (pageHeader)?.titleSpacing ?? NavigationToolbar.kMiddleSpacing,
        toolbarOpacity: (pageHeader)?.toolbarOpacity ?? 1.0,
        bottomOpacity: (pageHeader)?.bottomOpacity ?? 1.0,
      );
    }

    // (local?.pageHeader as AppBar).leading = (local?.pageHeader as PageOutlineRF).headerPanel.;

    if (flex == null) {
      appBar = customAppBar();
    }
    return appBar;
  }

  static getAppDrawer(OutlineRF local, TemplateRF template) {
    OutlineRF _source = OutlineRF(
      appBar: AppBarRF(
        //     leading: (local?.pageHeader as PageHeaderRF)?.leading ??
        //            (implemented.pageHeader as PageHeaderRF).leading,
//        actions: local?.pageHeader is EmptyRF
//            ? null
//            : (local?.pageHeader as PageHeaderRF)?.actions ??
//                (implemented.pageHeader as PageHeaderRF).actions,
        drawer: local?.appBar is EmptyRF
            ? null
            : (local?.appBar as AppBarRF)?.drawer ?? (implemented.appBar as AppBarRF).drawer,
      ),
    );
    var drawer = (_source.appBar as AppBarRF).drawer;
    return drawer is EmptyRF ? null : drawer;
  }

//default+local
//static getAppTabBar(){}

  //default check==>>
  static getAppBottomBar(OutlineRF local, TemplateRF template) {
    OutlineRF _source = OutlineRF();
    _source.footer = local?.footer ?? implemented.footer ?? implemented.footer;
    return _source.footer;
  }
}

class AppBarRF {
  Widget leading;
  List<Widget> actions;
  dynamic drawer;
  double elevation;
  Color backgroundColor;
  Color foregroundColor;

  AppBarRF({
    this.leading,
    this.actions,
    this.drawer,
    this.elevation,
    this.backgroundColor,
    this.foregroundColor,
  });
}
//#endregion PageOutline

//#region EmptyRF
class EmptyRF extends StatelessWidget {
  static of(Type type, [bool list = false]) {
    if (type == DrawerItemRF) {
      if (list) {
        return List<DrawerItemRF>();
      } else {
        return DrawerItemRF(template: TemplateRF(null, version: 'EmptyRF'));
      }
    } else if (type == List) {
      return List<Widget>();
    }
  }

  @override
  Widget build(BuildContext context) => Container(height: 0, width: 0);
}
//#endregion EmptyRF

//#region TabBar
class TabBuilderRF {
  final List<Widget> views;
  final Color selectedLabelColor;
  final Color unselectedLabelColor;
  final TabBarIndicatorSize indicatorSize;
  final Decoration indicatorDecoration;
  final Function onTap;
  final TickerProvider tickerProvider;
  TabController controller;
  List<Widget> tabs;
  Function tabBuilder;
  Function _moveBefore;
  Function _moveAfter;

  moveAfter() {
    _moveAfter();
  }

  moveBefore() {
    _moveBefore();
  }

  TabBuilderRF({
    this.tabs,
    this.views,
    this.controller,
    this.selectedLabelColor,
    this.unselectedLabelColor,
    this.indicatorSize = TabBarIndicatorSize.tab,
    this.indicatorDecoration, // = const BoxDecoration(color: Colors.blue),
    this.onTap,
    this.tickerProvider,
    this.tabBuilder,
  }) {
    _moveBefore = moveBefore;
    _moveAfter = moveAfter;
    //var tabs1 = tabBuilder(0) as List<Widget>;
    if (tabBuilder != null) this.tabs = tabBuilder(0) as List<Widget>;
    this.controller = TabController(length: tabs.length, vsync: tickerProvider);
  }
}

class TabBarRF extends StatefulWidget {
  TabBuilderRF tab;
  TabBuilderRF view;

  TabBarRF({this.tab, this.view});

  @override
  State<StatefulWidget> createState() {
    return _TabBarViewRF();
  }
}

class _TabBarViewRF extends State<TabBarRF> {
  TabController tabController;
  int _selectedIndex = 0;
  int tabLength = 0;

  void initState() {
    if (widget.tab != null) {
      tabController = widget.tab.controller;
      tabLength = widget.tab.tabs.length;
    } else {
      tabController = widget.view.controller;
      tabLength = widget.view.tabs.length;
    }
    tabController.addListener(() {
      setState(() {
        _selectedIndex = tabController.index;
      });
    });
    super.initState();
  }

  __moveBefore() {
    if (_selectedIndex > 0) {
      setState(() => _selectedIndex--);
      tabController.animateTo(_selectedIndex);
    }
  }

  __moveAfter() {
    if (_selectedIndex < (tabLength - 1)) {
      setState(() => _selectedIndex++);
      tabController.animateTo(_selectedIndex);
    }
  }

  @override
  Widget build(BuildContext context) {
    dynamic tabItem;

//    tabController =
//        (tabController == null) ? TabController(length: 1, vsync: widget.tab.tickerProvider) : tabController;

    if (widget.tab != null) {
      widget.tab._moveBefore = __moveBefore;
      widget.tab._moveAfter = __moveAfter;
      if (widget.tab.tabBuilder != null)
        widget.tab.tabs = widget.tab.tabBuilder(_selectedIndex) as List<Widget>;
//      tabController = (tabController == null)
//          ? TabController(length: widget.tab.tabs.length, vsync: widget.tab.tickerProvider)
//          : tabController;
//      widget.tab.controller = tabController;

      tabItem = TabBar(
        labelColor: widget.tab.selectedLabelColor,
        unselectedLabelColor: widget.tab.unselectedLabelColor,
        indicatorSize: widget.tab.indicatorSize,
        isScrollable: true,
        indicator: widget.tab.indicatorDecoration,
        tabs: widget.tab.tabs,
        controller: tabController,
        onTap: (index) {
          setState(() => _selectedIndex = index);
          widget.tab.tabs = widget.tab.tabBuilder(index) as List<Widget>;
          if (widget.tab.onTap != null) widget.tab.onTap(_selectedIndex);
        },
      );
    } else if (widget.view != null) {
      tabItem = Container(
          height: 20,
          child: TabBarView(
            controller: widget.view.controller, //widget.view.controller,
            children: widget.view.views,
          ));
    }
    return tabItem;
  }
}

class TabDesignerRF extends StatefulWidget {
  int index;
  int selected;
  String label;
  Icon icon;
  Decoration decoration;

  TabDesignerRF(
    this.index,
    this.selected, {
    this.label,
    this.icon,
    this.decoration,
  });

  @override
  _TabDesignerRFState createState() => _TabDesignerRFState();
}

class _TabDesignerRFState extends State<TabDesignerRF> {
  @override
  Widget build(BuildContext context) {
    return StoreRF.scaffold()
        .widgets
        .tabBar
        .getStyle(widget.index, widget.selected, label: widget.label);
  }
}
//#endregion TabBar

//#region ContainerRF
class ContainerRF extends StatefulWidget {
  final AlignmentGeometry alignment;

  const ContainerRF({
    this.alignment,
    this.padding,
    this.margin,
    this.height,
    this.decoration,
    this.child,
    this.template,
    this.style,
  });

  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;
  final double height;
  final Decoration decoration;
  final Widget child;
  final dynamic template;
  final WidgetStyleRF style;

  @override
  _ContainerRF createState() => _ContainerRF();
}

//IContainerStyleRF clint =
//    g.clientWidgetStyle?.everything ?? g.clientWidgetStyle.containerStyleRF;
class _ContainerRF extends State<ContainerRF> {
  @override
  Widget build(BuildContext context) {
    var currentTheme = widget.template ?? Libs.config.app.template ?? TemplateRF.get();
    IContainerRFStyle clint =
        Libs.config.app.widgetStyle.all ?? Libs.config.app.widgetStyle.container;

    StyleContainerRF containerStyle =
        clint.containerRFStyle() ?? StoreRF.style(currentTheme).container.containerRFStyle();

    var alignment = widget.alignment ?? containerStyle.alignment;
    var padding = widget.padding ?? containerStyle.padding;
    var margin = widget.margin ?? containerStyle.margin;
    var decoration = widget.decoration ?? containerStyle.decoration;
    decoration = BoxDecoration(
      color: (decoration as BoxDecoration).color,
      image: (decoration as BoxDecoration).image,
      border: (decoration as BoxDecoration).border,
      borderRadius: (decoration as BoxDecoration).borderRadius,
      boxShadow: widget.style?.shadow ?? (decoration as BoxDecoration).boxShadow,
      gradient: (decoration as BoxDecoration).gradient,
      backgroundBlendMode: (decoration as BoxDecoration).backgroundBlendMode,
      shape: BoxShape.rectangle,
    );
    return Container(
      alignment: alignment,
      padding: padding,
      margin: margin,
      child: widget.child,
      decoration: decoration, //==>> remove StyleContainerRF to StyleRF
    );
  }
}
//#endregion ContainerRF

//#region TextRF
class TextRF extends StatefulWidget {
  final String text;
  final languageSource;
  final double size;
  final FontWeight weight;
  final Color color;
  final TextStyle style;
  final TextAlign align;
  final bool visible;
  final TextOverflow overflow;
  final int maxLine;
  final dynamic template;

  const TextRF(
    this.text, {
    this.languageSource,
    this.size,
    this.weight,
    this.color,
    this.visible = true,
    this.style,
    this.align = TextAlign.left,
    this.overflow = TextOverflow.ellipsis,
    this.maxLine,
    this.template,
  });

  @override
  _TextRF createState() => _TextRF();
}

//ITextStyle clintTextRF =
//    g.clientWidgetStyle?.everything ?? g.clientWidgetStyle.textStyleRF;
class _TextRF extends State<TextRF> {
  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;

/*    ITextRFStyle clintTextRF = Libs.config.app.widgetStyle.all ?? Libs.config.app.widgetStyle.text;
    StyleTextRF textStyle = clintTextRF.textRFStyle() ?? StoreRF.style(widget.template).text.textRFStyle();

    var templateDefaultControlBG; // future link with stock object
    WidgetStyleRF styleRF = Libs.config.app?.theme?.label ?? templateDefaultControlBG;*/
    IStyleRF style = Libs.config.app.widgetStyle.all;
    WidgetStyleRF cws = style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);

    var fontSize = widget.size ?? cws.fontSize;
    var fontWeight = widget.weight ?? cws.fontWeight;
    var color = widget.color ?? cws.textColor;

    var _text = '';
    if (widget.languageSource != null) {
      _text = rf.lang.text(widget.languageSource, widget.text).replaceAll('not found', '').trim();
    } else {
      if (model != null) {
        _text = rf.lang
            .text(model.runtimeType.toString(), widget.text)
            .replaceAll('not found', '')
            .trim();
      } else {
        _text = widget.text;
      }
      /*if (_text == widget.text || _text == '') {
        _text = rf.lang.text('Common', widget.text).replaceAll('not found', '').trim();
      }*/
    }
    if (_text == widget.text || _text == '') {
      _text = rf.lang.text('Common', widget.text).replaceAll('not found', '').trim();
    }

    return Visibility(
        visible: widget.visible,
        child: Text(
          _text,
          textAlign: widget.align,
          overflow: widget.overflow,
          maxLines: widget.maxLine,
          style: widget.style ??
              TextStyle(
                fontSize: fontSize,
                fontWeight: fontWeight,
                color: color,
              ),
        ));
  }
}
//#endregion TextRF

//#region DropDownTextRF

class DropdownTextRF extends StatefulWidget {
  final String text;
  final String modelName;
  final String fieldName;
  final dynamic value;
  final languageSource;
  final double fontSize;
  final FontWeight weight;
  final Color color;
  final TextStyle style;
  final TextAlign align;
  final bool visible;
  final dynamic template;

  const DropdownTextRF(
    this.modelName,
    this.fieldName,
    this.value, {
    this.text,
    this.languageSource,
    this.fontSize,
    this.weight,
    this.color,
    this.visible = true,
    this.style,
    this.align = TextAlign.left,
    this.template,
  });

  @override
  _DropdownTextRFState createState() => _DropdownTextRFState();
}

class _DropdownTextRFState extends State<DropdownTextRF> {
  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    ITextRFStyle clintTextRF = Libs.config.app.widgetStyle.all ?? Libs.config.app.widgetStyle.text;
    StyleTextRF textStyle =
        clintTextRF.textRFStyle() ?? StoreRF.style(widget.template).text.textRFStyle();

    var templateDefaultControlBG; // future link with stock object
    WidgetStyleRF styleRF = Libs.config.app?.theme?.label ?? templateDefaultControlBG;

    var fontSize = widget.fontSize ?? textStyle?.fontSize ?? styleRF.fontSize;
    var fontWeight = widget.weight ?? textStyle?.fontWeight ?? styleRF.fontWeight;
    var color = widget.color ?? textStyle?.color ?? styleRF.textColor;

    var _text = '';
    if (widget.languageSource != null) {
      _text = rf.lang.text(widget.languageSource, widget.text).replaceAll('not found', '').trim();
    } else {
      if (model != null) {
        _text = rf.lang
            .text(model.runtimeType.toString(), widget.text)
            .replaceAll('not found', '')
            .trim();
      } else {
        _text = widget.text;
      }
      /*if (_text == widget.text || _text == '') {
        _text = rf.lang.text('Common', widget.text).replaceAll('not found', '').trim();
      }*/
    }
    if (_text == widget.text || _text == '') {
      _text = rf.lang.text('Common', widget.text).replaceAll('not found', '').trim();
    }

    return Visibility(
        visible: widget.visible,
        child: FutureBuilder<String>(
          future: Db.getDropdownText(widget.modelName, widget.fieldName, widget.value),
          builder: (context, AsyncSnapshot<String> snapshot) {
            if (snapshot.hasData) {
              return Text(
                snapshot.data,
                textAlign: widget.align,
                style: widget.style ??
                    TextStyle(
                      fontSize: fontSize,
                      fontWeight: fontWeight,
                      color: color,
                    ),
              );
            } else {
              return Text(
                _text,
                textAlign: widget.align,
                style: widget.style ??
                    TextStyle(
                      fontSize: fontSize,
                      fontWeight: fontWeight,
                      color: color,
                    ),
              );
            }
          },
        ));
    /*child:
        ;*/
  }
}

//#endregion TextRF

//#region RowRF
class RowRF extends StatefulWidget {
  const RowRF(this.controls, [this.flex, this.x = 25, this.y = 10]);

  final dynamic controls;
  final double x;
  final double y;
  final List<int> flex;

  @override
  _RowRF createState() => _RowRF();
}

class _RowRF extends State<RowRF> {
  @override
  Widget build(BuildContext context) {
    int count = 1;
    var widgets = List<Widget>();

    for (var item in widget.controls) {
      var _flex;
      if (widget.flex != null) {
        _flex = widget?.flex[count - 1];
      }
      var ctl = Expanded(child: item, flex: _flex ?? 100);
      //print(_flex.toString());
      widgets.add(ctl);
      if (widget.controls.length != count) widgets.add(SizedBox(width: widget.y));
      count++;
    }
    if (widget.x != 0) {
      widgets.add(Container(
        height: widget.x,
      ));
    }

    return Row(
      mainAxisAlignment: MainAxisAlignment.end,
      //mainAxisAlignment: MainAxisAlignment.start,
      mainAxisSize: MainAxisSize.max,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: widgets,
    );
  }
}
//#endregion RowRF

//region TextFieldRF
enum ValidationTypeRF { required, range, max, min }

class TextFieldRF extends StatefulWidget {
  TextFieldRF({
    this.model,
    this.field,
    this.inputType,
    this.maxLength,
    this.maxLines = 1,
    this.hintText,
    this.hintStyle,
    this.hint,
    this.caption,
    this.onSaved,
    this.validator,
    this.enabled = true,
    this.onFieldSubmitted,
    this.validator1,
    this.onChange,
    this.backgroundColor,
    this.textColor,
    this.borderColor,
    this.fontSize,
    this.borderWidth,
    this.borderRadius,
    this.widgetStyle,
    this.fontFamily,
    this.textStyle,
    this.textAlign = TextAlign.start,
    this.textAlignVertical,
    this.textDirection,
    this.textCapitalization = TextCapitalization.none,
    this.textInputAction,
    this.autocorrect = true,
    this.autofillHints,
    this.autofocus = false,
    this.obscureText = false,
    this.cursorColor,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorWidth = 2.0,
    this.expands = false,
    this.dragStartBehavior = DragStartBehavior.start,
    this.inputFormatters,
    this.onTap,
    this.enableInteractiveSelection = true,
    this.enableSuggestions = true,
    this.keyboardAppearance,
    this.minLines,
    this.mouseCursor,
    this.scrollController,
    this.maxLengthEnforcement,
    this.obscuringCharacter = '•',
    this.readOnly = false,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.scrollPhysics,
    this.onAppPrivateCommand,
    this.onEditingComplete,
    this.onSubmitted,
    this.restorationId,
    this.showCursor,
    this.buildCounter,
    this.selectionControls,
    this.selectionHeightStyle = BoxHeightStyle.tight,
    this.selectionWidthStyle = BoxWidthStyle.tight,
    this.smartDashesType,
    this.enableIMEPersonalizedLearning = true,
    this.smartQuotesType,
    this.strutStyle,
    this.toolbarOptions,
    this.prefixIcon,
    this.suffixIcon,
    this.inputDecoration,
    this.decoration,
    this.placeholder,
    this.width = 40,
    this.height = 50,
    this.padding = const EdgeInsets.only(left: 8, right: 8, top: 2),
    this.inputBorder,
    this.hintColor,
  });

  final Object model;
  dynamic field;
  final TextInputType inputType;
  final int maxLength;
  final int maxLines;
  final String hintText;
  final String hint;
  final TextStyle hintStyle;
  final String caption;
  final bool enabled;

  final double height;
  final double width;
  final EdgeInsets padding;

  final FormFieldSetter<String> onSaved;
  final FormFieldValidator<String> validator;
  final ValueChanged<String> onFieldSubmitted;

  final Function validator1;
  final Function onChange;

  final Color backgroundColor;
  final Color textColor;
  InputBorder inputBorder;
  final Color borderColor;
  final double borderWidth;
  final Widget placeholder;

  final double borderRadius;
  final double fontSize;

  final WidgetStyleRF widgetStyle;

  final Widget prefixIcon;
  final Widget suffixIcon;
  final InputDecoration inputDecoration;
  final BoxDecoration decoration;

  final String fontFamily;
  final TextStyle textStyle;
  final TextAlign textAlign;
  final TextAlignVertical textAlignVertical;
  final TextCapitalization textCapitalization;
  final TextDirection textDirection;
  final TextInputAction textInputAction;
  final bool obscureText;
  final bool autocorrect;
  final Iterable<String> autofillHints;
  final bool autofocus;
  final Color cursorColor;
  final double cursorHeight;
  final Radius cursorRadius;
  final double cursorWidth;
  final DragStartBehavior dragStartBehavior;
  final bool expands;
  final List inputFormatters;
  final bool enableInteractiveSelection;
  final Brightness keyboardAppearance;
  final bool enableSuggestions;
  final Function onTap;
  final int minLines;
  final MouseCursor mouseCursor;
  final MaxLengthEnforcement maxLengthEnforcement;
  final String obscuringCharacter;
  final ScrollController scrollController;
  final EdgeInsets scrollPadding;
  final ScrollPhysics scrollPhysics;
  final bool readOnly;
  final Function onSubmitted;
  final bool showCursor;
  final Function onAppPrivateCommand;
  final Function onEditingComplete;
  final String restorationId;
  final TextSelectionControls selectionControls;
  final BoxHeightStyle selectionHeightStyle;
  final BoxWidthStyle selectionWidthStyle;
  final SmartDashesType smartDashesType;
  final Function buildCounter;
  final bool enableIMEPersonalizedLearning;
  final StrutStyle strutStyle;
  final ToolbarOptions toolbarOptions;
  final SmartQuotesType smartQuotesType;
  final Color hintColor;

  @override
  _TextFieldRF createState() => _TextFieldRF();
}

class _TextFieldRF extends State<TextFieldRF> {
  TextEditingController controller = TextEditingController();
  FocusNode _focus = new FocusNode(canRequestFocus: false);

  @override
  void initState() {
    _focus.addListener(_onFocusChange);
    super.initState();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  void _onFocusChange() {
    if (!_focus.hasFocus) {
      setState(() {
        if (Libs.modelValidationData[widget.field] != null)
          Libs.modelValidationData[widget.field].isValid = true;
        //controller.selection = TextSelection.fromPosition(TextPosition(offset: controller.text.length));
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    var model = widget.model ?? PageRF.of(context)?.model;
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;
    //if (widget.model != null) model = widget.model;

    if (widget.field != null) {
      Libs.modelValidationFunctions[widget.field] = () {
        if (widget.field != null) {
          var value = Dynamic(model).get(widget.field);
          ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
          setState(() {
            Libs.modelValidationData[widget.field] = data;
          });
          return data?.isValid;
        }
        return false;
      };
    }

    var fieldValue = (model == null)
        ? null
        : (widget.field == null)
            ? null
            : Dynamic(model).get(widget.field);

    if (fieldValue != null) {
      controller..text = fieldValue.toString();
    } else {
      controller..text = "";
    }

    //WidgetStyleRF styleRF = Libs.config.app?.theme?.controlBackground ?? templateDefaultControlBG ?? widget.backgroundColor;

    IStyleRF style = Libs.config.app.widgetStyle.all;
    WidgetStyleRF cws = widget.widgetStyle ??
        style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);

    var templateDefaultControlBG; // future link with stock object

    var hint = '';

    if (model != null) {
      hint = rf.lang
          .text(model.runtimeType.toString(), widget.hint)
          .replaceAll('not found', '')
          .trim();
    } else {
      hint = widget.hint;
    }
    if (hint == widget.hint || hint == '') {
      hint = rf.lang.text('Common', widget.hint).replaceAll('not found', '').trim();
    }

    TextStyle _textStyle = widget.textStyle ?? cws.textStyle;
    if (widget.textColor != null || widget.fontSize != null || widget.fontFamily != null) {
      _textStyle = TextStyle(
        color: widget.textColor ?? cws.textColor,
        fontSize: widget.fontSize ?? cws.fontSize,
        fontFamily: widget.fontFamily ?? cws.fontFamily,
      );
    }

    // overriding Layout(cws) InputDecoration to alter local hint
    InputDecoration _inputDecoration = widget.inputDecoration ?? cws.inputDecoration;
    if (widget.inputBorder != null ||
        widget.inputDecoration?.border != null ||
        widget.prefixIcon != null ||
        widget.suffixIcon != null ||
        widget.hintText != null ||
        widget.hintStyle != null) {
      _inputDecoration = InputDecoration(
        border: widget.inputBorder ?? cws.inputDecoration?.border ?? cws.inputBorder,
        hintStyle: widget.hintStyle ?? cws.inputDecoration?.hintStyle,
        hintText: widget.hintText,
        prefixIcon: widget.prefixIcon ?? cws.inputDecoration?.prefixIcon,
        suffixIcon: widget.suffixIcon ?? cws.inputDecoration?.suffixIcon,
        // add other props when get time ...???
      );
    }

    var _id = widget.inputDecoration ?? cws.inputDecoration;
    var _decoration = (_id != null)
        ? BoxDecoration(
            color: widget.backgroundColor ?? cws.backColor,
            border: Border.all(
              color: widget.borderColor ?? cws.borderColor ?? Colors.transparent,
              width: widget.borderWidth ?? cws.borderWidth ?? 0,
            ),
            borderRadius:
                BorderRadius.all(Radius.circular(widget.borderRadius ?? cws.borderRadius ?? 0)),
            boxShadow: cws.shadow,
          )
        : (widget.decoration ?? cws.decoration);

    var textFieldContainer = Container(
      padding: widget.padding,
      height: widget.height,
      decoration: _decoration ??
          BoxDecoration(
            color: widget.backgroundColor ?? cws.backColor,
            border: Border.all(
                color: widget.borderColor ?? cws.borderColor,
                width: widget.borderWidth ?? cws.borderWidth),
            borderRadius: BorderRadius.all(
              Radius.circular(widget.borderRadius ?? cws.borderRadius),
            ),
            boxShadow: cws.shadow,
          ),
      child: TextField(
        //key: ,
        //textDirection: widget.textDirection,
        textAlign: widget.textAlign,
        textAlignVertical: widget.textAlignVertical,
        textCapitalization: widget.textCapitalization,
        textInputAction: widget.textInputAction,
        obscureText: widget.obscureText,
        autocorrect: widget.autocorrect,
        autofillHints: widget.autofillHints,
        autofocus: widget.autofocus,
        cursorColor: widget.cursorColor,
        cursorHeight: widget.cursorHeight,
        cursorRadius: widget.cursorRadius,
        cursorWidth: widget.cursorWidth,
        dragStartBehavior: widget.dragStartBehavior,
        expands: widget.expands,
        inputFormatters: widget.inputFormatters,
        enableInteractiveSelection: widget.enableInteractiveSelection,
        keyboardAppearance: widget.keyboardAppearance,
        enableSuggestions: widget.enableSuggestions,
        onTap: widget.onTap,
        maxLength: widget.maxLength,
        minLines: widget.maxLines,
        mouseCursor: widget.mouseCursor,
        maxLengthEnforcement: widget.maxLengthEnforcement,
        obscuringCharacter: widget.obscuringCharacter,
        scrollController: widget.scrollController,
        scrollPadding: widget.scrollPadding,
        scrollPhysics: widget.scrollPhysics,
        readOnly: widget.readOnly,
        onSubmitted: widget.onSubmitted,
        showCursor: widget.showCursor,
        onAppPrivateCommand: widget.onAppPrivateCommand,
        onEditingComplete: widget.onEditingComplete,
        restorationId: widget.restorationId,
        selectionControls: widget.selectionControls,
        selectionHeightStyle: widget.selectionHeightStyle,
        selectionWidthStyle: widget.selectionWidthStyle,
        smartDashesType: widget.smartDashesType,
        buildCounter: widget.buildCounter,
        // enableIMEPersonalizedLearning: widget.enableIMEPersonalizedLearning,
        strutStyle: widget.strutStyle,
        toolbarOptions: widget.toolbarOptions,
        smartQuotesType: widget.smartQuotesType,
        enabled: true,
        controller: controller,
        focusNode: _focus,
        maxLines: widget.maxLines,
        keyboardType: widget.inputType,
        onChanged: (input) {
          if (model != null) Dynamic(model).set(widget.field, input.trim());
          if (widget.onChange != null) widget.onChange(input);
        },
        style: _textStyle,
        decoration: _inputDecoration,
      ),
    );

    var textFieldUI = Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.start,
      children: <Widget>[
        widget.height == null ? Expanded(child: textFieldContainer) : textFieldContainer,
        ValidationRF.showErrorMessage(widget.field)
      ],
    );

    textFieldPlaceholder() {
      if (showPlaceholder != null) cws.placeholderVisible = showPlaceholder;
      var _placeholder = widget.placeholder ?? cws.placeholder;
      if (_placeholder is EmptyRF) {
        return textFieldUI;
      } else {
        cws.placeholderVisible = false;
        if (fieldValue == null && cws.placeholderVisible) {
          return _placeholder ??
              PlaceholderRF(
                type: PlaceholderTypeRF.Inline,
                backColor: widget.widgetStyle?.placeholderBackColor ?? Colors.white,
                lineColor: widget.widgetStyle?.placeholderLineColor ?? Colors.grey,
                lines: widget.widgetStyle?.placeholderLines ?? 2,
                lineHeight: widget.widgetStyle?.placeholderLineHeight ?? 10,
                border: widget.widgetStyle?.placeholderBorder,
                borderRadius: widget.widgetStyle?.placeholderRadius ?? 4,
              );
        } else {
          return textFieldUI;
        }
      }
    }

    return IgnorePointer(ignoring: !widget.enabled, child: textFieldPlaceholder());
  }
}
//endregion TextFieldRF

//region CheckBoxDialogRF
class CheckBoxDialogRF extends StatefulWidget {
  Object model;
  String title;
  String subTitle;
  double fontSize;
  String field;
  String value;
  Color backgroundColor;
  Color textColor;
  Color checkBoxColor;
  bool enabled;
  double widgetHeight;
  final checkList;
  final Color borderColor;
  final double borderWidth;
  final double borderRadius;
  final FontWeight fontWeight;
  Function onChange;

  @override
  _CheckBoxDialogRFState createState() => _CheckBoxDialogRFState();

  CheckBoxDialogRF(
      {this.model,
      this.title = "CheckBox Dialog",
      this.subTitle = "check your data",
      this.fontSize = 16,
      this.field,
      this.value,
      this.enabled = true,
      this.backgroundColor,
      this.textColor,
      this.checkBoxColor,
      this.widgetHeight = 50,
      this.checkList,
      this.borderColor,
      this.borderWidth,
      this.borderRadius,
      this.fontWeight,
      this.onChange}) {}
}

class _CheckBoxDialogRFState extends State<CheckBoxDialogRF> {
  String selectedData = 'Select Items';
  List<RadioCheckData> cheklistData = [];

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;

    IStyleRF style = Libs.config.app.widgetStyle.all;
    WidgetStyleRF cws = style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);

    cheklistData = widget.checkList is Future ? [] : widget.checkList;
    if (widget.model != null) model = widget.model;

    if (widget.field != null) {
      Libs.modelValidationFunctions[widget.field] = () {
        if (widget.field != null) {
          var value = Dynamic(model).get(widget.field) as String;
          ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
          setState(() {
            Libs.modelValidationData[widget.field] = data;
          });
          return data?.isValid;
        }
        return false;
      };
      var value = Dynamic(model).get(widget.field);
      if (value != null) {
        widget.value = value.toString();
        selectedData = getString(widget.value);
      }
    }

    var templateDefaultControlBG; // future link with stock object
    WidgetStyleRF styleRF = Libs.config.app?.theme?.controlBackground ??
        templateDefaultControlBG ??
        widget.backgroundColor;

    return (widget.checkList is Future)
        ? FutureBuilder(
            future: widget.checkList,
            builder: (context, AsyncSnapshot<List<RadioCheckData>> snapshot) {
              if (snapshot.hasData) {
                cheklistData = snapshot.data;
                return IgnorePointer(
                  ignoring: !widget.enabled,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      InkWell(
                          onTap: () {
                            DialogRF(context,
                                type: DialogType.popup,
                                height: (cheklistData.length * 50).toDouble() >
                                        (ScreenRF.height(context) - 200)
                                    ? (ScreenRF.height(context) - 200)
                                    : (cheklistData.length * 50).toDouble(),
                                title: widget.title,
                                avatarIcon: Icons.apps_rounded,
                                subTitle: widget.subTitle,
                                body: Container(
                                  child: CheckRF(
                                    activeColor: widget.checkBoxColor ?? cws.textColor,
                                    data: cheklistData,
                                    value: widget.value,
                                    separator: true,
                                    paddingTop: 10,
                                    onChange: (value) {
                                      widget.value = value;
                                    },
                                  ),
                                ), onAccept: () {
                              if (widget.field != null)
                                Dynamic(model).set(widget.field, widget.value);
                              selectedData = getString(widget.value);
                              if (widget.onChange != null) {
                                widget.onChange(widget.value);
                              }
                              setState(() {
                                widget.value = "";
                              });
                            }, onCancel: () {
                              setState(() {
                                widget.value = "";
                              });
                            });
                          },
                          child: Container(
                            constraints: BoxConstraints(
                              minHeight: widget.widgetHeight,
                            ),
                            padding: const EdgeInsets.all(8),
                            decoration: BoxDecoration(
                                color: widget.backgroundColor ?? cws.backColor,
                                border: Border.all(
                                    color: widget.borderColor ?? cws.borderColor,
                                    width: widget.borderWidth ?? cws.borderWidth),
                                borderRadius: BorderRadius.all(
                                    Radius.circular(widget.borderRadius ?? cws.borderRadius)),
                                boxShadow: cws.shadow),
                            child: Center(
                                child: TextRF(
                              getString(widget.value),
                              style: cws.textStyle ??
                                  TextStyle(
                                      fontSize: widget.fontSize ?? cws.fontSize,
                                      color: widget.textColor ?? cws.textColor,
                                      fontWeight: widget.fontWeight ?? cws.fontWeight),
                            )),
                          )),
                      ValidationRF.showErrorMessage(widget.field)
                    ],
                  ),
                );
              } else {
                return Container(
                  constraints: BoxConstraints(
                    minHeight: widget.widgetHeight,
                  ),
                  padding: const EdgeInsets.all(8),
                  decoration: BoxDecoration(
                      color: widget.backgroundColor ?? cws.backColor,
                      border: Border.all(
                          color: widget.borderColor ?? cws.borderColor,
                          width: widget.borderWidth ?? cws.borderWidth),
                      borderRadius: BorderRadius.all(
                          Radius.circular(widget.borderRadius ?? cws.borderRadius)),
                      boxShadow: cws.shadow),
                  child: Center(
                      child: TextRF(
                    getString(widget.value),
                    style: cws.textStyle ??
                        TextStyle(
                            fontSize: widget.fontSize ?? cws.fontSize,
                            color: widget.textColor ?? cws.textColor,
                            fontWeight: widget.fontWeight ?? cws.fontWeight),
                  )),
                );
              }
            },
          )
        : IgnorePointer(
            ignoring: !widget.enabled,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                InkWell(
                    onTap: () {
                      DialogRF(context,
                          type: DialogType.popup,
                          height: (widget.checkList.length * 50).toDouble() >
                                  (ScreenRF.height(context) - 200)
                              ? (ScreenRF.height(context) - 200)
                              : (widget.checkList.length * 50).toDouble(),
                          title: widget.title,
                          avatarIcon: Icons.apps_rounded,
                          subTitle: widget.subTitle,
                          body: Container(
                            child: CheckRF(
                              activeColor: widget.checkBoxColor ?? cws.backColor,
                              data: widget.checkList,
                              value: widget.value,
                              separator: true,
                              paddingTop: 10,
                              onChange: (value) {
                                widget.value = value;
                              },
                            ),
                          ), onAccept: () {
                        if (widget.field != null) Dynamic(model).set(widget.field, widget.value);
                        selectedData = getString(widget.value);
                        if (widget.onChange != null) {
                          widget.onChange(widget.value);
                        }
                        setState(() {
                          widget.value = "";
                        });
                      }, onCancel: () {
                        setState(() {
                          widget.value = "";
                        });
                      });
                    },
                    child: Container(
                      constraints: BoxConstraints(
                        minHeight: widget.widgetHeight,
                      ),
                      padding: const EdgeInsets.all(8),
                      decoration: BoxDecoration(
                          color: widget.backgroundColor ?? cws.backColor,
                          border: Border.all(
                              color: widget.borderColor ?? cws.borderColor,
                              width: widget.borderWidth ?? cws.borderWidth),
                          borderRadius: BorderRadius.all(
                              Radius.circular(widget.borderRadius ?? cws.borderRadius)),
                          boxShadow: cws.shadow),
                      child: Center(
                          child: TextRF(
                        selectedData,
                        style: cws.textStyle ??
                            TextStyle(
                                fontSize: widget.fontSize ?? cws.fontSize,
                                color: widget.textColor ?? cws.textColor,
                                fontWeight: widget.fontWeight ?? cws.fontWeight),
                      )),
                    )),
                ValidationRF.showErrorMessage(widget.field)
              ],
            ),
          );
  }

  getString(String selections) {
    String data = "";
    if (selections != null) {
      for (RadioCheckData r in cheklistData) {
        if (selections.contains(r.value.toString()))
          data = data.isEmpty ? r.label : (data + ',' + r.label);
      }
    }
    return data.isEmpty ? "Select Items" : data;
  }
}
//endregion PopCheckRF

//region DatePickerRF
class DatePickerRF extends StatefulWidget {
  Object model;
  String field;
  DateTime initialDate;
  DateTime maxDate;
  DateTime minDate;
  Color backgroundColor;
  double widgetHeight;
  bool showInitialDate;
  bool enabled;
  final BoxDecoration decoration;
  final Color borderColor;
  final Color textColor;
  final double borderWidth;
  final double fontSize;
  final double borderRadius;
  final FontWeight fontWeight;
  Function onChange;
  final String defaultText;
  final WidgetStyleRF widgetStyle;
  final Widget placeholder;

  @override
  _DatePickerRFState createState() => _DatePickerRFState();

  DatePickerRF({
    this.model,
    this.field,
    this.initialDate,
    this.showInitialDate = false,
    this.maxDate,
    this.minDate,
    this.decoration,
    this.backgroundColor,
    this.widgetHeight = 50,
    this.enabled = true,
    this.borderColor,
    this.borderWidth,
    this.textColor,
    this.fontSize,
    this.borderRadius,
    this.fontWeight,
    this.onChange,
    this.defaultText = 'Select Date',
    this.widgetStyle,
    this.placeholder,
  }) {
    if (maxDate == null) maxDate = DateTime.now();
    if (minDate == null) minDate = DateTime(1970);
  }
}

class _DatePickerRFState extends State<DatePickerRF> {
  String dateValue = '';

  @override
  void didChangeDependencies() {
    dateValue = widget.defaultText;
    var dateFormat = DateFormat('yyyy-MM-dd');
    dateValue = widget.showInitialDate == true
        ? dateFormat.format(widget.initialDate ?? DateTime.now())
        : widget.defaultText;
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;

    IStyleRF style = Libs.config.app.widgetStyle.all;
    WidgetStyleRF cws = style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);

    if (widget.model != null) model = widget.model;

    if (widget.field != null) {
      Libs.modelValidationFunctions[widget.field] = () {
        if (widget.field != null) {
          var value = Dynamic(model).get(widget.field) as String;
          ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
          setState(() {
            Libs.modelValidationData[widget.field] = data;
          });
          return data?.isValid;
        }
        return false;
      };
      var value = Dynamic(model).get(widget.field);
      if (value != null) if (value.toString().isNotEmpty) dateValue = value.toString();
    }
    if (widget.field != null && widget.showInitialDate) Dynamic(model).set(widget.field, dateValue);
    var templateDefaultControlBG; // future link with stock object
    WidgetStyleRF styleRF = Libs.config.app?.theme?.controlBackground ??
        templateDefaultControlBG ??
        widget.backgroundColor;

    //height: 48,
    // 'f7f7f7'.toColor,
    var _decoration = widget.decoration ?? cws.decoration;
    var datePicker = Container(
      decoration: _decoration ??
          BoxDecoration(
            color: widget.backgroundColor ?? cws.backColor,
            border: Border.all(
                color: widget.borderColor ?? cws.borderColor,
                width: widget.borderWidth ?? cws.borderWidth),
            borderRadius:
                BorderRadius.all(Radius.circular(widget.borderRadius ?? cws.borderRadius)),
            boxShadow: cws.shadow,
          ),
      child: InkWell(
        child: Center(
          child: Padding(
            padding: const EdgeInsets.symmetric(horizontal: 12),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Text(
                  dateValue,
                  style: cws.textStyle ??
                      TextStyle(
                        fontSize: widget.fontSize ?? cws.fontSize,
                        color: widget.textColor ?? cws.textColor,
                        fontWeight: widget.fontWeight ?? cws.fontWeight,
                      ),
                ),
                Icon(
                  Icons.calendar_today,
                  size: 26,
                  color: Color(0xff999999),
                )
                //IconData(0xef11, fontFamily: 'MaterialIcons');
              ],
            ),
          ),
        ),
        onTap: () async {
          FocusScope.of(context).requestFocus(FocusNode());
          var dateFormat = DateFormat('yyyy-MM-dd');
          DateTime initialDate = widget.initialDate ?? DateTime.now();
          initialDate = DateTime.parse(dateFormat.format(initialDate));
          final DateTime picked = await showDatePicker(
              context: context,
              initialDate: initialDate,
              firstDate: widget.minDate,
              lastDate: widget.maxDate);
          setState(() {
            dateValue = dateFormat.format(picked);
            if (widget.field != null) Dynamic(model).set(widget.field, dateValue);
            if (widget.onChange != null) widget.onChange(dateValue);
            if (Libs.modelValidationData[widget.field] != null)
              Libs.modelValidationData[widget.field].isValid = true;
          });
        },
      ),
    );
    dropdownPlaceholder(Widget ui) {
      var _placeholder = widget.placeholder ?? cws.placeholder;
      if (_placeholder is EmptyRF) {
        return ui;
      } else {
        if ((dateValue == null || dateValue.isEmpty || dateValue == widget.defaultText) &&
            cws.placeholderVisible) {
          return Stack(
            children: [
              ui,
              PlaceholderRF(
                type: PlaceholderTypeRF.Inline,
                backColor: widget.widgetStyle?.placeholderBackColor ?? Colors.white,
                lineColor: widget.widgetStyle?.placeholderLineColor ?? Colors.grey,
                lines: widget.widgetStyle?.placeholderLines ?? 2,
                lineHeight: widget.widgetStyle?.placeholderLineHeight ?? 10,
                border: widget.widgetStyle?.placeholderBorder,
                borderRadius: widget.widgetStyle?.placeholderRadius ?? 4,
              ),
            ],
          );
        } else {
          return ui;
        }
      }
    }

    return IgnorePointer(
      ignoring: !widget.enabled,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Container(
            constraints: BoxConstraints.expand(height: widget.widgetHeight),
            child: dropdownPlaceholder(datePicker),
          ),
          ValidationRF.showErrorMessage(widget.field)
        ],
      ),
    );
  }
}
//endregion DatePickerRF

//region TimePickerRF
class TimePickerRF extends StatefulWidget {
  String field;
  final Color backgroundColor;
  final Color borderColor;
  final Color textColor;
  final double borderWidth;
  final double fontSize;
  final double borderRadius;
  final FontWeight fontWeight;

  @override
  _TimePickerRFState createState() => _TimePickerRFState();

  TimePickerRF({
    this.field,
    this.backgroundColor,
    this.borderColor,
    this.borderWidth,
    this.textColor,
    this.fontSize,
    this.borderRadius,
    this.fontWeight,
  });
}

class _TimePickerRFState extends State<TimePickerRF> {
  String timeValue = 'Select Time';
  DateTime maxDate;
  DateTime minDate;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;

    IStyleRF style = Libs.config.app.widgetStyle.all;
    WidgetStyleRF cws = style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field) as String;
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) timeValue = value.toString();

    WidgetStyleRF styleRF = Libs.config.app?.theme?.controlBackground;
    var timePicker = Container(
      decoration: BoxDecoration(
          color: widget.backgroundColor ?? cws.backColor,
          border: Border.all(
              color: widget.borderColor ?? cws.borderColor ?? Colors.transparent,
              width: widget.borderWidth ?? cws.borderWidth ?? 0),
          borderRadius:
              BorderRadius.all(Radius.circular(widget.borderRadius ?? cws.borderRadius ?? 0)),
          boxShadow: cws.shadow),
      child: InkWell(
        child: Center(
            child: Text(
          timeValue,
          style: cws.textStyle ??
              TextStyle(
                  fontSize: widget.fontSize ?? cws.fontSize,
                  color: widget.textColor ?? cws.textColor,
                  fontWeight: widget.fontWeight ?? cws.fontWeight),
        )),
        onTap: () async {
          FocusScope.of(context).requestFocus(FocusNode());
          var myFormat = DateFormat('yyyy-MM-dd');
          DateTime initalDate = DateTime.now();
          initalDate = DateTime.parse(myFormat.format(initalDate));
          final selectedTime = await showTimePicker(
            initialTime: TimeOfDay.now(),
            context: context,
          );
          setState(() {
            if (selectedTime != null) {
              timeValue = selectedTime.format(context).toString();
              Dynamic(model).set(widget.field, timeValue);
              if (Libs.modelValidationData[widget.field] != null)
                Libs.modelValidationData[widget.field].isValid = true;
            }
          });
        },
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Container(constraints: BoxConstraints.expand(height: 48), child: timePicker),
        ValidationRF.showErrorMessage(widget.field)
      ],
    );
  }
}
//endregion TimePickerRF

//region SliderRF
class SliderRF extends StatefulWidget {
  final String field;

  @override
  _SliderRFState createState() => _SliderRFState();

  SliderRF({this.field});
}

class _SliderRFState extends State<SliderRF> {
  var currentValue = [0.0];
  var _lowerValue, _upperValue;

  @override
  Widget build(BuildContext context) {
    var hasFocus = false;
    double width = 18;

    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) currentValue = [value];

    var slider = Container(
      margin: const EdgeInsets.only(right: 15),
      //color: Colors.grey,
      constraints: BoxConstraints.expand(height: 40),
      child: Stack(
        children: <Widget>[
          Positioned.fill(
            top: width,
            bottom: width,
            child: Container(
              //color: Colors.yellow,
              height: 5, //width: MediaQuery.of(context).fontSize.width,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Container(
                    width: 20,
                    height: 3.5,
                    color: Colors.blue,
                    child: Text(' '),
                  ),
                  Container(
                    width: 20,
                    height: 3.5,
                    color: Color(0xfeeeeff),
                    child: Text(' '),
                  )
                ],
              ),
            ),
          ),
          Positioned(
            child: FlutterSlider(
              trackBar: FlutterSliderTrackBar(),
              values: currentValue,
              max: 500,
              min: 0,
              onDragStarted: (handlerIndex, lowerValue, upperValue) {
                FocusScope.of(context).requestFocus(FocusNode());
                print('ds');
              },
              onDragging: (handlerIndex, lowerValue, upperValue) {
                _lowerValue = lowerValue;
                _upperValue = upperValue;

                setState(() {
                  currentValue[0] = lowerValue;
                  print(currentValue[0]);
                  Dynamic(model).set(widget.field, currentValue[0]);
                  if (Libs.modelValidationData[widget.field] != null)
                    Libs.modelValidationData[widget.field].isValid = true;
                });
              },
            ),
          ),
        ],
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[slider, ValidationRF.showErrorMessage(widget.field)],
    );
  }
}

//endregion SliderRF

//region RatingRF
class RatingRF extends StatefulWidget {
  final String field;

  @override
  _RatingRFState createState() => _RatingRFState();

  RatingRF({this.field});
}

class _RatingRFState extends State<RatingRF> {
  double initialRating = 0;

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) initialRating = value;

    var rating = Container(
      height: 40,
      child: Stack(
        overflow: Overflow.visible,
        children: <Widget>[
          Positioned.fill(
            top: 2,
            left: -10,
            child: RatingBar.builder(
              initialRating: initialRating,
              minRating: 1,
              direction: Axis.horizontal,
              allowHalfRating: true,
              itemCount: 5,
              itemSize: 30,
              itemPadding: EdgeInsets.symmetric(horizontal: 4.0),
              itemBuilder: (context, index) => Icon(
                Icons.star,
                color: Colors.amber,
              ),
              onRatingUpdate: (rating) {
                FocusScope.of(context).requestFocus(FocusNode());
                print(rating);
                Dynamic(model).set(widget.field, rating);
                setState(() {
                  if (Libs.modelValidationData[widget.field] != null)
                    Libs.modelValidationData[widget.field].isValid = true;
                });
              },
            ),
          ),
        ],
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[rating, ValidationRF.showErrorMessage(widget.field)],
    );
  }
}

//endregion RatingRF

//region SwitchRF
class SwitchRF extends StatefulWidget {
  final String field;
  final String label;

  @override
  _SwitchRFState createState() => _SwitchRFState();

  SwitchRF({this.label = "Active", this.field});
}

class _SwitchRFState extends State<SwitchRF> {
  int isSwitched = 0;

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) isSwitched = value;

    var toggleSwitch = Row(
      children: <Widget>[
        Text(widget.label),
        SizedBox(
          width: widget.label == '' ? 0 : 28,
        ),
        Switch(
          value: isSwitched == 0 ? false : true,
          onChanged: (value) {
            setState(() {
              FocusScope.of(context).requestFocus(FocusNode());
              isSwitched = value ? 1 : 0;
              Dynamic(model).set(widget.field, isSwitched);
              if (Libs.modelValidationData[widget.field] != null)
                Libs.modelValidationData[widget.field].isValid = true;
            });
          },
//          activeTrackColor: Colors.lightGreenAccent,
//          activeColor: Colors.green,
        ),
      ],
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[toggleSwitch, ValidationRF.showErrorMessage(widget.field)],
    );
  }
}

//endregion SwitchRF

//region RadioGroup
class RadioCheckData {
  final String label;
  final TextStyle textStyle;
  final dynamic value;

  RadioCheckData({this.label, this.value, this.textStyle});
}

class RadioRF extends StatefulWidget {
  final String field;
  final int value;
  bool enabled;
  final List<RadioCheckData> data;
  final dynamic dimension;
  final bool scrollable;
  final double height;
  final Function onChange;
  final bool horizontal;
  final double spaceBetween;
  final Color textColor;

  @override
  _RadioRFState createState() => _RadioRFState();

  RadioRF({
    this.data,
    this.dimension,
    this.field,
    this.scrollable,
    this.height,
    this.value,
    this.onChange,
    this.enabled = true,
    this.horizontal = false,
    this.spaceBetween = 0,
    this.textColor,
  });
}

class _RadioRFState extends State<RadioRF> {
  dynamic selectedValue; // value could be String or anything ?

  @override
  void didChangeDependencies() {
    var model = PageRF.of(context)?.model;

    var value = (widget.field == null)
        ? widget.value ?? null
        : (model == null)
            ? null
            : Dynamic(model).get(widget.field);

    //var value = Dynamic(model).get(widget.field);
    if (value != null) selectedValue = value.toString();
    //if (widget.onChange != null) widget.onChange(value);
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    updateSelectionStatus(dynamic val) {
      setState(() {
        FocusScope.of(context).requestFocus(FocusNode());
        selectedValue = val;
        if (model != null) Dynamic(model).set(widget.field, selectedValue);
        if (widget.onChange != null) widget.onChange(selectedValue);
        if (Libs.modelValidationData[widget.field] != null)
          Libs.modelValidationData[widget.field].isValid = true;
      });
    }

    var radioList = widget.horizontal
        ? Wrap(
            spacing: widget.spaceBetween,
            direction: Axis.horizontal,
            children: <Widget>[],
          )
        : Wrap(
            spacing: widget.spaceBetween,
            direction: Axis.vertical,
            children: <Widget>[],
          );

    var _baseHeight = generateRadioList(
        widget.data, selectedValue, updateSelectionStatus, radioList, widget.textColor);

    return IgnorePointer(
      ignoring: !widget.enabled,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Container(child: radioList),
          ValidationRF.showErrorMessage(widget.field)
          //Text('test')
        ],
      ),
    );
  }

  double generateRadioList(
      dynamic _data, dynamic _selectedValue, _fn, dynamic _radioStack, Color textColor) {
    _radioStack.children.clear();
    for (RadioCheckData item in _data) {
      var pos = Row(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          Radio(
              materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
              value: item.value.toString(),
              groupValue: _selectedValue,
              onChanged: (value) {
                _fn(value.toString());
                print(value);
              }),
          //Text(item.label, style: TextStyle(fontWeight: FontWeight.bold))
          CheckRadioLabelRF(
            index: item.value.toString(),
            label: item.label,
            style: item.textStyle,
            textColor: textColor,
            onChange: (value) {
              _fn(value.toString());
              print(value.toString());
            },
          )
        ],
      );

      _radioStack.children.add(Container(child: pos));
    }
    return 10; // 10 for click last item
  }
}
//endregion RadioGroup

//region CheckboxGroup
class CheckRF extends StatefulWidget {
  final List<RadioCheckData> data;
  final dynamic dimension;
  final String field;
  final dynamic value;
  final bool scrollable;
  final bool enabled;
  final double height;
  final Function onChange;
  final bool separator;
  final double paddingTop;
  final Color activeColor;

  @override
  _CheckRFState createState() => _CheckRFState();

  CheckRF({
    this.data,
    this.dimension,
    this.field,
    this.value,
    this.scrollable,
    this.height = 30,
    this.onChange,
    this.enabled = true,
    this.separator = false,
    this.paddingTop = 0,
    this.activeColor,
  });
}

class _CheckRFState extends State<CheckRF> {
  Map<dynamic, bool> currentValueMap = Map();
  double baseHeight = 30;
  WidgetStyleRF cws;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  void initState() {
    var model = PageRF.of(context)?.model;
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;

    IStyleRF style = Libs.config.app.widgetStyle.all;
    cws = style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);
    // set value
    var value = (widget.field == null)
        ? widget.value ?? null
        : (model == null)
            ? null
            : Dynamic(model).get(widget.field);

    if (value != null) {
      Map<dynamic, bool> currentValueMapFromString = Map();
      for (var item in value.toString().split(RegExp(','))) {
        if (!item.isEmpty) currentValueMapFromString[item.toString()] = true;
      }
      currentValueMap = currentValueMapFromString;
      //if (widget.onChange != null) widget.onChange(value);
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    // on Select
    updateSelectionStatus(dynamic value, bool val) {
      var string = '';

      FocusScope.of(context).requestFocus(FocusNode());
      currentValueMap[value] = val;
      currentValueMap.forEach((k, v) {
        if (v) {
          if (string.isNotEmpty)
            string += ',${k.toString()}';
          else
            string = '${k.toString()}';
        }
      });
      print(string);
      if (widget.onChange != null) widget.onChange(string);
      if (model != null) if (widget.field != null) Dynamic(model).set(widget.field, string);
      if (Libs.modelValidationData[widget.field] != null)
        Libs.modelValidationData[widget.field].isValid = true;
      setState(() {});
    }

    var checkStack = Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[],
    );
    var _baseHeight =
        generateCheckList(widget.data, currentValueMap, updateSelectionStatus, checkStack);

    return IgnorePointer(
      ignoring: !widget.enabled,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Container(
            height: widget.data.length > 1 ? (widget.data.length * 50).toDouble() : widget.height,

            child: Container(child: checkStack),
            //color: Colors.transparent,
          ),
          ValidationRF.showErrorMessage(widget.field)
        ],
      ),
    );
  }

  double generateCheckList(
      List<RadioCheckData> _data, Map<dynamic, bool> _currentValueMap, _fn, dynamic _checkStack) {
    _checkStack.children.clear();
    for (RadioCheckData item in _data) {
      var pos = Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          SizedBox(height: widget.paddingTop),
          Container(
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                SizedBox(
                  width: 24,
                  height: 24,
                  child: Checkbox(
                    activeColor: widget.activeColor ?? cws.textColor,
                    value: _currentValueMap[item.value] ?? false,
                    onChanged: (newValue) => _fn(item.value, newValue),
                  ),
                ),
                SizedBox(
                  width: 12,
                ),
                //Text(item.label, style: TextStyle(fontWeight: FontWeight.bold))
                item.label.trim().isEmpty
                    ? Container()
                    : Expanded(
                        child: CheckRadioLabelRF(
                          index: item.value,
                          label: item.label,
                          /*onChange: (value) {
                            _fn(value, _currentValueMap[value] == null ? false : !_currentValueMap[value]);
                          },*/
                        ),
                      )
              ],
            ),
          ),
          (widget.separator)
              ? (item == _data[_data.length - 1])
                  ? Container()
                  : Container(
                      height: 1,
                      color: Colors.black12,
                      width: MediaQuery.of(context).size.width - 100,
                    )
              : Container(),
        ],
      );
      _checkStack.children.add(pos);
    }
  }
}

class CheckRadioLabelRF extends StatefulWidget {
  dynamic index;
  String label;
  TextStyle style;
  Function onChange;
  Color textColor;

  CheckRadioLabelRF({this.index, this.label, this.style, this.onChange, this.textColor});

  @override
  _CheckRadioLabelRFState createState() => _CheckRadioLabelRFState();
}

class _CheckRadioLabelRFState extends State<CheckRadioLabelRF> {
  @override
  Widget build(BuildContext context) {
    var pageData = PageRF.of(context)?.pageData;
    var showPlaceholder = PageRF.of(context)?.showPlaceholder;

    IStyleRF style = Libs.config.app.widgetStyle.all;
    WidgetStyleRF cws = style.customStyle(context, widget.runtimeType, pageData, showPlaceholder);
    return Material(
      color: Colors.transparent,
      child: InkWell(
        onTap: () {
          widget.onChange(widget.index.toString());
        },
        child: Container(
          child: Padding(
            padding: const EdgeInsets.only(top: 9.0, bottom: 8.0),
            child: TextRF(
              widget.label,
              style: widget.style ??
                  cws.textStyle ??
                  TextStyle(
                    fontWeight: FontWeight.bold,
                    color: widget.textColor ?? cws.textColor ?? Colors.black,
                  ),
            ),
          ),
        ),
      ),
    );
  }
}

//CheckboxGroup(
//orientation: GroupedButtonsOrientation.VERTICAL,
//padding: EdgeInsets.only(left: 0),
//margin: EdgeInsets.all(0),
//labels: <String>[
//"Sports",
//"Reading",
//],
//onSelected: (List<String> checked) => print(checked.toString()),
//
//)
//endregion CheckboxGroup

//region GridViewRF
const gridPadding = EdgeInsets.all(10);

class GridViewRFOLD extends StatefulWidget {
  final dynamic itemList;
  final dynamic child;
  final EdgeInsetsGeometry padding;
  final dynamic decoration;
  final int column;
  final double spaceX;
  final double spaceY;
  final bool isScroll;
  final GridBuild build;

  const GridViewRFOLD(
      {this.itemList,
      this.child,
      this.padding = gridPadding,
      this.decoration,
      this.column = 2,
      this.spaceX = 2.0,
      this.spaceY = 2.0,
      this.isScroll = true,
      this.build});

  @override
  _GridViewRF createState() => _GridViewRF();
}

typedef GridBuild = Widget Function(BuildContext context, int index);

class _GridViewRF extends State<GridViewRFOLD> {
  @override
  Widget build(BuildContext context) {
    dynamic modelList = widget.itemList;

    return Container(
        padding: widget.padding,
        child: GridView.builder(
          physics: !widget.isScroll ? const NeverScrollableScrollPhysics() : null,
          itemCount: modelList.length,
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: widget.column,
              crossAxisSpacing: widget.spaceY,
              mainAxisSpacing: widget.spaceX),
          itemBuilder: (BuildContext context, int index) {
            return widget.build(context, index);
          },
        ));
  }
}
//endregion

//#region FilterRF

class FilterRF extends StatefulWidget {
  Function onAdvanceFilter;
  List<DataSourceRF> dataSource;
  Color color;
  String quickFilterFields;
  String defaultSorting;
  bool showAdvanceButton;
  bool showGeoButton;
  bool showRefreshButton;
  Color quickFilterBackColor;
  Color showAdvanceButtonBackColor;
  Color showRefreshButtonBackColor;
  Color showGeoButtonBackColor;
  Color quickFilterForeColor;
  Color quickFilterHintForeColor;
  Color showAdvanceButtonForeColor;
  Color showRefreshButtonForeColor;
  Color showGeoButtonForeColor;
  Function onCustomActionTap;
  Function onTextChange;
  Function onRefresh;

  @override
  _FilterRFState createState() => _FilterRFState();

  FilterRF({
    this.dataSource,
    this.onAdvanceFilter,
    this.quickFilterFields,
    this.defaultSorting = 'DESC',
    this.color,
    this.showAdvanceButton = true,
    this.showGeoButton = true,
    this.showRefreshButton = true,
    this.quickFilterBackColor = Colors.white,
    this.showAdvanceButtonBackColor = Colors.white,
    this.showRefreshButtonBackColor = Colors.white,
    this.showGeoButtonBackColor = Colors.white,
    this.quickFilterForeColor = Colors.grey,
    this.quickFilterHintForeColor = Colors.grey,
    this.showAdvanceButtonForeColor = Colors.grey,
    this.showRefreshButtonForeColor = Colors.grey,
    this.showGeoButtonForeColor = Colors.grey,
    this.onCustomActionTap,
    this.onTextChange,
    this.onRefresh,
  });
}

class _FilterRFState extends State<FilterRF> {
  var quickFilterValue;
  final textController = TextEditingController();

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: const EdgeInsets.only(left: 20, right: 20),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Expanded(
            child: Container(
              height: 42,
              decoration: BoxDecoration(
                color: widget.quickFilterBackColor ?? Colors.grey[200],
                borderRadius: BorderRadius.all(Radius.circular(8)),
              ),
              child: Row(
                children: [
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Icon(Icons.search, color: widget.quickFilterForeColor ?? Colors.grey),
                  ),
                  //Search Textbox field
                  Expanded(
                    child: Center(
                      child: TextField(
                        style: TextStyle(color: widget.quickFilterForeColor ?? Colors.black),
                        autofocus: false,
                        controller: textController,
                        decoration: InputDecoration(
                          border: InputBorder.none,
                          hintText: 'Search',
                          hintStyle: TextStyle(
                            color: widget.quickFilterHintForeColor ?? Colors.grey,
                          ),
                        ),
                        onChanged: (input) async {
                          if (widget.onTextChange != null) widget.onTextChange(input);
                          for (var source in widget.dataSource) {
                            quickFilterValue = input;
                            source.currentPage = 1;
                            await source.quickFilter(
                              '${widget.quickFilterFields}#$quickFilterValue#${widget.defaultSorting}',
                            );
                          }
                        },
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
          // Advance Button
          Visibility(
            visible: widget.showAdvanceButton,
            child: InkWell(
              onTap: () {
                FocusScope.of(context).requestFocus(FocusNode());
                widget.dataSource.forEach((element) {
                  element.currentPage = 1;
                });
                widget.onAdvanceFilter();
              },
              child: Container(
                margin: EdgeInsets.only(left: 10),
                //advance filter
                width: 40,
                height: 40,
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Image.asset("assets/images/settings.png",
                      color: widget.showAdvanceButtonForeColor ?? Colors.grey),
                ),
                decoration: BoxDecoration(
                    color: widget.showAdvanceButtonBackColor ?? Colors.white,
                    //shape: BoxShape.circle,
                    boxShadow: [
                      BoxShadow(
                        color: Colors.grey.withOpacity(0.3),
                        blurRadius: 3.0,
                        spreadRadius: 1.0,
                      ),
                    ],
                    borderRadius: BorderRadius.all(Radius.circular(10))),
              ),
            ),
          ),
          // Refresh Button
          Visibility(
            visible: widget.showRefreshButton,
            child: InkWell(
              onTap: () {
                FocusScope.of(context).requestFocus(FocusNode());
                widget.dataSource.forEach((element) {
                  element.filterClear();
                });
                textController.text = '';
                widget.onRefresh != null ? widget.onRefresh() : {};
              },
              child: Container(
                margin: EdgeInsets.only(left: 10),
                width: 40,
                height: 40,
                child: Icon(Icons.refresh, color: widget.showRefreshButtonForeColor ?? Colors.grey),
                decoration: BoxDecoration(
                    color: widget.showRefreshButtonBackColor ?? Colors.white,
                    //shape: BoxShape.circle,
                    boxShadow: [
                      BoxShadow(
                        color: Colors.grey.withOpacity(0.3),
                        blurRadius: 3.0,
                        spreadRadius: 1.0,
                      ),
                    ],
                    borderRadius: BorderRadius.all(Radius.circular(10))),
              ),
            ),
          ),
          // Geo Button
          Visibility(
            visible: widget.showGeoButton,
            child: InkWell(
              onTap: () {
                FocusScope.of(context).requestFocus(FocusNode());
                widget.onCustomActionTap("geo");
              },
              child: Container(
                margin: EdgeInsets.only(left: 10),
                width: 40,
                height: 40,
                child: Icon(Icons.my_location, color: widget.showGeoButtonForeColor ?? Colors.grey),
                decoration: BoxDecoration(
                    color: widget.showGeoButtonBackColor ?? Colors.white, //shape: BoxShape.circle,
                    boxShadow: [
                      BoxShadow(
                        color: Colors.grey.withOpacity(0.3),
                        blurRadius: 3.0,
                        spreadRadius: 1.0,
                      ),
                    ],
                    borderRadius: BorderRadius.all(Radius.circular(10))),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
//#endregion FilterRF

//#region ListViewRFItems
class ListCardRF extends StatefulWidget {
  final leading;
  final body;
  final action;
  final height;
  final statusColor;
  final dateTextColor;
  final dateBackgroundColor;
  final bottomLevel;
  final EdgeInsets padding;

  @override
  _ListCardRFState createState() => _ListCardRFState();

  ListCardRF({
    this.leading,
    this.statusColor = Colors.green,
    this.body,
    this.height,
    this.action,
    this.bottomLevel,
    this.dateTextColor,
    this.dateBackgroundColor,
    this.padding,
  });
}

class _ListCardRFState extends State<ListCardRF> {
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(
        top: widget?.padding?.top ?? 5,
        bottom: widget?.padding?.bottom ?? 10,
        left: widget?.padding?.left ?? 10,
        right: widget?.padding?.right ?? 10,
      ),
      child: Container(
        padding: EdgeInsets.only(top: 8),
        child: Container(
          decoration: BoxDecoration(
              boxShadow: [
                BoxShadow(
                  color: Colors.black.withOpacity(0.1),
                  spreadRadius: 0,
                  blurRadius: 5,
                  offset: Offset(0, -5), // changes position of shadow
                ),
                BoxShadow(
                  color: Colors.black.withOpacity(0.2),
                  spreadRadius: 0,
                  blurRadius: 5,
                  offset: Offset(0, 5), // changes position of shadow
                ),
              ],
              color: widget.statusColor,
              borderRadius: new BorderRadius.only(
                  topLeft: Radius.circular(20),
                  bottomLeft: Radius.circular(20),
                  topRight: Radius.circular(20),
                  bottomRight: Radius.circular(20))),
          child: Container(
            height: widget.height ?? 90,
            margin: EdgeInsets.only(left: 6),
            decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: new BorderRadius.only(
                    topLeft: Radius.circular(20),
                    bottomLeft: Radius.circular(20),
                    topRight: Radius.circular(20),
                    bottomRight: Radius.circular(20))),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                Expanded(
                  child: Stack(
                    children: [
                      Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: <Widget>[
                          SizedBox(
                            width: 10,
                          ),
                          widget.leading ?? Container(),
                          Expanded(
                            child: Padding(
                              padding: EdgeInsets.only(left: 18.0, top: 5, right: 20),
                              child: widget.body,
                            ),
                            flex: 70,
                          ),
                        ],
                      ),
                      Positioned(
                        right: 0,
                        bottom: 0,
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.end,
                          mainAxisAlignment: MainAxisAlignment.end,
                          children: [
                            Container(
                              margin: EdgeInsets.only(right: 12),
                              child: widget.action,
                            ),
                            widget.bottomLevel == null
                                ? Container()
                                : Align(
                                    alignment: Alignment.bottomRight,
                                    child: Container(
                                      height: 30,
                                      width: 140,
                                      decoration: BoxDecoration(
                                          color: widget.dateBackgroundColor ?? Color(0Xffffb1b3),
                                          borderRadius: new BorderRadius.only(
                                            topLeft: Radius.circular(30),
                                            bottomRight: Radius.circular(20),
                                          )),
                                      child: Padding(
                                        padding: const EdgeInsets.only(left: 12.0),
                                        child: Center(
                                          child: TextRF(
                                            widget.bottomLevel,
                                            size: 14,
                                            color: widget.dateTextColor ?? '7f3133'.toColor,
                                            //Colors.brown[800],
                                            weight: FontWeight.w600,
                                          ),
                                        ),
                                      ),
                                    ),
                                  )
                          ],
                        ),
                      )
                    ],
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}

class CardAvatarRF extends StatefulWidget {
  @required
  final image;
  final borderColor;
  final borderWidth;
  final statusColor;

  @override
  _CardAvatarRFState createState() => _CardAvatarRFState();

  CardAvatarRF({
    this.image,
    this.borderColor,
    this.borderWidth,
    this.statusColor = Colors.green,
  });
}

class _CardAvatarRFState extends State<CardAvatarRF> {
  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        Container(
            width: 50.0,
            height: 50.0,
            decoration: new BoxDecoration(
              color: Color(0XffFC6767).withOpacity(0.3),
/*            image: new DecorationImage(
              image: AssetImage(widget.image),
              fit: BoxFit.cover,
            ),*/
              borderRadius: new BorderRadius.all(new Radius.circular(50.0)),
              /*    border: new Border.all(
              color: Colors.red,
              width: 2.0,
            ),*/
            ),
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Image(
                image: AssetImage(widget.image),
              ),
            )),
/*        Positioned(
          bottom: 0,
          right: 0,
          child: CircleAvatar(
            backgroundColor:  Colors.white,
            radius: 9,
            child: CircleAvatar(
              backgroundColor: widget.statusColor,
              radius: 7,
            ),
          ),
        )*/
      ],
    );
  }
}

class CardActionRF extends StatefulWidget {
  final icon;
  final deleteId;
  final index;
  final size;
  final color;
  final onAction;
  DataSourceRF dataSource;
  List<Function> modelValidators;

  @override
  _CardActionRFState createState() => _CardActionRFState();

  CardActionRF(this.icon, this.deleteId, this.dataSource, this.index,
      {this.color, this.size, this.onAction, this.modelValidators});
}

class _CardActionRFState extends State<CardActionRF> {
  @override
  Widget build(BuildContext context) {
    var model = Models.models[Dynamic.genericModelName(widget.dataSource.modelList)].model;
    return IconButton(
        icon: Icon(widget.icon),
        iconSize: 40,
        //color: Color.fromRGBO(140, 140, 140, 1),
        color: Colors.grey.shade400,
        onPressed: () {
          AlertRF(
              context: context,
              title: 'DeleteAlertTitle',
              icon: Icons.error,
              msg: 'DeleteALertDescription',
              confirmText: 'DELETE',
              onConfirm: (value) async {
                if (value) {
                  var ret = await rf.db.delete(model, widget.deleteId, widget.modelValidators);
                  if (ret == 0) {
                    rf.msg(context, 'DeleteALertSyncedData');
                  } else {
                    widget.dataSource.delete(widget.index);
                    widget.dataSource.refresh();
                    setState(() {});
                  }
                }
                if (widget.onAction != null) widget.onAction('');
              }).show();
        }
        //widget.onAction,
        );
  }
}
//#endregion ListViewRFItems

//#region GridCardItems
class GridCardRF extends StatefulWidget {
  int index;
  List<String> cardTitleList;
  List<IconData> cardImageList;
  Function onTap;

  GridCardRF({
    this.index,
    this.cardTitleList,
    this.cardImageList,
    this.onTap,
  });

  @override
  _GridCardRFState createState() => _GridCardRFState();
}

class _GridCardRFState extends State<GridCardRF> {
  @override
  Widget build(BuildContext context) {
    return FlatButton(
      onPressed: () => widget.onTap(widget.index),
      color: Colors.white,
      child: Container(
        //color: Colors.red,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            FaIcon(widget.cardImageList[widget.index], size: 40, color: Colors.pink),
            SizedBox(height: 5),
            Center(
              child: Text(
                widget.cardTitleList[widget.index].toUpperCase(),
                textAlign: TextAlign.center,
                style: TextStyle(fontWeight: FontWeight.bold, fontSize: 15),
              ),
            )
          ],
        ),
      ),
    );
  }
}
//#endregion GridCardItems

//#region LabelAndTitles
class LabelRF extends StatefulWidget {
  final text;

  @override
  _LabelRFState createState() => _LabelRFState();

  LabelRF(this.text);
}

class _LabelRFState extends State<LabelRF> {
  @override
  Widget build(BuildContext context) {
    return TextRF(
      widget.text,
      style: TextStyle(fontWeight: FontWeight.bold, fontSize: 14),
    );
  }
}

class PageTitleRF extends StatefulWidget {
  final text;
  final Color textColor;
  final Color borderColor;
  final double textSize;

  @override
  _PageTitleRFState createState() => _PageTitleRFState();

  PageTitleRF(this.text, {this.textColor, this.borderColor, this.textSize});
}

class _PageTitleRFState extends State<PageTitleRF> {
  @override
  Widget build(BuildContext context) {
    return Container(
      height: widget.text.length > 40 ? 70 : 40,
      width: 100,
      child: Stack(
        overflow: Overflow.visible,
        children: <Widget>[
          Positioned(
            left: -10,
            child: Container(
                width: ScreenRF.width(context),
                child: Padding(
                  padding: const EdgeInsets.only(left: 3, top: 5, bottom: 5),
                  child: Text(
                    widget.text,
                    style: TextStyle(
                      color: widget.textColor ?? Colors.pink,
                      fontSize: widget.textSize ?? 20,
                      fontWeight: FontWeight.bold,
                      fontFamily: 'Roboto',
                      letterSpacing: 1,
                      decoration: TextDecoration.none,
                    ),
                  ),
                ),
                decoration: BoxDecoration(
                    border: Border(
                        left: BorderSide(
                  color: widget.borderColor ?? Colors.pink,
                  width: 8,
                )))),
          )
        ],
      ),
    );
  }
}

class PageSubTitleRF extends StatefulWidget {
  final text;
  final Color textColor;
  final int textSize;

  @override
  _PageSubTitleRFState createState() => _PageSubTitleRFState();

  PageSubTitleRF(this.text, {this.textColor, this.textSize});
}

class _PageSubTitleRFState extends State<PageSubTitleRF> {
  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 15.0),
      child: TextRF(
        widget.text,
        style: TextStyle(
          color: widget.textColor ?? Colors.blueAccent,
          fontSize: widget.textSize ?? 17,
          fontWeight: FontWeight.bold,
          fontFamily: 'Roboto',
          letterSpacing: 1,
          decoration: TextDecoration.none,
        ),
      ),
    );
  }
}
//#endregion LabelAndTitles

//#region GapRF
enum GapTypeRF { Gap5, Gap10 }

class GapRF extends StatefulWidget {
  double gap;

  GapRF([this.gap = 5]);

  @override
  _GapRFState createState() => _GapRFState();
}

class _GapRFState extends State<GapRF> {
  @override
  Widget build(BuildContext context) {
    return Container(
      height: widget.gap,
      width: widget.gap,
    );
  }
}
//#endregion GapRF

//#region GraphRF

enum GraphTypeRF { line, polar }

class GraphRF extends StatefulWidget {
  String modelName;
  String fieldName;
  GraphTypeRF graphType;

  GraphRF({this.modelName, this.fieldName, this.graphType = GraphTypeRF.line});

  @override
  _GraphRFState createState() => _GraphRFState();
}

class _GraphRFState extends State<GraphRF> {
  DataSourceRF dataSource;
  List<Map<String, dynamic>> graphData = [];

  Future<List<Map<String, dynamic>>> getData() async {
    Models models = Models();
    for (var element in models.get().entries) {
      if (element.key.toString() == widget.modelName) {
        String sql =
            "select COUNT(*) as sold,${widget.fieldName} as genre from ${widget.modelName} group by ${widget.fieldName}";
        print(sql);
        await rf.db.get(GraphItem, sql, null, (modellist) {
          for (int i = 0; i < modellist.length; i++) {
            graphData.add((modellist[i] as GraphItem).toJson());
          }
          print(graphData.toString());
        });
      }
    }
    return graphData;
  }

  @override
  Widget build(BuildContext context) {
    return Container();
    // return Container(
    //   height: 200,
    //   width: ScreenRF.width(context) - 20,
    //   child: FutureBuilder<List<Map<String, dynamic>>>(
    //     future: getData(),
    //     builder: (context, AsyncSnapshot<List<Map<String, dynamic>>> snapshot) {
    //       if (!snapshot.hasData) {
    //         return Container();
    //       } else {
    //         if (widget.graphType == GraphTypeRF.line) {
    //           return graphic.Chart(
    //             data: snapshot.data,
    //             scales: {
    //               'genre': graphic.CatScale(
    //                 accessor: (map) => map['genre'] as String,
    //               ),
    //               'sold': graphic.LinearScale(
    //                 accessor: (map) => map['sold'] as num,
    //                 nice: true,
    //               )
    //             },
    //             geoms: [
    //               graphic.IntervalGeom(
    //                 position: graphic.PositionAttr(field: 'genre*sold'),
    //                 shape: graphic.ShapeAttr(values: [graphic.RectShape(radius: Radius.circular(5))]),
    //               )
    //             ],
    //             axes: {
    //               'genre': graphic.Defaults.horizontalAxis,
    //               'sold': graphic.Defaults.verticalAxis,
    //             },
    //           );
    //         } else {
    //           return graphic.Chart(
    //             data: snapshot.data,
    //             scales: {
    //               'genre': graphic.CatScale(
    //                 accessor: (map) => map['genre'] as String,
    //               ),
    //               'sold': graphic.LinearScale(
    //                 accessor: (map) => map['sold'] as num,
    //                 nice: true,
    //               )
    //             },
    //             coord: graphic.PolarCoord(),
    //             geoms: [
    //               graphic.IntervalGeom(
    //                 position: graphic.PositionAttr(field: 'genre*sold'),
    //                 color: graphic.ColorAttr(field: 'genre'),
    //               )
    //             ],
    //             axes: {
    //               'genre': graphic.Defaults.circularAxis,
    //               'sold': graphic.Defaults.radialAxis..label = null,
    //             },
    //           );
    //         }
    //       }
    //     },
    //   ),
    // );

    /*  locations.add(GraphItem(genre: "afdas", sold: 11).toJson());
    return Container(
      height: 200,
      width: ScreenRF.width(context) - 20,
      child: graphic.Chart(
        data: locations,
        scales: {
          'genre': graphic.CatScale(
            accessor: (map) => map['genre'] as String,
          ),
          'sold': graphic.LinearScale(
            accessor: (map) => map['sold'] as num,
            nice: true,
          )
        },
        geoms: [
          graphic.IntervalGeom(
            position: graphic.PositionAttr(field: 'genre*sold'),
          )
        ],
        axes: {
          'genre': graphic.Defaults.horizontalAxis,
          'sold': graphic.Defaults.verticalAxis,
        },
      ),
    );*/
  }
}

@annotation.Model
class GraphItem {
  String genre;
  int sold;

  GraphItem({this.genre, this.sold});

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['genre'] = this.genre;
    data['sold'] = this.sold;
    return data;
  }
}

//#endregion GapRF

//#region PageRFItems
Widget fab(Function onPressed) {
  return FloatingActionButton(
    backgroundColor: Colors.red,
    elevation: 10,
    onPressed: () {
      onPressed();
      //navigateToDetail(Profile(), 'Add Profile');
    },
    tooltip: 'Add',
    child: Icon(
      Icons.add,
      color: Colors.white,
    ),
  );
}

class BounceScrollBehavior extends ScrollBehavior {
  @override
  getScrollPhysics(_) => const BouncingScrollPhysics();
}
//#endregion PageRFItems

//#region OnboardItems
enum RippleEffectRF { style1 }

class SplashRF {
  String title;
  String subTitle;
  Color color;
  String font;
  Color titleColor;
  Color subTitleColor;
  Color rippleColor;
  String rippleImagePath;
  double rippleImageSize;
  String orgLogoPath;
  int duration;

  SplashRF({
    this.title,
    this.subTitle,
    this.color,
    this.font,
    this.titleColor,
    this.subTitleColor,
    this.rippleColor,
    this.rippleImagePath,
    this.rippleImageSize,
    this.orgLogoPath,
    this.duration = 5,
  });
}

class IntroRF {
  Widget title;
  Widget image;
  Widget description;
  Widget pointerImage;
  Color backgroundColor;
  RippleEffectRF rippleEffect;
  Color rippleColor;

  IntroRF({
    this.title,
    this.image,
    this.description,
    this.pointerImage,
    this.backgroundColor,
    this.rippleEffect,
    this.rippleColor,
  });
}

class WidgetsRF {
  static selectWidget(local, clint, store) {
    var widget = Object();
    if (local is EmptyRF) {
      widget = null;
    } else if (local.runtimeType.toString().lastIndexOf('RF') > 1) {
      if (local is TextRF || local is ContainerRF || local is DrawerRF) {
        if (local?.template?.version == 'EmptyRF') {
          widget = null;
        }
      }
    }

    if (clint is EmptyRF) {
      widget = null;
    } else if (clint.runtimeType.toString().lastIndexOf('RF') > 1) {
      if (clint is TextRF || clint is ContainerRF || clint is DrawerRF) {
        if (clint?.template?.version == 'EmptyRF') {
          widget = null;
        }
      }
    }

    if (widget == null) {
      return null;
    } else {
      return local ?? clint ?? store;
    }
  }
}
//#endregion OnboardItems

//#region Experiment
class StoryboardRF {
  String title; //
  OutlineRF outline; //
  FlexBarRF flex; //
  //List<Widget> actions; ////
  Object model;
  List<DataSourceRF> dataSources;
  Widget stickyHeader;
  Widget child;
  final Widget fab; //
  List<DrawerItemRF> drawerItems;
  DrawerItemRF drawerFooter;
  List<Color> osBarColors;

  StoryboardRF(
      {this.title,
      this.outline,
      this.flex,
      this.stickyHeader,
      this.model,
      this.dataSources, //this.actions,
      this.child,
      this.fab,
      this.drawerItems,
      this.drawerFooter,
      this.osBarColors});
}

class PageBuilderRF extends StatefulWidget {
  final Function(BuildContext) builder;

  const PageBuilderRF({Key key, this.builder}) : super(key: key);

  @override
  PageBuilderRFState createState() => new PageBuilderRFState();

  static PageBuilderRFState of(BuildContext context) {
    return context.findAncestorStateOfType<PageBuilderRFState>();
    // return context.ancestorStateOfType(const TypeMatcher<PageBuilderRFState>());
  }
}

class PageBuilderRFState extends State<PageBuilderRF> {
  @override
  Widget build(BuildContext context) {
    return widget.builder(context);
  }

  void rebuild() {
    setState(() {});
  }
}
//#endregion Experiment

//region BackupOld
/*
Stack(
        children: [
        Container(
          width: 10,
            height: 90,
            decoration:BoxDecoration(color: Colors.red, borderRadius: new BorderRadius.only(topLeft:Radius.circular(10),bottomLeft: Radius.circular(10))),
          ),
        Container(
          height: 90,
          margin: EdgeInsets.only(left:10),
          decoration:BoxDecoration(color:  Colors.white, borderRadius: new BorderRadius.only(topRight:Radius.circular(10),bottomRight:Radius.circular(10))),
          child: Column(
            mainAxisAlignment:MainAxisAlignment.spaceEvenly ,
            children: <Widget>[
              Expanded(
                child: Row(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: <Widget>[
                    SizedBox(width: 10,),
                    widget.leading,
                    Expanded(
                      child: Padding(
                        padding: EdgeInsets.only(left: 18.0, top: 5),
                        child: widget.body,
                      ),
                      flex: 70,
                    ),
                    Expanded(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.end,
                        mainAxisAlignment: MainAxisAlignment.end,
                        children: [
                        widget.action,
                        Align(
                          alignment: Alignment.bottomRight,
                          child: Container(
                            height: 30,
                            decoration:BoxDecoration(color: Color(0XffF6E3B1), borderRadius: new BorderRadius.only(topLeft:Radius.circular(30),bottomRight:Radius.circular(10))),
                            child: Padding(
                              padding: const EdgeInsets.only(left:12.0),
                              child: Center(child: TextRF("December 20.2020",color: Color(0XffFF8A00),weight: FontWeight.w300,size: 12,),),
                            ),
                          ),
                        )
                      ],),
                      flex: 40,
                    ),

                  ],
                ),
              ),
            ],
          ),
        ),
      ],

      )
*/
//endregion BackupOld

//region StepperRF
class StepperPageRF extends StatefulWidget {
  List<String> titles;
  List<Widget> forms;
  Color buttonColor;
  Color tabActiveColor;
  Color tabActiveTextColor;
  Color tabInActiveColor;
  Color tabActiveAvatarColor;
  Color tabInActiveTextColor;
  Color tabInActiveAvatarColor;
  Color buttonTextColor;
  Icon buttonNextIcon;
  Icon buttonSubmitIcon;
  Function onSubmit;
  Function onNext;

  @override
  _StepperPageRFState createState() => _StepperPageRFState();

  StepperPageRF({
    this.titles,
    this.forms,
    this.onSubmit,
    this.onNext,
    this.buttonColor,
    this.tabActiveColor,
    this.buttonTextColor,
    this.buttonNextIcon,
    this.buttonSubmitIcon,
    this.tabInActiveColor,
  }) : assert(titles.length == forms.length, "Stepper title and forms count not matched");
}

class _StepperPageRFState extends State<StepperPageRF> {
  int tag = 0;
  ScrollController cntrl = new ScrollController();
  ScrollController cntrlVar = new ScrollController();

  @override
  Widget build(BuildContext context) {
    return Container(
      height: ScreenRF.height(context) - 80,
      child: Stack(
        children: [
          Column(
            children: [
              SingleChildScrollView(
                controller: cntrl,
                scrollDirection: Axis.horizontal,
                child: ChipsChoice<int>.single(
                  wrapped: true,
                  value: tag,
                  onChanged: (val) {
                    if (val < tag) {
                      Libs.modelValidationFunctions.clear();
                      tag = val;
                      setState(() {
                        cntrlVar.animateTo(cntrl.position.minScrollExtent,
                            duration: const Duration(milliseconds: 500), curve: Curves.ease);
                      });
                    }
                  },
                  choiceAvatarBuilder: (data) {
                    return data.selected
                        ? CircleAvatar(
                            backgroundColor: widget.tabActiveAvatarColor ?? Colors.white,
                            child: TextRF(
                              (data.value + 1).toString(),
                              color: Colors.black,
                            ),
                          )
                        : CircleAvatar(
                            backgroundColor: widget.tabInActiveAvatarColor ?? Colors.blueGrey,
                            child: TextRF(
                              (data.value + 1).toString(),
                              color: Colors.white,
                            ),
                          );
                  },
                  choiceItems: C2Choice.listFrom<int, String>(
                    source: widget.titles,
                    value: (i, v) => i,
                    label: (i, v) => v,
                  ),
                  choiceStyle: C2ChoiceStyle(
                      color: Colors.red,//widget.tabInActiveColor ?? Colors.white,
                      labelStyle: TextStyle(
                          color: widget.tabInActiveTextColor ?? Colors.black, fontSize: 16),
                      labelPadding: const EdgeInsets.only(left: 10, right: 10, top: 5, bottom: 5),
                      brightness: Brightness.dark,
                      margin: const EdgeInsets.all(5),
                      padding: const EdgeInsets.all(5),
                      showCheckmark: false,
                      elevation: 10,
                      pressElevation: 0,
                      borderColor: Colors.grey),
                  choiceActiveStyle: C2ChoiceStyle(
                    color: widget.tabActiveColor ?? Colors.red,
                    labelStyle:
                        TextStyle(color: widget.tabActiveTextColor ?? Colors.white, fontSize: 16),
                    brightness: Brightness.dark,
                  ),
                ),
              ),
              GapRF(10),
              Expanded(
                child: SingleChildScrollView(
                  controller: cntrlVar,
                  child: widget.forms[tag],
                ),
              ),
            ],
          ),
          (tag + 1) == widget.titles.length
              ? Positioned(
                  bottom: 15,
                  right: 15,
                  child: FloatingActionButton(
                    onPressed: () {
                      widget.onSubmit();
                    },
                    focusColor: Colors.green,
                    backgroundColor: widget.buttonColor ?? Colors.white,
                    child: widget.buttonSubmitIcon ??
                        Icon(
                          Icons.check,
                          color: widget.buttonTextColor ?? Colors.red,
                        ),
                  ),
                )
              : Positioned(
                  bottom: 15,
                  right: 15,
                  child: FloatingActionButton(
                    focusColor: Colors.green,
                    backgroundColor: widget.buttonColor ?? Colors.white,
                    child: widget.buttonNextIcon ??
                        Icon(
                          Icons.navigate_next,
                          color: widget.buttonTextColor ?? Colors.red,
                        ),
                    onPressed: () {
                      if (widget.onNext != null) {
                        if (widget.onNext()) {
                          Libs.modelValidationFunctions.clear();
                          tag++;
                          if (tag > 1) {
                            if (cntrl.position.pixels != cntrl.position.maxScrollExtent) {
                              cntrl.animateTo(
                                (cntrl.offset + 170).toDouble(),
                                curve: Curves.ease,
                                duration: const Duration(milliseconds: 500),
                              );
                            }
                          }
                          setState(() {
                            cntrlVar.animateTo(cntrl.position.minScrollExtent,
                                duration: const Duration(milliseconds: 500), curve: Curves.ease);
                          });
                        }
                        ;
                      }
                    },
                  ),
                ),
        ],
      ),
    );
  }

  getView(int tagid) {
    var cm = Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[],
    );
    for (int i = 0; i < widget.forms.length; i++) {
      cm.children.add(Visibility(visible: tagid == i, child: widget.forms[i]));
    }
    return cm;
  }
}

//endregion

//region DynamicField
class DynamicField extends StatefulWidget {
  String LebelText;
  Widget InputFiled;
  bool Visible;

  @override
  _DynamicFieldState createState() => _DynamicFieldState();

  DynamicField({this.LebelText, this.InputFiled, this.Visible});
}

class _DynamicFieldState extends State<DynamicField> {
  @override
  Widget build(BuildContext context) {
    return widget.Visible
        ? Container(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                TextRF(
                  widget.LebelText,
                  align: TextAlign.left,
                ),
                GapRF(5),
                widget.InputFiled
              ],
            ),
          )
        : Container();
  }
}

//endregion

//region VideoPlayer
class VideoPlayerRF extends StatefulWidget {
  bool fromNetwork;
  String url;
  bool fullScreen;
  Function listener;

  @override
  _VideoPlayerRFState createState() => _VideoPlayerRFState();

  VideoPlayerRF({this.fromNetwork = true, this.url, this.fullScreen, this.listener});
}

class _VideoPlayerRFState extends State<VideoPlayerRF> {
  vp.VideoPlayerController controller;

  @override
  void initState() {
    super.initState();
    controller = widget.fromNetwork
        ? vp.VideoPlayerController.network(
            widget.url,
            videoPlayerOptions: vp.VideoPlayerOptions(mixWithOthers: true),
          )
        : vp.VideoPlayerController.file(
            File(widget.url),
            videoPlayerOptions: vp.VideoPlayerOptions(mixWithOthers: true),
          );

    controller.addListener(() {
      if (widget.listener != null) widget.listener(controller);
    });

    controller.setLooping(false);
    controller.initialize();
  }

  /* Future setLandScape()async{
    await SystemChrome.setPreferredOrientations([DeviceOrientation.landscapeLeft,DeviceOrientation.landscapeRight]);
  }*/

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  void didChangeDependencies() {
    controller.play();
  }

  @override
  Widget build(BuildContext context) {
    return AspectRatio(
      aspectRatio: controller.value.aspectRatio,
      child: Stack(
        alignment: Alignment.bottomCenter,
        children: <Widget>[
          vp.VideoPlayer(controller),
          _ControlsOverlay(controller: controller),
          vp.VideoProgressIndicator(controller, allowScrubbing: true),
        ],
      ),
    );
  }
}

class _ControlsOverlay extends StatelessWidget {
  const _ControlsOverlay({Key key, this.controller}) : super(key: key);

  static const _examplePlaybackRates = [
    0.25,
    0.5,
    1.0,
    1.5,
    2.0,
    3.0,
    5.0,
    10.0,
  ];

  final vp.VideoPlayerController controller;

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        AnimatedSwitcher(
          duration: Duration(milliseconds: 50),
          reverseDuration: Duration(milliseconds: 200),
          child: controller.value.isPlaying
              ? SizedBox.shrink()
              : Container(
                  color: Colors.black26,
                  child: Center(
                    child: Icon(
                      Icons.play_arrow,
                      color: Colors.white,
                      size: 100.0,
                    ),
                  ),
                ),
        ),
        GestureDetector(
          onTap: () {
            controller.value.isPlaying ? controller.pause() : controller.play();
          },
        ),
        Align(
          alignment: Alignment.topRight,
          child: PopupMenuButton<double>(
            initialValue: controller.value.playbackSpeed,
            tooltip: 'Playback speed',
            onSelected: (speed) {
              controller.setPlaybackSpeed(speed);
            },
            itemBuilder: (context) {
              return [
                for (final speed in _examplePlaybackRates)
                  PopupMenuItem(
                    value: speed,
                    child: Text('${speed}x'),
                  )
              ];
            },
            child: Padding(
              padding: const EdgeInsets.symmetric(
                // Using less vertical padding as the text is also longer
                // horizontally, so it feels like it would need more spacing
                // horizontally (matching the aspect ratio of the video).
                vertical: 12,
                horizontal: 16,
              ),
              child: Text('${controller.value.playbackSpeed}x'),
            ),
          ),
        ),
      ],
    );
  }
}
//endregion
