import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:rapidfire/Db.dart';
import 'package:rapidfire/GlobalScopeRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:rapidfire/uiux/AlertRF.dart';
import 'package:rapidfire/GlobalRF.dart';

import '../../Api.dart';
import '../../Config.dart';
import '../../Dynamic.dart';

class GridViewRF extends StatefulWidget {
  DataSourceRF dataSource;
  int pageSize;
  final dynamic decoration;
  final EdgeInsets padding;
  final int column;
  final List<double> gridSpace;
  final bool scrollable;
  final dynamic builder;
  final bool showDataLoader;
  final bool enableSwipe;
  final Function onTap;
  final Function onItemDelete;
  final double childAspectRatio;

  @override
  _GridViewRFState createState() => _GridViewRFState();

  GridViewRF({
    this.dataSource,
    this.pageSize,
    this.decoration,
    this.padding,
    this.column = 2,
    this.scrollable = false,
    this.gridSpace = const [10, 10],
    this.builder,
    this.showDataLoader = true,
    this.enableSwipe = false,
    this.onItemDelete,
    this.onTap,
    this.childAspectRatio = 1.0,
  });
}

class _GridViewRFState extends State<GridViewRF> {
  int totalData = 0;

  var _pageSize = Libs.config.data?.pageSize ?? 50;

  Future _loadMore(pageNo) async {
    if (!widget.dataSource.isLoading) {
      widget.dataSource.updateSql(pageNo);
      await widget.dataSource.read(pageNo, _pageSize);
    }
  }

  onUpdateState(models, ds, sourceType) async {
    if (models != null) {
      totalData = await ds.getCount(type: sourceType);
      if (mounted) {
        setState(() {
          if (ds.type == SourceType.local) {
            if (models != null) for (var item in models) ds.modelList.add(item);
          } else if (ds.type == SourceType.remote) {
            if (models != null) {
              var modelRF = Models().get()[Dynamic.genericModelName(ds.modelList)] as ModelRF;
              for (var item in models) {
                var newModel = Dynamic(modelRF.model).newClass();
                Dynamic(newModel).dataMapToModel(item);
                //widget.dataSource.modelList.add(newModel);
                ds.modelList.add(newModel);
              }
            }
          }
        });
      }
      ds.isLoading = false;
    }
  }

  // no need
  onFilter() async {
    await widget.dataSource.read(0, _pageSize);
  }

  @override
  void didChangeDependencies() async {
    widget.dataSource.modelList.clear();
    widget.dataSource.updateSql(0);
    widget.dataSource.read(0, _pageSize);

    if (widget.dataSource.type == SourceType.local) {
      totalData = await widget.dataSource.getCount();
    } else if (widget.dataSource.type == SourceType.remote) {
      totalData = await widget.dataSource.getCount(type: SourceType.remote);
    }

    PageRF.pageScrollListener.add(() {
      if (mounted) {
        if (!widget.scrollable) {
          var listItemCount = widget.dataSource.modelList.length;
          if (!widget.dataSource.isLoading) {
            if (listItemCount < totalData) {
              print('Loading Data: $listItemCount / $totalData');
              _loadMore(widget.dataSource.currentPage++);
              setState(() => widget.dataSource.isLoading = true);
            }
          }
          print('bottom from grid');
        }
      }
    });

    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    widget.dataSource.onFilter = onFilter;
    widget.dataSource.onUpdateState = onUpdateState;

    print('GridView is ${widget.dataSource.isLoading}');
    return NotificationListener<ScrollNotification>(
      onNotification: (ScrollNotification scrollInfo) {
        if (widget.scrollable) {
          var listItemCount = widget.dataSource.modelList.length;
          if (!widget.dataSource.isLoading &&
              scrollInfo.metrics.pixels == scrollInfo.metrics.maxScrollExtent) {
            if (listItemCount < totalData) {
              print('Loading Data: $listItemCount / $totalData');
              _loadMore(widget.dataSource.currentPage++);
              setState(() => widget.dataSource.isLoading = true);
            }
          }
        }

        return true;
      },
      child: Column(
        children: [
          MediaQuery.removePadding(
            context: context,
            removeTop: true,
            child: Padding(
              padding: widget.padding ?? const EdgeInsets.all(0.0),
              child: GridView.builder(
                shrinkWrap: true,
                physics: widget.scrollable
                    ? const AlwaysScrollableScrollPhysics()
                    : const NeverScrollableScrollPhysics(),
                itemCount: widget.dataSource.modelList.length,
                gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                  crossAxisCount: widget.column,
                  crossAxisSpacing: widget.gridSpace[0],
                  mainAxisSpacing: widget.gridSpace[1],
                  childAspectRatio: widget.childAspectRatio,
                ),
                itemBuilder: (BuildContext context, int index) {
                  return widget.builder(widget.dataSource.modelList[index], index);
                },
              ),
            ),
          ),
          GapRF(10),
          widget.showDataLoader
              ? Visibility(
                  visible: widget.dataSource.isLoading,
                  child: Container(
                    height: 40,
                    color: Colors.white,
                    padding: EdgeInsets.only(top: 12),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Container(
                          height: 20,
                          child: SpinKitWave(
                              color: Colors.black, type: SpinKitWaveType.start, size: 30),
                        ),
                        SizedBox(width: 10),
                        TextRF('loading, wait for a moment...', size: 12, color: Colors.black),
                      ],
                    ),
                  ),
                )
              : Container(),
          GapRF(50),
        ],
      ),
    );
  }

  buildItems(context, model, index) {
    return widget.enableSwipe
        ? Dismissible(
            key: UniqueKey(),
            confirmDismiss: (direction) async {
              return await AlertRF(
                  context: context,
                  title: 'Confirm',
                  icon: Icons.error,
                  msg: 'Do you want to delete this record?',
                  confirmText: 'DELETE',
                  onConfirm: (value) {
                    if (value) {
                      var pkValue = Db().getPrimaryKeyValue(model);
                      if (widget.dataSource.type == SourceType.local) {
                        rf.db.delete(model, pkValue);
                        widget.dataSource.modelList.removeAt(index);
                      } else if (widget.dataSource.type == SourceType.remote) {
                        //rf.rdb.delete(model, pkValue.toString());
                        Api().delete(model: model);
                        widget.dataSource.modelList.removeAt(index);
                      } else if (widget.dataSource.type == SourceType.memory) {
                        widget.dataSource.modelList.removeAt(index);
                      }
                      if (widget.onItemDelete != null) widget.onItemDelete("");
                      print('deleting...$pkValue');
                    }
                  }).show();
            },
            //background: Container(color: Colors.transparent),
            child: InkWell(
              onTap: () => widget?.onTap(model, index),
              // child: widget.builder(model, index),
            ),
          )
        : InkWell(
            onTap: () => widget.onTap(model, index),
            // child: widget.builder(model, index),
          );
  }
}
