import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:rapidfire/Db.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:rapidfire/uiux/AlertRF.dart';
import 'package:rapidfire/GlobalRF.dart';

class ListViewRFX extends StatefulWidget {
  DataSourceRF dataSource;
  int pageSize;
  int bottomPadding;
  final dynamic builder;
  final Function onTap;

  @override
  _ListViewRFXState createState() => _ListViewRFXState();

  ListViewRFX({this.dataSource, this.pageSize = 20, this.bottomPadding = 25, this.builder, this.onTap});
}

class _ListViewRFXState extends State<ListViewRFX> {
  ScrollController _controller = new ScrollController();

  Future _loadMore(pageNo) async {
    if (!widget.dataSource.isLoading) {
      widget.dataSource.updateSql(pageNo);
      await widget.dataSource.read(pageNo, widget.pageSize);
    }
  }

  onFilter() {
    widget.dataSource.read(0, widget.pageSize);
  }

  @override
  void dispose() {
    //var className = PageRF.callerStack.pop();
    //PageRF.dataSourcesCallerRefreshMap.remove(className);
    super.dispose();
  }

  @override
  void initState() {
    widget.dataSource.onFilter = onFilter;
    widget.dataSource.updateSql(0);
    widget.dataSource.read(0, widget.pageSize);

    // if (PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName] == null) {
    //   PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName] = [widget.dataSource];
    // } else {
    //   Iterable found = PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName]
    //       .where((e) => e.model.toString() == widget.dataSource.model.toString());
    //   if (found == null || found.length == 0) {
    //     PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName].add(widget.dataSource);
    //   }
    // }

    super.initState();
  }

  ScrollController cc = ScrollController();

  final double expandedHight = 150.0;

  double get top {
    double res = expandedHight;
    if (cc.hasClients) {
      double offset = cc.offset;
      if (offset < (res - kToolbarHeight)) {
        res -= offset;
      } else {
        res = kToolbarHeight;
      }
    }
    return res;
  }

  @override
  Widget build(BuildContext context) {
    return CustomScrollView(
      slivers: [
        SliverAppBar(
          pinned: true,
          expandedHeight: 300,
          flexibleSpace: Text('HI'),
        ),
        SliverPadding(
          sliver: Container(
            child: SliverList(
              delegate: SliverChildBuilderDelegate(
                (BuildContext context, int index) {
                  return buildItems(context, widget.dataSource.modelList[index], index);
                },
                childCount: widget.dataSource.modelList.length,
              ),
            ),
          ),
        )
      ],
    );

    return SliverPadding(
      sliver: Container(
        child: SliverList(
          delegate: SliverChildBuilderDelegate(
            (BuildContext context, int index) {
              return buildItems(context, widget.dataSource.modelList[index], index);
            },
            childCount: widget.dataSource.modelList.length,
          ),
        ),
      ),
    );

    return NestedScrollView(
      controller: cc,
      headerSliverBuilder: (context, value) {
        return [SliverAppBar(expandedHeight: 300)];
        // return [
        //   SliverAppBar(
        //     pinned: true,
        //     expandedHeight: 300,
        //     flexibleSpace: Text('hihi'),
        //     // ListView(
        //     //   physics: const NeverScrollableScrollPhysics(),
        //     //   children: [
        //     //     // AppBar(
        //     //     //   title: Text('AfroJack'),
        //     //     //   elevation: 0.0,
        //     //     // ),
        //     //     Container(
        //     //       color: Colors.blue,
        //     //       height: 100,
        //     //       alignment: Alignment.center,
        //     //       child: RaisedButton(
        //     //         child: Text('folow'),
        //     //         onPressed: () => print('folow pressed'),
        //     //       ),
        //     //     ),
        //     //   ],
        //     // ),
        //   ),
        // ];
      },
      body: Container(
        height: 100,
        color: Colors.red,
        padding: EdgeInsets.only(bottom: 250),
        child: ListView.builder(
          physics: const NeverScrollableScrollPhysics(),
          itemCount: 280,
          itemBuilder: (BuildContext context, int index) {
            return Text(
              'text_string'.toUpperCase() + ' - ' + index.toString(),
              style: TextStyle(
                color: Colors.black,
              ),
            );
          },
        ),
      ),
    );

    cc.addListener(() {
      if (!widget.dataSource.isLoading && cc.position.pixels == cc.position.maxScrollExtent) {
        _loadMore(widget.dataSource.currentPage++);
        setState(() => widget.dataSource.isLoading = true);
        print('working...LV');
      }
    });

    return CustomScrollView(
      controller: cc,
      slivers: [
        SliverList(
          delegate: SliverChildBuilderDelegate(
            (BuildContext context, int index) {
              return buildItems(context, widget.dataSource.modelList[index], index);
            },
            childCount: widget.dataSource.modelList.length,
          ),
        ),
      ],
    );
  }

  buildItems(context, model, index) {
    var pkValue = Db().getPrimaryKeyValue(model);
    return Dismissible(
      key: UniqueKey(),
      confirmDismiss: (direction) async {
        return await AlertRF(
            context: context,
            title: 'Confirm',
            icon: Icons.error,
            msg: 'Do you want to delete this record?',
            confirmText: 'DELETE',
            onConfirm: (value) {
              if (value) {
                if (widget.dataSource.type == SourceType.local) {
                  rf.db.delete(model, pkValue);
                  widget.dataSource.modelList.removeAt(index);
                } else if (widget.dataSource.type == SourceType.remote) {
                  rf.rdb.delete(model, pkValue);
                  widget.dataSource.modelList.removeAt(index);
                } else if (widget.dataSource.type == SourceType.memory) {
                  widget.dataSource.modelList.removeAt(index);
                }
                setState(() {});
                print('deleting...$pkValue');
              }
            }).show();
      },
      background: Container(color: Colors.transparent),
      child: InkWell(
        onTap: () => widget.onTap(model, index),
        child: widget.builder(model, index),
      ),
    );
  }
}
