import 'package:flutter/material.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:smooth_page_indicator/smooth_page_indicator.dart';

class ListViewSliderRF extends StatefulWidget {
  final dynamic modelList;
  final Function builder;
  final double height;
  final EdgeInsets padding;
  final WidgetStyleRF style;
  final Widget placeholder;
  final placeholderLines;
  final bool showIndicator;
  final Color activeDotColor;
  final Color dotColor;

  const ListViewSliderRF({
    this.modelList,
    this.builder,
    this.height = 150,
    this.padding,
    this.style,
    this.placeholder,
    this.placeholderLines = 6,
    this.showIndicator = false,
    this.activeDotColor = Colors.red,
    this.dotColor = Colors.grey,
  });

  @override
  _ListViewSliderRFState createState() => _ListViewSliderRFState();
}

class _ListViewSliderRFState extends State<ListViewSliderRF> {
  final controllerPageIndicator = PageController(viewportFraction: 0.8, keepPage: true);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        SizedBox(
          height: widget.height,
          child: widget.modelList?.length == 0
              ? widget.placeholder ??
                  PlaceholderRF(
                    type: PlaceholderTypeRF.PageView,
                    backColor: widget.style?.placeholderBackColor ?? Color(0xffF3F2EE),
                    lineColor: widget.style?.placeholderLineColor ?? Color(0xffdadada),
                    lineHeight: widget.style?.placeholderLineHeight ?? 10,
                    lines: widget.placeholderLines ?? 6,
                  )
              : Padding(
                  padding: widget.padding,
                  child: Container(
                    height: widget.height,
                    child: ListView.builder(
                      scrollDirection: Axis.horizontal,
                      itemCount: widget.modelList?.length ?? 0,
                      itemBuilder: (_, index) {
                        if (widget.builder != null) {
                          return widget.builder(widget?.modelList[index]);
                        } else {
                          return Container();
                        }
                      },
                    ),
                  ),
                ),
        ),
        SizedBox(height: 10),
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Visibility(
              visible: widget.showIndicator,
              child: SmoothPageIndicator(
                controller: controllerPageIndicator,
                count: widget.modelList?.length ?? 0,
                effect: WormEffect(
                  dotHeight: 10,
                  dotWidth: 10,
                  type: WormType.thin,
                  activeDotColor: widget.activeDotColor ?? Colors.red,
                  dotColor: widget.dotColor ?? Colors.grey,
                  // strokeWidth: 5,
                ),
              ),
            ),
          ],
        ),
        SizedBox(
          height: 20,
        ),
      ],
    );
  }
}
