import 'package:flutter/material.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:smooth_page_indicator/smooth_page_indicator.dart';

class PageViewSliderRF extends StatefulWidget {
  final dynamic modelList;
  final Function builder;
  final double height;
  final WidgetStyleRF style;
  final Widget placeholder;
  final placeholderLines;
  final bool showIndicator;
  final Color activeDotColor;
  final Color dotColor;

  const PageViewSliderRF({
    this.modelList,
    this.builder,
    this.height = 150,
    this.style,
    this.placeholder,
    this.placeholderLines = 6,
    this.showIndicator = true,
    this.activeDotColor = Colors.red,
    this.dotColor = Colors.grey,
  });

  @override
  _PageViewSliderRFState createState() => _PageViewSliderRFState();
}

class _PageViewSliderRFState extends State<PageViewSliderRF> {
  final controllerPageIndicator = PageController(viewportFraction: 0.8, keepPage: true);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        SizedBox(
          height: widget.height,
          child: widget.modelList?.length == 0
              ? widget.placeholder ??
                  PlaceholderRF(
                    type: PlaceholderTypeRF.PageView,
                    backColor: widget.style?.placeholderBackColor ?? Color(0xffF3F2EE),
                    lineColor: widget.style?.placeholderLineColor ?? Color(0xffdadada),
                    lineHeight: widget.style?.placeholderLineHeight ?? 10,
                    lines: widget.placeholderLines ?? 6,
                  )
              : PageView.builder(
                  controller: controllerPageIndicator,
                  itemCount: widget.modelList?.length ?? 0,
                  itemBuilder: (_, index) {
                    if (widget.builder != null) {
                      return widget.builder(widget?.modelList[index]);
                    } else {
                      return Container();
                    }
                  },
                ),
        ),
        SizedBox(height: 10),
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Visibility(
              visible: widget.showIndicator,
              child: SmoothPageIndicator(
                controller: controllerPageIndicator,
                count: widget.modelList?.length ?? 0,
                effect: WormEffect(
                  dotHeight: 10,
                  dotWidth: 10,
                  type: WormType.thin,
                  activeDotColor: widget.activeDotColor ?? Colors.red,
                  dotColor: widget.dotColor ?? Colors.grey,
                  // strokeWidth: 5,
                ),
              ),
            ),
          ],
        ),
        SizedBox(
          height: 20,
        ),
      ],
    );
  }
}
