import 'package:flutter/material.dart';
import 'package:rapidfire/Dynamic.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:share_plus/share_plus.dart';

class SocialRF {
  static Widget buttonAll({
    dynamic likeModel,
    String likeField,
    dynamic savedItemModel,
    String shareSubject,
    String shareText,
    String shareUrl,
    List<String> shareFiles,
    WidgetStyleRF style,
  }) {
    //style = style ?? WidgetStyleRF(padding: EdgeInsets.all(8.0));
    return Row(
      children: [
        SocialRF.buttonLike(model: likeModel, likeField: likeField, style: style),
        GapRF(10),
        SocialRF.buttonSave(savedShareModel: savedItemModel, style: style),
        GapRF(10),
        SocialRF.buttonShare(
            subject: shareSubject,
            text: shareText,
            url: shareUrl,
            //files:
            style: style),
      ],
    );
  }

  static Widget buttonLike({
    dynamic model,
    String likeField,
    Function builder,
    Function onTap,
    WidgetStyleRF style,
  }) {
    return LikeRF(
      model: model,
      fieldName: likeField,
      builder: builder,
      onTap: onTap,
      style: style,
    );
  }

  static Widget buttonSave({
    dynamic savedShareModel,
    Function builder,
    Function onTap,
    WidgetStyleRF style,
  }) {
    return SaveRF(model: savedShareModel, builder: builder, onTap: onTap, style: style);
  }

  static Widget buttonShare({
    String subject,
    String text,
    String url,
    List files,
    Function builder,
    Function onTap,
    WidgetStyleRF style,
  }) {
    return ShareRF(
      subject: subject,
      text: text,
      url: url,
      attachment: files,
      builder: builder,
      onTap: onTap,
      style: style,
    );
  }

  static Widget infoAll({
    bool all,
    int watchCount,
    int shareCount,
    double ratingCount,
    bool showWatch,
    bool showShare,
    bool showRating,
    Icon iconWatch,
    Icon iconShare,
    Icon iconRating,
    Function builder,
    Function onTap,
  }) {
    return Row(
      children: [
        InfoWatchRF(
            count: watchCount, icon: iconWatch, builder: builder, onTap: onTap, show: showWatch),
        GapRF(10),
        InfoShareRF(
            count: shareCount, icon: iconShare, builder: builder, onTap: onTap, show: showShare),
        GapRF(10),
        InfoRatingRF(
          rating: ratingCount,
          icon: iconRating,
          builder: builder,
          onTap: onTap,
          show: showRating,
        ),
      ],
    );
  }

  static Widget infoWatch({int count, int watchCount = 0, Function builder, Function onTap}) {
    return InfoWatchRF(count: watchCount);
  }

  static Widget infoShare({bool share = true, Function builder, Function onTap}) {
    return InfoShareRF();
  }

  static Widget infoRating({bool rating = true, Function builder, Function onTap}) {
    return InfoRatingRF();
  }
}

class LikeRF extends StatefulWidget {
  final dynamic model;
  final String fieldName;
  final Function builder;
  final Function onTap;
  final WidgetStyleRF style;

  const LikeRF({
    this.model,
    this.fieldName = '',
    this.builder,
    this.onTap,
    this.style,
  });

  @override
  _LikeRFState createState() => _LikeRFState();
}

class _LikeRFState extends State<LikeRF> {
  dynamic value;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () async {
        if (widget.model != null) {
          value = Dynamic(widget.model).get(widget.fieldName);
          value = (value == null || value == 0) ? 1 : 0;
          Dynamic(widget.model).set(widget.fieldName, value);
          var ret = await rf.api.save(model: widget.model);
          if (ret.Success) value = (value == 0) ? 1 : 0;
          setState(() {});
          if (widget.onTap != null) widget.onTap();
        }
      },
      child: widget.builder != null ? widget.builder() : likeUI(),
    );
  }

  likeUI() {
    return Container(
      decoration: BoxDecoration(
        color: Colors.white,
        boxShadow: widget.style?.shadow ??
            <BoxShadow>[
              BoxShadow(color: Colors.black54, blurRadius: 2.0, offset: Offset(0.0, 0.50))
            ],
        border: widget.style?.border,
        borderRadius: BorderRadius.all(Radius.circular(widget.style?.borderRadius ?? 50)),
      ),
      child: Padding(
        padding: widget.style?.padding ?? EdgeInsets.all(8.0),
        child: (value == null || value == 0)
            ? Icon(
                widget.style?.icons ?? Icons.favorite,
                color: widget.style?.iconColor ?? Colors.black54,
              )
            : Icon(
                widget.style?.iconsAlt ?? Icons.favorite,
                color: widget.style?.iconColorAlt ?? Colors.red,
              ),
      ),
    );
  }
}

class SaveRF extends StatefulWidget {
  final dynamic model;
  final Function builder;
  final Function onTap;
  final WidgetStyleRF style;

  const SaveRF({this.model, this.builder, this.onTap, this.style});

  @override
  _SaveRFState createState() => _SaveRFState();
}

class _SaveRFState extends State<SaveRF> {
  dynamic value = 0;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () async {
        if (widget.model != null) {
          var ret = await rf.api.save(model: widget.model);
          if (ret.Success) value = 1;
          setState(() {});
          if (widget.onTap != null) widget.onTap();
        }
      },
      child: widget.builder != null ? widget.builder() : saveUI(),
    );
  }

  saveUI() {
    return Container(
      decoration: BoxDecoration(
        color: Colors.white,
        boxShadow: widget.style?.shadow ??
            <BoxShadow>[
              BoxShadow(color: Colors.black54, blurRadius: 2.0, offset: Offset(0.0, 0.50))
            ],
        border: widget.style?.border,
        borderRadius: BorderRadius.all(Radius.circular(widget.style?.borderRadius ?? 50)),
      ),
      child: Padding(
        padding: widget.style?.padding ?? EdgeInsets.all(8.0),
        child: (value == null || value == 0)
            ? Icon(
                widget.style?.icons ?? Icons.bookmark,
                color: widget.style?.iconColor ?? Colors.black54,
              )
            : Icon(
                widget.style?.iconsAlt ?? Icons.bookmark,
                color: widget.style?.iconColorAlt ?? Colors.red,
              ),
      ),
    );
  }
}

class ShareRF extends StatefulWidget {
  final String subject;
  final String text;
  final String url;
  final List<String> attachment;
  final Function builder;
  final Function onTap;
  final Rect sharePositionOrigin;
  final List<String> mimeType;
  final WidgetStyleRF style;

  const ShareRF({
    this.subject,
    this.text,
    this.url,
    this.attachment,
    this.builder,
    this.onTap,
    this.mimeType,
    this.sharePositionOrigin,
    this.style,
  });

  @override
  _ShareRFState createState() => _ShareRFState();
}

class _ShareRFState extends State<ShareRF> {
  dynamic value = 0;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () {
        if (widget.attachment == null) {
          Share.share('shareText', subject: 'subject');
        } else {
          Share.shareFiles([], subject: 'subject', text: 'shareText');
        }
        setState(() {
          // value = Dynamic(widget.model).get(widget.fieldName);
          // value = (value == null || value == 0) ? 1 : 0;
          // Dynamic(widget.model).set(widget.fieldName, value);
          //rf.api.save(model: widget.model);
        });
        if (widget.onTap != null) widget.onTap();
      },
      child: widget.builder != null ? widget.builder() : shareUI(),
    );
  }

  shareUI() {
    return Container(
      decoration: BoxDecoration(
        color: Colors.white,
        boxShadow: widget.style?.shadow ??
            <BoxShadow>[
              BoxShadow(color: Colors.black54, blurRadius: 2.0, offset: Offset(0.0, 0.50))
            ],
        border: widget.style?.border,
        borderRadius: BorderRadius.all(Radius.circular(widget.style?.borderRadius ?? 50)),
      ),
      child: Padding(
        padding: widget.style?.padding ?? EdgeInsets.all(8.0),
        child: Icon(
          widget.style?.icons ?? Icons.share_outlined,
          color: widget.style?.iconColor ?? Colors.black54,
        ),
      ),
    );
  }
}

class InfoWatchRF extends StatefulWidget {
  final Icon icon;
  final int count;
  final bool show;
  final Function builder;
  final Function onTap;

  const InfoWatchRF({key, this.icon, this.count = 0, this.show = true, this.builder, this.onTap})
      : super(key: key);

  @override
  _InfoWatchRFState createState() => _InfoWatchRFState();
}

class _InfoWatchRFState extends State<InfoWatchRF> {
  @override
  Widget build(BuildContext context) {
    return widget.builder != null ? widget.builder() : showWatchUI(widget.count);
  }

  showWatchUI(count) {
    return widget.show == null || widget.show
        ? Row(
            crossAxisAlignment: CrossAxisAlignment.end,
            children: [
              InkWell(
                  onTap: () {
                    if (widget.onTap != null) widget.onTap();
                  },
                  child: widget.icon ?? Icon(Icons.person, color: Colors.grey, size: 20)),
              Text(
                count.toString(),
                textAlign: TextAlign.end,
                style:
                    TextStyle(fontSize: 12, fontWeight: FontWeight.normal, color: Colors.blueGrey),
              ),
            ],
          )
        : Container();
  }
}

class InfoShareRF extends StatefulWidget {
  final Icon icon;
  final int count;
  final bool show;
  final Function builder;
  final Function onTap;

  const InfoShareRF({key, this.icon, this.count = 0, this.show = true, this.builder, this.onTap})
      : super(key: key);

  @override
  _InfoShareRFState createState() => _InfoShareRFState();
}

class _InfoShareRFState extends State<InfoShareRF> {
  @override
  Widget build(BuildContext context) {
    return widget.builder != null ? widget.builder() : showShareUI(widget.count);
  }

  showShareUI(count) {
    return widget.show == null || widget.show
        ? Row(
            crossAxisAlignment: CrossAxisAlignment.end,
            children: [
              InkWell(
                  onTap: () {
                    if (widget.onTap != null) widget.onTap();
                  },
                  child: Padding(
                    padding: const EdgeInsets.only(top: 4.0),
                    child: widget.icon ?? Icon(Icons.share_outlined, color: Colors.grey, size: 16),
                  )),
              GapRF(5),
              Text(
                count.toString(),
                textAlign: TextAlign.end,
                style:
                    TextStyle(fontSize: 12, fontWeight: FontWeight.normal, color: Colors.blueGrey),
              ),
            ],
          )
        : Container();
  }
}

class InfoRatingRF extends StatefulWidget {
  final Icon icon;
  final double rating;
  final String info;
  final bool show;
  final Function builder;
  final Function onTap;

  const InfoRatingRF(
      {key, this.icon, this.rating = 0.0, this.info, this.show = true, this.builder, this.onTap})
      : super(key: key);

  @override
  _InfoRatingRFState createState() => _InfoRatingRFState();
}

class _InfoRatingRFState extends State<InfoRatingRF> {
  @override
  Widget build(BuildContext context) {
    return widget.builder != null ? widget.builder() : showRatingUI(widget.rating, widget.info);
  }

  showRatingUI(count, info) {
    return widget.show == null || widget.show
        ? Row(
            crossAxisAlignment: CrossAxisAlignment.end,
            children: [
              InkWell(
                  onTap: () {
                    if (widget.onTap != null) widget.onTap();
                  },
                  child: Padding(
                    padding: const EdgeInsets.only(top: 4.0),
                    child: widget.icon ?? Icon(Icons.star, color: Colors.orangeAccent, size: 18),
                  )),
              GapRF(5),
              Text(
                count.toString(),
                textAlign: TextAlign.end,
                style:
                    TextStyle(fontSize: 12, fontWeight: FontWeight.normal, color: Colors.blueGrey),
              ),
              // GapRF(3),
              // Text(
              //   'info',
              //   style: TextStyle(color: Colors.black26, fontWeight: FontWeight.bold),
              // ),
            ],
          )
        : Container();
  }
}
