import 'package:flutter/material.dart';
import 'package:rapidfire/models/SynFetchModels.dart';

import '../../RapidFire.dart';

enum ApiType { sync, fetch }

class SyncFetchRF extends StatefulWidget {
  String jobName;
  Function dispatcher;
  ApiType apiType;
  final Function builder;
  final Function onComplete;
  @override
  _SyncFetchRFState createState() => _SyncFetchRFState();

  SyncFetchRF({this.apiType = ApiType.fetch, this.jobName, this.dispatcher, this.builder, this.onComplete});
}

class _SyncFetchRFState extends State<SyncFetchRF> {
  List<SyncableItem> syncableItems = [];
  List<SyncableItem> fetchableItems = [];
  double _newValue = 0;
  //region colros
  List<Color> colors = [
    Color(0xff71b4fb),
    Color(0xfffa8c73),
    Color(0xff8873f4),
    Color(0xff02c39a),
    Color(0xff70a288),
    Color(0xfff15bb5),
    Color(0xffedb183),
    Color(0xffa5668b),
    Color(0xff8873f4),
    Color(0xff71b8fb),
    Color(0xfffa8c73),
    Color(0xff8873f4),
    Color(0xff71b4fb),
    Color(0xfffa8c73),
    Color(0xff8873f4),
    Color(0xff71b4fb),
    Color(0xfffa8c73),
    Color(0xff8873f4),
    Color(0xff02c39a),
    Color(0xff70a288),
    Color(0xfff15bb5),
    Color(0xffedb183),
    Color(0xffa5668b),
    Color(0xff8873f4),
    Color(0xff71b4fb),
    Color(0xfffa8c73),
    Color(0xff8873f4),
    Color(0xff71b4fb),
    Color(0xfffa8c73),
    Color(0xff8873f4),
  ];
  //endregion

  @override
  void initState() {
    Api.onSyncCompleted = () {
      setState(() {});
    };
    Api.onFetchCompleted = (model, count) {
      if (widget.onComplete != null) widget.onComplete(model);
      if (model.toString() == "all fetch completed") setState(() {});
    };
  }

  Future<List<SyncableItem>> dataSyncQuery() async {
    Models models = Models();
    List<SyncableItem> Items = [];
    for (var item in models.get().entries) {
      var modelRF = item.value as ModelRF;
      if (modelRF.syncable) {
        var sql =
            'select \'${modelRF.model.runtimeType.toString()}\' as ItemName,\'${modelRF.displayName}\' as DisplayName, count(*) as SyncableCount from ${item.key} where ${modelRF.syncStatusField} = 0';
        await rf.db.get(SyncableItem, sql, null, (model) {
          Items.add(model.first);
        });
      }
    }
    return Items;
  }

  Future<List<SyncableItem>> dataFetchQuery() async {
    Models models = Models();
    List<SyncableItem> Items = [];
    for (var item in models.get().entries) {
      var modelRF = item.value as ModelRF;
      if (modelRF.syncable) {
        var sql = 'select \'${modelRF.model.runtimeType.toString()}\' as ItemName,\'${modelRF.displayName}\' as DisplayName, count(*) as SyncableCount from ${item.key}';
        await rf.db.get(SyncableItem, sql, null, (model) {
          Items.add(model.first);
        });
      }
    }
    return Items;
  }

  @override
  Widget build(BuildContext context) {
    print("Building ui");
    return Container(
      child: FutureBuilder<List<SyncableItem>>(
        future: widget.apiType == ApiType.fetch ? dataFetchQuery() : dataSyncQuery(),
        builder: (context, AsyncSnapshot<List<SyncableItem>> snapshot) {
          if (!snapshot.hasData) {
            return Container();
          } else {
            syncableItems.clear();
            syncableItems.addAll(snapshot.data);
            return GridView.builder(
                shrinkWrap: true,
                itemCount: syncableItems.length,
                gridDelegate: new SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 3),
                itemBuilder: (BuildContext context, int index) {
                  return TweenAnimationBuilder(
                    tween: Tween<double>(begin: _newValue, end: 1),
                    curve: Curves.elasticOut,
                    duration: Duration(milliseconds: 400 * index),
                    builder: (BuildContext context, double size, Widget child) {
                      return Transform.scale(
                        scale: size,
                        child: InkWell(
                          onTap: () {
                            if (widget.apiType == ApiType.sync) {
                              rf.api.syncModels(widget.jobName, context, widget.dispatcher, [rf.models[syncableItems[index].ItemName].model.runtimeType]);
                            } else {
                              rf.api.fetchModel(context: context, serviceName: widget.jobName, dispatcher: widget.dispatcher, modelsName: syncableItems[index].ItemName);
                            }
                          },
                          child: widget.builder != null
                              ? widget.builder(syncableItems[index], index)
                              : _syncCard(syncableItems[index].DisplayName, syncableItems[index].SyncableCount.toString(), color: colors[index], lightColor: Colors.white.withAlpha(40)),
                        ),
                      );
                    },
                  );
                });
          }
        },
      ),
      /*child: ,*/
    );
  }

  Widget _syncCard(String title, String count, {Color color, Color lightColor}) {
    return Container(
      margin: EdgeInsets.only(left: 15, right: 15, bottom: 20, top: 10),
      decoration: BoxDecoration(
        color: color,
        borderRadius: BorderRadius.all(Radius.circular(20)),
        boxShadow: <BoxShadow>[
          BoxShadow(
            offset: Offset(4, 4),
            blurRadius: 10,
            color: color.withOpacity(.5),
          )
        ],
      ),
      child: ClipRRect(
        borderRadius: BorderRadius.all(Radius.circular(20)),
        child: Container(
          child: Stack(
            children: <Widget>[
              Positioned(
                top: -20,
                left: -20,
                child: CircleAvatar(
                  backgroundColor: lightColor,
                  radius: 40,
                  child: Center(
                    child: TextRF(
                      count,
                      style: TextStyle(fontWeight: FontWeight.bold, color: Colors.white, fontSize: 20),
                    ),
                  ),
                ),
              ),
              Column(
                mainAxisAlignment: MainAxisAlignment.end,
                children: <Widget>[
                  Flexible(
                    child: Align(
                      alignment: Alignment.bottomCenter,
                      child: TextRF(
                        title,
                        align: TextAlign.center,
                        size: 18.0,
                        style: TextStyle(fontWeight: FontWeight.bold, color: Colors.black.withOpacity(0.4)),
                      ),
                    ),
                  ),
                  SizedBox(height: 10),
                ],
              )
            ],
          ),
        ),
      ),
    );
  }
}

class SyncAllButtonRF extends StatefulWidget {
  String jobName;
  Function dispatcher;

  @override
  _SyncAllButtonRFState createState() => _SyncAllButtonRFState();

  SyncAllButtonRF({this.jobName, this.dispatcher});
}

class _SyncAllButtonRFState extends State<SyncAllButtonRF> {
  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () {
        print("top sync button");
        rf.api.syncModels(
          widget.jobName,
          context,
          widget.dispatcher,
        );
      },
      child: Container(
        margin: EdgeInsets.only(right: 10, top: 5),
        width: 105,
        child: Stack(
          children: [
            Positioned(
              child: Container(
                height: 35,
                margin: EdgeInsets.only(left: 13, top: 0),
                decoration: BoxDecoration(
                  color: Colors.blueGrey,
                  borderRadius: BorderRadius.all(Radius.circular(50)),
                ),
                child: Center(
                  child: Row(
                    children: [
                      Container(
                        height: 30,
                        width: 30,
                        margin: EdgeInsets.all(3),
                        decoration: BoxDecoration(
                          color: Colors.white,
                          borderRadius: BorderRadius.all(Radius.circular(100)),
                          boxShadow: <BoxShadow>[BoxShadow(offset: Offset(0, 0), blurRadius: 4, color: Colors.black.withOpacity(.4))],
                        ),
                        child: Center(
                          child: Icon(
                            Icons.arrow_upward,
                            color: Colors.blueGrey,
                          ),
                        ),
                      ),
                      SizedBox(width: 0),
                      Center(child: TextRF("Sync All", style: TextStyle(color: Colors.white, fontSize: 12))),
                    ],
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class FetchAllButtonRF extends StatefulWidget {
  String jobName;
  Function dispatcher;
  @override
  _FetchAllButtonRFState createState() => _FetchAllButtonRFState();

  FetchAllButtonRF({this.jobName, this.dispatcher});
}

class _FetchAllButtonRFState extends State<FetchAllButtonRF> {
  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () {
        print("fetch button clicked");
        rf.api.fetchModel(context: context, serviceName: widget.jobName, dispatcher: widget.dispatcher);
      },
      child: Container(
        margin: EdgeInsets.only(right: 10, top: 5),
        width: 105,
        child: Stack(
          children: [
            Positioned(
              child: Container(
                height: 35,
                margin: EdgeInsets.only(left: 13, top: 0),
                decoration: BoxDecoration(
                  color: Colors.blueGrey,
                  borderRadius: BorderRadius.all(Radius.circular(50)),
                ),
                child: Center(
                  child: Row(
                    children: [
                      Container(
                        height: 30,
                        width: 30,
                        margin: EdgeInsets.all(3),
                        decoration: BoxDecoration(
                          color: Colors.white,
                          borderRadius: BorderRadius.all(Radius.circular(100)),
                          boxShadow: <BoxShadow>[BoxShadow(offset: Offset(0, 0), blurRadius: 4, color: Colors.black.withOpacity(.4))],
                        ),
                        child: Center(
                          child: Icon(
                            Icons.arrow_downward,
                            color: Colors.blueGrey,
                          ),
                        ),
                      ),
                      SizedBox(width: 0),
                      Center(child: TextRF("Fetch All", style: TextStyle(color: Colors.white, fontSize: 12))),
                    ],
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
    ;
  }
}
